package logrus;

import jade.core.Agent;
import jade.core.Profile;
import jade.core.ProfileImpl;
import jade.core.Runtime;
import jade.wrapper.AgentController;
import jade.wrapper.ContainerController;
import jade.wrapper.StaleProxyException;

/**
 * Wrapper for JADE platform, this class is a singleton
 */
public class AgentsPlatform
{
	// external JADE controller
	private Runtime runtime;

	// main platform container
	private ContainerController mainContainer;

	// singleton instance
	private static AgentsPlatform instance;

	/**
	 *  
	 */
	private AgentsPlatform ()
	{
		super();
	}

	/**
	 * Get singleton instance od AgentsPlatform
	 * 
	 * @return Singleton instance od AgentsPlatform
	 */
	public static AgentsPlatform getInstance ()
	{
		if (AgentsPlatform.instance == null)
		{
			AgentsPlatform.instance = new AgentsPlatform();
		}
		return AgentsPlatform.instance;
	}

	/**
	 * Initialization of JADE enviroment
	 * 
	 * @throws StaleProxyException
	 */
	private void InitJADE ()
	{
		// get instance of Runtime
		this.runtime = Runtime.instance();

		// exit the JVM when there are no more containers around
		this.runtime.setCloseVM(true);

		// launch a complete platform on the 8888 port, create a default Profile
		Profile profMain = new ProfileImpl(null,8888,null);
		this.mainContainer = this.runtime.createMainContainer(profMain);

		// set now the default Profile to start a container
		// Profile profAgent = new ProfileImpl(null,8888,null);
		// this.agentsContainer = this.runtime.createAgentContainer(profAgent);
	}

	/**
	 * Initializes agents
	 * 
	 * @throws StaleProxyException
	 */
	private void InitAgents (Object obj) throws StaleProxyException
	{
		/*
		 * AgentController mainDialogAgent = this.mainContainer
		 * .createNewAgent(AgentNames.MainGUIAgent,MainDialogAgent.class.getName(),null);
		 * mainDialogAgent.start();
		 */
		/*
		 * // run main controller agent AgentController mainControllerAgent =
		 * this.mainContainer.createNewAgent( "MainController",
		 * "logrus.agents.mainControllerAgent.MainControllerAgent",null); mainControllerAgent.start(); //
		 * run query agent AgentController queryAgent = this.mainContainer.createNewAgent(
		 * "QueryAgent","logrus.agents.queryAgent.QueryAgent",null); queryAgent.start();
		 */
	}

	/**
	 * Run JADE container and OntoStorage agents
	 * 
	 * @throws StaleProxyException
	 */
	public void runPlatform (Object obj) throws StaleProxyException
	{
		this.InitJADE();
		this.InitAgents(obj);
	}

	/**
	 * Shutdown OntoStorage agents and whole platform
	 * 
	 * @throws StaleProxyException
	 */
	public void shutdownPlatform () throws StaleProxyException
	{
		// shutdown container
		this.mainContainer.kill();
	}

	/**
	 * Adds an agent to this container.
	 * 
	 * @param agent
	 *           Agent to be added, should be instantiated and configured.
	 * @return An <code>AgentController</code>, allowing to call state-transition forcing methods
	 *         on the real agent instance.
	 * @throws StaleProxyException
	 */
	public AgentController addAgent (Agent agent, String name)
			throws StaleProxyException
	{
		AgentController controller = this.mainContainer
				.acceptNewAgent(name,agent);
		controller.start();
		return controller;
	}
}
