package logrus.agents;

import jade.content.Concept;
import jade.content.ContentElement;
import jade.util.leap.ArrayList;
import jade.util.leap.Serializable;
import logrus.agents.searchCoordinatorAgent.SearchTask;
import logrus.ontmodel.Quad;
import logrus.ontmodel.QuadsList;

/**
 * 
 */
public class OSMessage implements ContentElement, Concept, Serializable
{
	public static final int MA_Boot_OK = 101;
	public static final int MA_Boot_Failed = 102;
	public static final int MA_Crawl_Request = 110;
	public static final int MA_Crawl_Request_OK = 111;
	public static final int MA_Crawl_Request_Failed = 112;
	
	public static final int DBA_Boot_OK = 301;
	public static final int DBA_Boot_Failed = 302;
	public static final int DBA_Inject_Data = 310;
	public static final int DBA_Inject_Data_OK = 311;
	public static final int DBA_Inject_Data_Failed = 312;
	public static final int DBA_Extract_Object_Info = 320;
	public static final int DBA_Extract_Object_Result = 321;
	public static final int DBA_Extract_Object_Failed = 322;
	public static final int DBA_Find_Matching = 330;
	public static final int DBA_Find_Matching_Result = 331;
	public static final int DBA_Find_Matching_Failed = 332;	
	public static final int DBA_Find_Matching_NotFound = 333;
	public static final int DBA_Execute_RDQL = 340;
	public static final int DBA_Execute_RDQL_Result = 341;
	public static final int DBA_Execute_RDQL_Failed = 342;
	public static final int DBA_Save_Model = 350;
	public static final int DBA_Save_Model_OK = 351;
	public static final int DBA_Save_Model_Failed = 352;
	public static final int DBA_Build_Report = 360;
	public static final int DBA_Build_Report_OK = 361;
	public static final int DBA_Build_Report_Failed = 362;
	public static final int DBA_Find_Outdated = 370;	
	public static final int DBA_Find_Outdated_Result = 371;
	public static final int DBA_Find_Outdated_Failed = 372;	
	
	public static final int IA_Boot_OK = 401;
	public static final int IA_Boot_Failed = 402;
	public static final int IA_Mass_Index = 410;
	public static final int IA_Mass_Index_OK = 411;
	public static final int IA_Mass_Index_Failed = 412;
	public static final int IA_Indexed_OK = 421;
	public static final int IA_Indexed_Failed = 422;
	
	public static final int DSA_Boot_OK = 501;
	public static final int DSA_Boot_Failed = 502;
	public static final int DSA_Update = 510;
	public static final int DSA_Update_OK = 511;
	public static final int DSA_Update_Failed = 512;	
	
	public static final int SCA_Boot_OK = 600;
	public static final int SCA_Boot_Failed = 601;		
	public static final int SCA_Extract_Object_Info = 610;
	public static final int SCA_Extract_Object_Info_Result = 611;
	public static final int SCA_Extract_Object_Info_Failed = 612;
	public static final int SCA_Crawl = 620;	
	public static final int SCA_Crawl_Result = 621;	
	public static final int SCA_Crawl_Failed = 622;
	
	public static final int WA_Boot_OK = 701;
	public static final int WA_Boot_Failed = 702;	
	public static final int WA_Extract_Object_Info = 710;
	public static final int WA_Extract_Object_Info_Result = 711;
	public static final int WA_Extract_Object_Info_Failed = 712;
	public static final int WA_Crawl = 720;	
	public static final int WA_Crawl_Result = 721;	
	public static final int WA_Crawl_Failed = 722;
	

	// message code
	private int messageCode = -1;

	// textual announcement
	private String announcement = null;

	// quad describing RDF statement + provider
	private Quad quadStatement = null;
	
	// used when there is need to transmit multiple quads at once
	private QuadsList multipleQuads = null;
	
	// used when search command is issued
	private SearchTask searchCommand = null;
	
	// used when a group of URIs is transfered
	private ArrayList uris = null;
	
	// list of object infos
	private ArrayList objectsCollection = null;
	
	
	public QuadsList getObjectInfo ()
	{
		return this.multipleQuads;
	}
	
	public void setObjectInfo (QuadsList objInfo)
	{
		this.multipleQuads = objInfo;
	}	

	public boolean hasAnnouncement ()
	{
		return this.announcement!=null;	
	}
	
	public boolean hasSingleQuad ()
	{
		return this.quadStatement!=null;
	}
	
	public boolean hasObjectInfo ()
	{
		return this.multipleQuads!=null;
	}		
	
	public String toString ()
	{
		StringBuffer buffer = new StringBuffer();
		buffer.append("Message code: ");
		buffer.append(this.messageCode);
		buffer.append('\n');
		if (this.hasAnnouncement())
		{
			buffer.append("Announcement: ");
			buffer.append(this.announcement);
			buffer.append('\n');
		}
		if (this.hasSingleQuad())
		{
			buffer.append("Quad statement: ");
			buffer.append(this.quadStatement);
			buffer.append('\n');
		}
		if (this.hasObjectInfo())
		{
			buffer.append("Quads collection:\n");
			buffer.append(this.multipleQuads);
		}
		return buffer.toString();
	}

	/**
	 * 
	 */
	public OSMessage ()
	{
		super();
	}

	public OSMessage (int messageCode)
	{
		super();
		this.setMessageCode(messageCode);
	}

	public OSMessage (int messageCode, String announcement)
	{
		super();
		this.setMessageCode(messageCode);
		this.setAnnouncement(announcement);
	}
	
	public OSMessage (int messageCode, String announcement, Quad quad)
	{
		super();
		this.setMessageCode(messageCode);
		this.setAnnouncement(announcement);
		this.setQuadStatement(quad);
	}
	
	public OSMessage (int messageCode, String announcement, QuadsList quads)
	{
		super();
		this.setMessageCode(messageCode);
		this.setAnnouncement(announcement);
		this.setMultipleQuads(quads);
	}		

	/**
	 * @param msgCode
	 *           The msgCode to set.
	 */
	public void setMessageCode (int messageCode)
	{
		this.messageCode = messageCode;
	}

	/**
	 * @return Returns the msgCode.
	 */
	public int getMessageCode ()
	{
		return this.messageCode;
	}

	/**
	 * @param announcement
	 *           The announcement to set.
	 */
	public void setAnnouncement (String announcement)
	{
		this.announcement = announcement;
	}

	/**
	 * @return Returns the announcement.
	 */
	public String getAnnouncement ()
	{
		return this.announcement;
	}

	/**
	 * @param quadStatement
	 *           The quadStatement to set.
	 */
	public void setQuadStatement (Quad quadStatement)
	{
		this.quadStatement = quadStatement;
	}

	/**
	 * @return Returns the quadStatement.
	 */
	public Quad getQuadStatement ()
	{
		return this.quadStatement;
	}

	/**
	 * @param multipleQuads The multipleQuads to set.
	 */
	public void setMultipleQuads (QuadsList multipleQuads)
	{
		this.multipleQuads = multipleQuads;
	}

	/**
	 * @return Returns the multipleQuads.
	 */
	public QuadsList getMultipleQuads ()
	{
		return this.multipleQuads;
	}

	/**
	 * @param searchCommand The searchCommand to set.
	 */
	public void setSearchCommand (SearchTask searchCommand)
	{
		this.searchCommand = searchCommand;
	}

	/**
	 * @return Returns the searchCommand.
	 */
	public SearchTask getSearchCommand ()
	{
		return this.searchCommand;
	}

	/**
	 * @param uris The uris to set.
	 */
	public void setUris (ArrayList uris)
	{
		this.uris = uris;
	}

	/**
	 * @return Returns the uris.
	 */
	public ArrayList getUris ()
	{
		return this.uris;
	}

	/**
	 * @param objectsCollection The objectsCollection to set.
	 */
	public void setObjectsCollection (ArrayList objectsCollection)
	{
		this.objectsCollection = objectsCollection;
	}

	/**
	 * @return Returns the objectsCollection.
	 */
	public ArrayList getObjectsCollection ()
	{
		return this.objectsCollection;
	}
}
		