package logrus.agents;

import java.util.ArrayList;
import java.util.Iterator;

import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.ContentManager;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.UngroundedException;
import jade.content.onto.basic.Action;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

/**
 *
 */
public class OSMessageMatcher implements MessageTemplate.MatchExpression
{
	// list of accepted codes
	private ArrayList acceptedCodes = new ArrayList();

	// accepted conversation ID (used when multiple conversations of the same type are performed by
	// agent in parallel)
	private String acceptedConversationID;

	// content manager used for extracting contents
	private ContentManager contentManager = null;

	/**
	 * 
	 */
	public OSMessageMatcher (ContentManager manager)
	{
		super();
		this.contentManager = manager;
	}

	public static OSMessage extractMessage (ACLMessage msg,
			ContentManager manager)
	{
		try
		{
			ContentElement elem = manager.extractContent(msg);
			if (elem instanceof Action)
			{
				Concept concept = ((Action)elem).getAction();
				if (concept instanceof OSMessage)
				{
					return (OSMessage)concept;
				}
			}
			return null;
		}
		catch (UngroundedException e)
		{
			e.printStackTrace();
			return null;
		}
		catch (CodecException e)
		{
			e.printStackTrace();
			return null;
		}
		catch (OntologyException e)
		{
			e.printStackTrace();
			System.out.println(msg);
			return null;
		}
	}

	public boolean match (ACLMessage msg)
	{
		// first check if we wait for message with specific conversation ID and if so, check it's
		// value with the one extracted from ACL message
		if (this.acceptedConversationID != null)
		{
			if (this.acceptedConversationID.compareTo(msg.getConversationId())!=0)
			{
				return false;
			}
		}

		// extract content and check if we've recived OSMessage
		OSMessage osMsg = OSMessageMatcher
				.extractMessage(msg,this.contentManager);
		if (osMsg != null)
		{
			return this.checkMsgCodeMatch(osMsg);
		}
		// it's not OSMessage so rejected it
		return false;
	}

	private boolean checkMsgCodeMatch (OSMessage osMsg)
	{
		// first check if we wait for message with specified code
		if (this.acceptedCodes.size() > 0)
		{
			int code = osMsg.getMessageCode();
			for (Iterator iter = this.acceptedCodes.iterator(); iter.hasNext();)
			{
				// if code extracted from message is equal to one of the codes on list of
				// accepted codes then return true
				if (code == ((Integer)iter.next()).intValue())
				{
					return true;
				}
			}
			// we've checked all accepted codes and none has matched the code form message
			// so reject whole message
			return false;
		}
		else
		{
			// we're not waiting for any special type of message so accept it
			// whatever the code value is
			return true;
		}
	}

	/**
	 * @param patternMsgCode
	 *           The patternMsgCode to set.
	 */
	public void addAcceptedMsgCode (int patternMsgCode)
	{
		this.acceptedCodes.add(new Integer(patternMsgCode));
	}

	/**
	 * @param acceptedConversationID The acceptedConversationID to set.
	 */
	public void setAcceptedConversationID (String acceptedConversationID)
	{
		this.acceptedConversationID = acceptedConversationID;
	}

	/**
	 * @return Returns the acceptedConversationID.
	 */
	public String getAcceptedConversationID ()
	{
		return this.acceptedConversationID;
	}
}
