package logrus.agents.base;


import jade.core.AID;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.lang.acl.ACLMessage;


/**
 * Baseclass for ACL subscription notification message handlers
 */
public abstract class SubscriptionNotificationHandler
{
	/**
	 * Extracts AIDs of agents from ACL subscription notification message
	 * @param message Message from which AIDs should be extracted
	 * @return Array of AIDS from ACL subscription notification message
	 * or null if there where no agent AIDs
	 * @throws FIPAException
	 */
	public AID[] extractAIDs (ACLMessage message) throws FIPAException
	{
		AID[] agents = null;
		
		DFAgentDescription[] results = 
			DFService.decodeNotification(message.getContent());
		if (results.length>0) 
		{
			agents = new AID[results.length];
			for (int i=0; i<results.length; i++)
			{
				agents[i] = results[i].getName();
			}
		}
		return agents;
	}
	
	/**
	 * Handles ACL subscription notification message
	 * @param message ACL subscription notification message
	 */
	public abstract void handleNotification (ACLMessage message);
}
