package logrus.agents.databaseAgent;

import jade.content.onto.OntologyException;
import jade.core.AID;
import jade.domain.FIPAException;
import logrus.agents.OntoStorageAgent;
import logrus.agents.ServiceNames;
import logrus.agents.databaseAgent.behaviours.BootDBABehaviour;
import logrus.agents.ontology.OntoStorageOntology;
import logrus.ontmodel.IOntModelWrapper;
import logrus.tools.Utils;
import logrus.tools.logging.InitializeLoggerFailedException;

/**
 * 
 */
public class DatabaseAgent extends OntoStorageAgent
{
	// RDF file path
	private String databaseName;

	// Object wrapping JENA ontological model
	private IOntModelWrapper wrapper = null;

	// AID of manager Agent
	private AID aidManager;

	// modes of agent
	public static final int MODE_WRITE = 1;

	public static final int MODE_READ = 2;

	// mode in which this agent should be launched (read/write, only write or only read)
	private int mode;

	/**
	 * @throws InitializeLoggerFailedException
	 * @throws OntologyException
	 */
	public DatabaseAgent (String databaseName, AID aidManager, int mode,
			IOntModelWrapper wrapper)
			throws InitializeLoggerFailedException, OntologyException
	{
		super();
		this.databaseName = databaseName;
		this.aidManager = aidManager;
		this.mode = mode;
		this.wrapper = wrapper;
	}

	public void setup ()
	{
		this.addBehaviour(new BootDBABehaviour(this,this.databaseName));
		try
		{
			Utils tools = Utils.getInstance();
			if (this.mode == MODE_WRITE)
			{
				this.registerService(tools.getDBAServiceName(MODE_WRITE),
						OntoStorageOntology.ONTOLOGY_NAME);
			}
			else
			{
				this.registerService(tools.getDBAServiceName(MODE_READ),
						OntoStorageOntology.ONTOLOGY_NAME);
			}
		}
		catch (FIPAException exc)
		{
			this.logg("DBA couldn't register service",exc);
		}
	}

	public void takeDown ()
	{
		try
		{
			this.deregisterService(ServiceNames.ServiceDBA,
					OntoStorageOntology.ONTOLOGY_NAME);
		}
		catch (FIPAException exc)
		{
			this.logg("DBA couldn't deregister service",exc);
		}
	}

	/**
	 * @param wrapper
	 *           The wrapper to set.
	 */
	public void setWrapper (IOntModelWrapper wrapper)
	{
		this.wrapper = wrapper;
	}

	/**
	 * @return Returns the wrapper.
	 */
	public IOntModelWrapper getWrapper ()
	{
		return this.wrapper;
	}

	/**
	 * @param aidManager
	 *           The aidManager to set.
	 */
	public void setAidManager (AID aidManager)
	{
		this.aidManager = aidManager;
	}

	/**
	 * @return Returns the aidManager.
	 */
	public AID getAidManager ()
	{
		return this.aidManager;
	}
}
