package logrus.agents.databaseAgent.behaviours;

import jade.core.Agent;
import jade.lang.acl.ACLMessage;

import java.util.logging.Level;

import logrus.agents.OSMessage;
import logrus.agents.base.behaviours.OSOneShotBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.databaseAgent.DatabaseAgent;

/**
 * 
 */
public class BootDBABehaviour extends OSOneShotBehaviour
{
	// Database agent
	private DatabaseAgent dbAgent;

	private String databaseName;

	/**
	 * @param a
	 */
	public BootDBABehaviour (Agent a, String databaseName)
	{
		super(a);
		this.dbAgent = (DatabaseAgent)a;
		this.databaseName = databaseName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		try
		{
			this.myAgent.addBehaviour(new ServiceDBABehaviour(this.osAgent,
					this.dbAgent.getWrapper()));
	
			// send message with information about successful initialization
			OSMessage msg = new OSMessage(OSMessage.DBA_Boot_OK);
			this.dbAgent.sendMessage(ACLMessage.INFORM,msg,this.dbAgent
					.getAidManager());
			this.osAgent.logg(Level.INFO,"DBA Booted");
		}
		catch (Exception exc)
		{
			this.OnLoadDatabaseFailed(exc);
		}
	}

	private void OnLoadDatabaseFailed (Exception exc)
	{
		// log error
		if (exc != null)
		{
			exc.printStackTrace();
		}
		String errMsg = "Model load failure" + exc == null ? "" : ": "
				+ exc.toString();
		this.osAgent.logg(Level.SEVERE,errMsg);

		// try to inform manager agent about failure
		OSMessage msg = new OSMessage(OSMessage.DBA_Boot_Failed,
				exc != null ? "DBA error: " + exc.getMessage() : null);
		try
		{
			this.dbAgent.sendMessage(ACLMessage.INFORM,msg,this.dbAgent
					.getAidManager());
		}
		catch (SendingMessageFailed e)
		{
			this.osAgent.logg(Level.SEVERE,"Sending DBA failure info failed " + e);
			this.osAgent.doDelete();
		}
	}
}
