package logrus.agents.datasources.chefmoz.paragraphProcessors;

import logrus.agents.datasources.DataProviderID;
import logrus.ontmodel.QuadsList;
import logrus.tools.Utils;

/**
 * Interface used for extracting information from paragraphs
 */
public abstract class ParagraphProcessor
{
	protected String base;
	
	/**
	 * Process string extracted from text node
	 * 
	 * @param content
	 *           Content of text node
	 * @param results
	 *           Set of result statements
	 * @param uriRestaurant
	 *           URI of restaurant
	 */
	public abstract void process (String content, QuadsList results,
			String uriRestaurant);

	/**
	 * Standard content parsing plus extending result with <code>extension</code>
	 * 
	 * @param content
	 *           Content to be parsed
	 * @param results
	 *           Set of result statements
	 * @param uriRestaurant
	 *           URI of restaurant
	 * @param uriProperty
	 *           Name of used property
	 * @param toRemove
	 *           Substrings that should be removed form content
	 */
	protected void standardProcess (String content,
			QuadsList results, String uriRestaurant,
			String uriProperty, String extension, String[] toRemove)
	{
		if(content.length()==0)
		{
			return;
		}
		if (toRemove != null)
		{
			for (int j = 0; j < toRemove.length; j++)
			{
				content = content.replaceAll(toRemove[j]," ");
			}
		}
		String[] parts = content.split(" ");
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < parts.length; i++)
		{
			if (parts[i].trim().length() > 0)
			{
				buffer.append(Utils.strToUpperAt(parts[i].trim(),0));
			}
		}
		content = this.additionalProcessing(buffer.toString());
		if (extension != null)
		{
			content += extension;
		}
		content = this.base + "#" + content;
		results.add(uriRestaurant,uriProperty,content,DataProviderID.CHEFMOZ);
	}

	abstract protected String additionalProcessing (String content);
}
