/**
 * 
 */
package logrus.agents.datasources.restauracjecom;

import java.util.ArrayList;
import java.util.Iterator;

import logrus.agents.datasources.WebpageParseException;
import logrus.agents.datasources.LinkInfo;
import logrus.agents.datasources.ILinkExtactor;
import logrus.tools.ContentNotAvilableException;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.XMLAnalysisException;
import logrus.tools.queue.Queue;
import logrus.tools.web.WebUtils;

/**
 * Extracts information from restauracje.com pages
 */
public class RestComLinkExtractor implements ILinkExtactor
{
	// queue of pages to be parsed
	private Queue toProcess;

	private ArrayList restaurantLinks;

	Queue results;

	/**
	 * 
	 */
	public RestComLinkExtractor (LinkInfo[] startLinks)
	{
		super();
		this.toProcess = new Queue();
		this.results = new Queue();
		for (int i = 0; i < startLinks.length; i++)
		{
			this.enqueue(startLinks[i]);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see logrus.agents.datasources.ILinkExtactor#extractRestaurantLinks()
	 */
	public String[] extractRestaurantLinks () throws WebpageParseException
	{
		this.restaurantLinks = new ArrayList();
		while (!this.toProcess.empty())
		{
			this.parseListPage((LinkInfo)this.toProcess.pop());
		}
		if (this.restaurantLinks.size() > 0)
		{
			String[] links = new String[this.restaurantLinks.size()];
			int i = 0;
			for (Iterator iter = this.restaurantLinks.iterator(); iter.hasNext();)
			{
				links[i++] = (String)iter.next();
			}
			return links;
		}
		else
		{
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see logrus.agents.datasources.ILinkExtactor#extractRestaurantInfo()
	 */
	public String[] extractRestaurantInfo () throws WebpageParseException
	{
		while (!this.toProcess.empty())
		{
			LinkInfo info = this.dequeue();
			switch (info.type)
			{
				case RestComPageTypes.ListPage:
					this.parseListPage(info);
					break;
				case RestComPageTypes.DetailsPage:
					try
					{
						// System.out.println("Extracting "+info.link);
						RestComPropertiesExtractor extr = new RestComPropertiesExtractor();
						extr.extract(info.link);
					}
					catch (IdentifierGenerationFailedException e)
					{
						throw new WebpageParseException(e);
					}
					catch (XMLAnalysisException e)
					{
						throw new WebpageParseException(e);
					}
					break;
			}
		}
		return null;
	}

	/**
	 * Enqueue link to be processed
	 * 
	 * @param info
	 *           Link info
	 */
	void enqueue (LinkInfo info)
	{
		this.toProcess.injectUnique(info);
	}

	/**
	 * Get link to be parsed from queue
	 * 
	 * @return First link in queue to be parsed or null if all links have been processed
	 */
	private LinkInfo dequeue ()
	{
		return (LinkInfo)this.toProcess.pop();
	}

	public void parseListPage (LinkInfo info) throws WebpageParseException
	{

		try
		{
			String link = info.link;
			String page = WebUtils.getInstance().getContentByURL(link,
					"windows-1250");
			String strStart = "<TR bgcolor=#DAD9B5>";
			String strEnd = "class='archiwum'>dalej >></A>";
			int iStart = page.indexOf(strStart);
			int iEnd = page.indexOf(strEnd);
			if (iEnd <0)
			{
				iEnd = page.indexOf("<div align=\"left\" class=\"restauracje9\">");
			}
			if (iStart > 0 && iEnd > 0)
			{
				String part = page.substring(iStart,iEnd);
				String[] resParts = part.split("</tr>");
				for (int i = 0; i < resParts.length - 1; i++)
				{
					String aux = resParts[i];
					strStart = "<A HREF='";
					strEnd = "' class=\"przepish\"";
					iStart = aux.indexOf(strStart);
					iEnd = aux.indexOf(strEnd);

					// enqueue restaurant link
					// System.out.println("http://www.restauracje.com/"
					// + resParts[i].substring(iStart + strStart.length(),iEnd));
					this.restaurantLinks.add("http://www.restauracje.com/"
							+ resParts[i].substring(iStart + strStart.length(),iEnd));
					// this.enqueue(new LinkInfo("http://www.restauracje.com/"
					// + resParts[i].substring(iStart + strStart.length(),iEnd),
					// RestComPageTypes.DetailsPage));

				}
				strStart = "<TR><TD align=left></TD><TD align=right><A HREF='";
				strEnd = "' class='archiwum'>dalej >></A>";
				iStart = page.indexOf(strStart);
				if (iStart == -1)
				{
					strStart = "class='archiwum'><< wstecz</A></TD><TD align=right><A HREF='";
					iStart = page.indexOf(strStart);
				}
				iEnd = page.indexOf(strEnd);
				if (iStart != -1 && iEnd != -1)
				{
					// System.out.println(page.substring(iStart + strStart.length(),
					// iEnd));
					this.enqueue(new LinkInfo("http://www.restauracje.com/"
							+ page.substring(iStart + strStart.length(),iEnd),
							RestComPageTypes.ListPage));
				}
			}
		}
		catch (ContentNotAvilableException e)
		{
			throw new WebpageParseException(e);
		}
	}
}
