package logrus.agents.datasources.restauracjecom;

import java.util.Hashtable;

import logrus.agents.datasources.WebpageParseException;
import logrus.agents.datasources.DataProviderID;
import logrus.agents.datasources.IPropertiesExtractor;
import logrus.ontmodel.Prefixes;
import logrus.ontmodel.QuadsList;
import logrus.ontmodel.restaurant.AcceptedPaymentMethods;
import logrus.ontmodel.restaurant.AccessibilityCodes;
import logrus.ontmodel.restaurant.FeatureCodes;
import logrus.ontmodel.restaurant.ParkingCodes;
import logrus.ontmodel.restaurant.ReservationCodes;
import logrus.ontmodel.restaurant.tools.RestaurantCleaner;
import logrus.ontmodel.validation.ObjectValidator;
import logrus.ontmodel.validation.ValidationStatus;
import logrus.tools.ContentNotAvilableException;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.IdentifierGenerator;
import logrus.tools.Utils;
import logrus.tools.XMLAnalysisException;
import logrus.tools.web.WebUtils;

/**
 * 
 */
public class RestComPropertiesExtractor implements IPropertiesExtractor
{
	// ID of processed restaurant
	private String uriRestaurant;

	// extracted data transformed into RDF statements
	private QuadsList result;

	// language strings mappings
	private Hashtable languageMap = new Hashtable();

	// cuisine strings mappings
	private Hashtable cuisineMap = new Hashtable();

	//
	private String streetCheck = null;

	private static final String headerName = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Nazwa restauracji:</td>";

	private static final String headerAddress = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Adres:</td>";

	private static final String headerPhone = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Telefon:</td>";

	private static final String headerEmail = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Adres e-mail:</td>";

	private static final String headerWWW = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Strona internetowa</td>";

	private static final String headerCuisine = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Rodzaj kuchni:</td>";

	private static final String headerSeats = "cych:</td>";

	private static final String headerOpenHours = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Godziny otwarcia:</td>";

	private static final String headerFeatures = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Udogodnienia:</td>";

	private static final String headerForeignLanguages = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Jzyki obce:</td>";

	private static final String headerBankCards = "<td width=\"174\" background=\"images/bg1.gif\" class=\"restauracje3\">Karty patnicze:</td>";

	private static final String tagTDLine = "<td width=\"150\" background=\"images/bg2.gif\" class=\"restauracje4\">";

	private static final String tagFeatParking = "img src=pictogramy/parking.gif";

	private static final String tagFeatReservations = "img src=pictogramy/rezerwacje.gif";

	private static final String tagFeatGarden = "src=pictogramy/ogrod.gif";

	private static final String tagFeatParties = "src=pictogramy/imprezy.gif";

	private static final String tagFeatAccessibility = "src=pictogramy/niepelnosprawni.gif";

	private static final String tagFeatCatering = "src=pictogramy/catering.gif";

	private static final String tagFeatAirCondition = "src=pictogramy/klima.gif";

	private static final String tagFeatRegularCustomerCard = "src=pictogramy/kartaklienta.gif";

	private static final String tagFeatLiveMuisc = "src=pictogramy/muzyka.gif";

	private static String[] StreetPrefixTags = { " ul ", " Ul ", " ul. ",
			" Ul. ", " ul.", " Ul.", " al ", " Al ", " al. ", " Al. ", " al.",
			" Al.", " pl ", " Pl ", " pl. ", " Pl. ", " pl.", " Pl.", };

	private static String[] StreetPrefixTags2 = { "ul ", "Ul ", "ul. ", "Ul. ",
			"al. ", "Al. ", "al ", " Al ", "pl. ", "Pl. ", "pl ", "Pl " };

	private String streetPref = null;

	/**
	 * 
	 */
	public RestComPropertiesExtractor (String uriRestaurant)
	{
		super();
		this.setResult(new QuadsList());
		this.uriRestaurant = Prefixes.BaseRestaurantURI + "/" + uriRestaurant;
		this.initLanguageMap();
		this.initCuisineMap();
	}

	/**
	 * @throws IdentifierGenerationFailedException
	 */
	public RestComPropertiesExtractor ()
			throws IdentifierGenerationFailedException
	{
		super();
		this.setResult(new QuadsList());
		this.uriRestaurant = Prefixes.BaseRestaurantURI + "/"
				+ IdentifierGenerator.getInstance().GenerateID();
		this.initLanguageMap();
		this.initCuisineMap();
	}

	private void initCuisineMap ()
	{
		this.cuisineMap.put("polska",Prefixes.BaseRestaurantURI
				+ "#PolishCuisine");
		this.cuisineMap.put("staropolska",Prefixes.BaseRestaurantURI
				+ "#PolishCuisine");
		this.cuisineMap.put("francuska",Prefixes.BaseRestaurantURI
				+ "#FrenchCuisine");
		this.cuisineMap.put("chiska",Prefixes.BaseRestaurantURI
				+ "#ChineseCuisine");
		this.cuisineMap.put("meksykaska",Prefixes.BaseRestaurantURI
				+ "#MexicanCuisine");
		this.cuisineMap.put("woska",Prefixes.BaseRestaurantURI
				+ "#ItalianCuisine");
		this.cuisineMap.put("wgierska",Prefixes.BaseRestaurantURI
				+ "#HungarianCuisine");
		this.cuisineMap.put("wietnamska",Prefixes.BaseRestaurantURI
				+ "#VietnameseCuisine");
		this.cuisineMap.put("wegetariaska",Prefixes.BaseRestaurantURI
				+ "#VegetarianCuisine");
		this.cuisineMap.put("ryby",Prefixes.BaseRestaurantURI + "#FishCuisine");
		this.cuisineMap.put("pizzeria",Prefixes.BaseRestaurantURI
				+ "#PizzaCuisine");
		this.cuisineMap.put("bakaska",Prefixes.BaseRestaurantURI
				+ "#BalkanCuisine");
		this.cuisineMap.put("europejska",Prefixes.BaseRestaurantURI
				+ "#EuropeanCuisine");
		this.cuisineMap.put("egzotyczna",Prefixes.BaseRestaurantURI
				+ "#ExoticCuisine");
		this.cuisineMap
				.put("grecka",Prefixes.BaseRestaurantURI + "#GreekCuisine");
		this.cuisineMap.put("niemiecka",Prefixes.BaseRestaurantURI
				+ "#GermanCuisine");
		this.cuisineMap.put("orientalna",Prefixes.BaseRestaurantURI
				+ "#OrientCuisine");
		this.cuisineMap.put("rdziemnomorska",Prefixes.BaseRestaurantURI
				+ "#MediterraneanCuisine");
		this.cuisineMap.put("ukraiska",Prefixes.BaseRestaurantURI
				+ "#UkrainianCuisine");
		this.cuisineMap.put("ydowska",Prefixes.BaseRestaurantURI
				+ "#JewishCuisine");
	}

	private void initLanguageMap ()
	{
		this.languageMap.put("j. angielski","English");
		this.languageMap.put("j angielski","English");
		this.languageMap.put("j.angielski","English");
		this.languageMap.put("ang","English");
		this.languageMap.put("jzyk angielski","English");
		this.languageMap.put("angielski","English");
		this.languageMap.put("ang.","English");
		this.languageMap.put("inglish","English");
		this.languageMap.put("arabski","Arabic");
		this.languageMap.put("czajna","Chinese");
		this.languageMap.put("czeski","Czecj");
		this.languageMap.put("francuski","French");
		this.languageMap.put("hindu","Hindu");
		this.languageMap.put("hindi","Hindu");
		this.languageMap.put("indyjski","Hindu");
		this.languageMap.put("hiszpaski","Spanish");
		this.languageMap.put("japoski","Japan");
		this.languageMap.put("niemiecki","German");
		this.languageMap.put("j. niemiecki","German");
		this.languageMap.put("niem.","German");
		this.languageMap.put("niemiec","German");
		this.languageMap.put("doicz.","German");
		this.languageMap.put("polski","Polish");
		this.languageMap.put("portugalski","Portuguese");
		this.languageMap.put("rosyjski","Russian");
		this.languageMap.put("j rosyjski","Russian");
		this.languageMap.put("j. rosyjski","Russian");
		this.languageMap.put("jzyk rosyjski","Russian");
		this.languageMap.put("ruski","Russian");
		this.languageMap.put("serbo-chorwacki","Serbo-Croatian");
		this.languageMap.put("wloski","Italian");
		this.languageMap.put("woski","Italian");
	}

	private String cleanAddress (String strAddress)
	{
		String strAddr = strAddress;
		if (strAddr.length() == 0)
		{
			return strAddr;
		}
		strAddr = strAddr.replaceAll("     "," ");
		strAddr = strAddr.replaceAll("    "," ");
		strAddr = strAddr.replaceAll("   "," ");
		strAddr = strAddr.replaceAll("  "," ");

		while (true)
		{
			int idxLeft = strAddr.indexOf("(");
			int idxRight = strAddr.indexOf(")");
			int idxEnd = strAddr.length() - 1;
			if (idxLeft < 0 || idxRight < 0 || idxLeft > idxRight)
			{
				break;
			}
			else
			{
				strAddr = Utils.getInstance().removeSubstring(strAddr,idxLeft,
						idxRight);
			}
		}
		return strAddr.trim();
	}

	private int locateStreetTag (String str)
	{
		int idx = -1;
		for (int i = 0; i < RestComPropertiesExtractor.StreetPrefixTags.length; i++)
		{
			idx = str.indexOf(RestComPropertiesExtractor.StreetPrefixTags[i]);
			if (idx >= 0)
			{
				this.streetCheck = str.substring(idx
						+ RestComPropertiesExtractor.StreetPrefixTags[i].length());
				switch (i)
				{
					case 0:
					case 1:
					case 2:
					case 3:
					case 4:
					case 5:
						this.streetPref = "Ul. ";
						break;
					case 6:
					case 7:
					case 8:
					case 9:
					case 10:
					case 11:
						this.streetPref = "Al. ";
						break;
					case 12:
					case 13:
					case 14:
					case 15:
					case 16:
					case 17:
						this.streetPref = "Plac ";
						break;
				}
				return idx;
			}
		}
		for (int i = 0; i < RestComPropertiesExtractor.StreetPrefixTags2.length; i++)
		{
			if (str.startsWith(RestComPropertiesExtractor.StreetPrefixTags2[i]))
			{
				this.streetCheck = str.substring(idx
						+ RestComPropertiesExtractor.StreetPrefixTags[i].length());
				return 0;
			}
		}
		return idx;
	}

	private boolean startsStreetTag (String str)
	{
		for (int i = 0; i < RestComPropertiesExtractor.StreetPrefixTags2.length; i++)
		{
			if (str.startsWith(RestComPropertiesExtractor.StreetPrefixTags2[i]))
			{
				return true;
			}
		}
		return false;
	}

	private String extractStreet (String strCheck)
	{
		String street = null;
		int idx = this.locateStreetTag(strCheck);
		if (idx < 0)
		{
			return null;
		}
		// street = str.substring(idx).trim();
		street = this.streetCheck;
		int i = 0;
		for (; i < street.length(); i++)
		{
			char ch = street.charAt(i);
			if (!Character.isLetter(ch) && ch != '-')
			{
				break;
			}
		}
		// all string passed so return full string
		if (i == street.length())
		{
			return this.streetPref + street;
			// previous: return street;
		}
		// else: check if sign was space
		if (i >= 3 && street.charAt(i) == ' ')
		{
			int j = i + 1;
			for (; j < street.length(); j++)
			{
				char ch = street.charAt(j);
				// require first character to be digit
				if (j == i + 1)
				{
					if (!Character.isDigit(ch))
					{
						return null;
					}
				}
				if (!Character.isLetterOrDigit(ch) && ch != '/')
				{
					break;
				}
			}
			// all string passed so return full string
			if (j == street.length())
			{
				// return street;
				return this.streetPref + street;
			}
			// else: return substring
			return this.streetPref + street.substring(0,j).trim();
			// previous: return street.substring(0,j).trim();
		}
		else
		{
			return null;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see logrus.agents.datasources.IPropertiesExtractor#extract(java.lang.String)
	 */	
	public QuadsList extract (String url) throws WebpageParseException, XMLAnalysisException
	{
		this.extract_inner(url);
		QuadsList quads = this.getResult();
		RestaurantCleaner.getInstance().cleanInfo(quads);
		switch (ObjectValidator.getInstance().validate(quads))
		{
			case ValidationStatus.Complete:
			case ValidationStatus.Incomplete:
				return quads;
			case ValidationStatus.Incorrect:
			case ValidationStatus.Unknown:
				return null;
		}
		return null;
	}

	public QuadsList extract_inner (String url) throws WebpageParseException
	{
		// System.out.println(url);

		String page = null;
		try
		{
			page = WebUtils.getInstance().getContentByURL(url,"windows-1250");
		}
		catch (ContentNotAvilableException e)
		{
			throw new WebpageParseException(e);
		}

		// add RDFS class binding
		this.result.add(this.uriRestaurant,Prefixes.RDF_TYPE,Prefixes.CLASS_RES,
				DataProviderID.RESTCOM);

		// add metadata about information source
		this.result.add(this.uriRestaurant,Prefixes.SourceLink,url,
				DataProviderID.RESTCOM);

		page = page.replaceAll("\n"," ");
		page = page.replaceAll("\t"," ");

		String aux;
		int idx1, idx2;

		// extract restaurant name
		int idxName = page.indexOf(RestComPropertiesExtractor.headerName);
		if (idxName < 0)
		{
			throw new WebpageParseException("Can't find restaurant name header");
		}
		page = page.substring(idxName
				+ RestComPropertiesExtractor.headerName.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strRestName = Utils.getInstance().strTrim(
				page.substring(
						idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2),
				'"');
		this.result.add(this.uriRestaurant,Prefixes.RES_NAME,strRestName,
				DataProviderID.RESTCOM);

		// extract address
		int idxAddress = page.indexOf(RestComPropertiesExtractor.headerAddress);
		if (idxAddress < 0)
		{
			throw new WebpageParseException("Can't find adress header");
		}
		page = page.substring(idxAddress
				+ RestComPropertiesExtractor.headerAddress.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strAddress = page.substring(
				idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2).trim();
		strAddress = this.cleanAddress(strAddress);
		String[] addrParts = strAddress.split("\r");
		String tmp = null;
		int len;
		String parsedZipCode = null;
		String parsedCityName = null;
		String parsedStreetAddr = null;
		Utils utils = Utils.getInstance();
		if (addrParts.length > 1)
		{
			boolean zipParsedThisCycle = false;
			for (int i = 0; i < addrParts.length; i++, zipParsedThisCycle = false)
			{
				String part = addrParts[i].trim();
				len = part.length();
				// try to extract zip code
				if (len >= 6)
				{
					tmp = part.substring(0,6);
					if (utils.isPolishZipCode(tmp))
					{
						parsedZipCode = tmp;
						zipParsedThisCycle = true;
					}
					else
					{
						tmp = part.substring(len - 6,len);
						if (utils.isPolishZipCode(tmp))
						{
							parsedZipCode = tmp;
							zipParsedThisCycle = true;
						}
					}
				}
				if (parsedZipCode != null && zipParsedThisCycle)
				{
					// guess: city name is near
					int iZipLoc = part.indexOf(tmp);
					tmp = utils.removeSubstring(part,iZipLoc,iZipLoc + 5);
					if (utils.isSimpleName(tmp))
					{
						parsedCityName = tmp;
					}
				}
				if (!zipParsedThisCycle)
				{
					// check if it's street address
					if (this.startsStreetTag(part))
					{
						parsedStreetAddr = part;
					}
				}
			}
		}
		else
		{
			// try to find zip code in full string
			for (int i = 0; i < strAddress.length() - 6; i++)
			{
				if (utils.isPolishZipCode(strAddress.substring(i,i + 6)))
				{
					parsedZipCode = strAddress.substring(i,i + 6);

					// heuristics: guess next string is city name
					String strProbCity = strAddress.substring(i + 6).trim();
					// if it is some string after zip code and it not starts with something that looks
					// like street tag and it not starts with digit then assume that it is city name
					if (strProbCity.length() > 0
							&& !this.startsStreetTag(strProbCity)
							&& !Character.isDigit(strProbCity.charAt(0)))
					{
						int iCh = 0;
						while (iCh<strProbCity.length() && Character.isLetter(strProbCity.charAt(iCh)))
						{
							iCh++;
						}
						if (iCh > 1)
						{
							parsedCityName = strProbCity.substring(0,iCh);
						}
					}
				}
			}
		}
		if (parsedZipCode != null && parsedCityName == null)
		{
			int idxParsedZipCode = strAddress.indexOf(parsedZipCode);

			// heuristics: guess next string is city name
			String strProbCity = strAddress.substring(idxParsedZipCode + 6).trim();
			// if it is some string after zip code and it not starts with something that looks
			// like street tag and it not starts with digit then assume that it is city name
			if (strProbCity.length() > 0 && !this.startsStreetTag(strProbCity)
					&& !Character.isDigit(strProbCity.charAt(0)))
			{
				int iCh = 0;
				while (Character.isLetter(strProbCity.charAt(iCh)))
				{
					iCh++;
				}
				if (iCh > 1)
				{
					parsedCityName = strProbCity.substring(0,iCh);
				}
			}
		}
		if (parsedStreetAddr == null)
		{
			// try to find street address in full string
			strAddress = strAddress.replaceAll("\r"," ");
			strAddress = strAddress.replaceAll("\t"," ");
			strAddress = strAddress.replaceAll("\n"," ");
			strAddress = strAddress.replaceAll("     "," ");
			strAddress = strAddress.replaceAll("    "," ");
			strAddress = strAddress.replaceAll("   "," ");
			strAddress = strAddress.replaceAll("  "," ");
			parsedStreetAddr = this.extractStreet(strAddress);
		}
		this.result.add(this.uriRestaurant,Prefixes.LOC_COUNTRY,"Poland",
				DataProviderID.RESTCOM);
		if (parsedCityName != null && parsedCityName.length()>0)
		{
			this.result.add(this.uriRestaurant,Prefixes.LOC_CITY,parsedCityName,
					DataProviderID.RESTCOM);
		}
		if (parsedZipCode != null && parsedZipCode.length()>0)
		{
			this.result.add(this.uriRestaurant,Prefixes.LOC_ZIP,parsedZipCode,
					DataProviderID.RESTCOM);
		}
		if (parsedStreetAddr != null && parsedStreetAddr.length()>0)
		{
			this.result.add(this.uriRestaurant,Prefixes.LOC_STREET_ADDR,
					parsedStreetAddr,DataProviderID.RESTCOM);
		}

		// extract phone number
		int idxPhone = page.indexOf(RestComPropertiesExtractor.headerPhone);
		if (idxPhone < 0)
		{
			throw new WebpageParseException("Can't find phone header");
		}
		page = page.substring(idxPhone
				+ RestComPropertiesExtractor.headerPhone.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strPhone = page.substring(idx1
				+ RestComPropertiesExtractor.tagTDLine.length(),idx2);
		if (strPhone.length() > 0)
		{
			char c = strPhone.charAt(0);
			if (Character.isDigit(c) || c == '+' || c == '(')
			{
				this.result.add(this.uriRestaurant,Prefixes.LOC_PHONE,strPhone,
						DataProviderID.RESTCOM);
			}
		}

		// extract email
		int idxEmail = page.indexOf(RestComPropertiesExtractor.headerEmail);
		if (idxEmail < 0)
		{
			throw new WebpageParseException("Can't find e-mail header");
		}
		page = page.substring(idxEmail
				+ RestComPropertiesExtractor.headerEmail.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strEmail = page.substring(
				idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2).trim()
				.toLowerCase();
		if (strEmail.length() > 0 && strEmail.compareTo("brak") != 0
				&& strEmail.compareTo("brak informacji") != 0)
		{
			String strAux1 = "class='szczegolylink'>";
			String strAux2 = "</a>";
			int iAux1 = strEmail.indexOf(strAux1);
			int iAux2 = strEmail.indexOf(strAux2);
			strEmail = strEmail.substring(iAux1 + strAux1.length(),iAux2);
			this.result.add(this.uriRestaurant,Prefixes.RES_EMAIL,strEmail,
					DataProviderID.RESTCOM);
		}

		// extract www
		int idxWWW = page.indexOf(RestComPropertiesExtractor.headerWWW);
		if (idxWWW < 0)
		{
			throw new WebpageParseException("Can't find WWW header");
		}
		page = page.substring(idxWWW
				+ RestComPropertiesExtractor.headerWWW.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strWWW = page.substring(
				idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2).trim()
				.toLowerCase();
		if (strWWW.length() > 0 && strWWW.compareTo("brak") != 0
				&& strWWW.compareTo("brak informacji") != 0)
		{
			String strAux1 = "class='szczegolylink'>";
			String strAux2 = "</a>";
			int iAux1 = strWWW.indexOf(strAux1);
			int iAux2 = strWWW.indexOf(strAux2);
			strWWW = strWWW.substring(iAux1 + strAux1.length(),iAux2).trim();
			if (!strWWW.startsWith("http://"))
			{
				strWWW = "http://" + strWWW;
			}
			if (strWWW.compareTo("http://www.") != 0)
			{
				this.result.add(this.uriRestaurant,Prefixes.RES_WWW,strWWW,
						DataProviderID.RESTCOM);
			}
		}

		// extract cuisine
		int idxCuisine = page.indexOf(RestComPropertiesExtractor.headerCuisine);
		if (idxCuisine < 0)
		{
			throw new WebpageParseException("Can't find cuisine header");
		}
		page = page.substring(idxCuisine
				+ RestComPropertiesExtractor.headerCuisine.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strCuisine = page.substring(
				idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2).trim();
		String[] cuisines = strCuisine.split("<br>");
		for (int i = 0; i < cuisines.length; i++)
		{
			String cui = cuisines[i].trim().toLowerCase();
			if (cui.length() == 0)
			{
				continue;
			}
			Object obj = this.cuisineMap.get(cui);
			if (obj != null)
			{
				this.result.add(this.uriRestaurant,Prefixes.RES_CUISINE,
						(String)obj,DataProviderID.RESTCOM);
			}
			else
			{
				System.out.println("Unknown cuisine " + cui + " " + url);
			}
		}

		// extract seats
		int idxSeats = page.indexOf(RestComPropertiesExtractor.headerSeats);
		if (idxSeats < 0)
		{
			throw new WebpageParseException("Can't find seats header");
		}
		page = page.substring(idxSeats
				+ RestComPropertiesExtractor.headerSeats.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strSeats = page.substring(
				idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2).trim();
		try
		{
			int iSeats = Integer.parseInt(strSeats);
			if (iSeats > 0)
			{
				this.result.add(this.uriRestaurant,Prefixes.RES_SEATS,strSeats,
						DataProviderID.RESTCOM);
			}
		}
		catch (NumberFormatException ne)
		{
			// ignore
		}

		// extract open hours
		int idxOpenHours = page
				.indexOf(RestComPropertiesExtractor.headerOpenHours);
		if (idxOpenHours < 0)
		{
			throw new WebpageParseException("Can't find open hours header");
		}
		page = page.substring(idxOpenHours
				+ RestComPropertiesExtractor.headerOpenHours.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strOpenHours = page.substring(
				idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2).trim();
		// hardcore, to many formats :(
		// if (strOpenHours.compareTo("brak")!=0)
		// {
		//	
		// //System.out.println(strOpenHours);
		// }

		// extract features
		int idxFeatures = page.indexOf(RestComPropertiesExtractor.headerFeatures);
		if (idxFeatures < 0)
		{
			throw new WebpageParseException("Can't find features header");
		}
		page = page.substring(idxOpenHours
				+ RestComPropertiesExtractor.headerFeatures.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");

		if (page.indexOf(RestComPropertiesExtractor.tagFeatAccessibility) > 0)
		{
			// add accessibility description
			this.result.add(this.uriRestaurant,Prefixes.RES_HANDACC,
					AccessibilityCodes.SomeAccessibility,DataProviderID.RESTCOM);
		}
		if (page.indexOf(RestComPropertiesExtractor.tagFeatAirCondition) > 0)
		{
			// add air condition feature
			this.result.add(this.uriRestaurant,Prefixes.RES_FEAT,
					FeatureCodes.AirCondition,DataProviderID.RESTCOM);
		}
		if (page.indexOf(RestComPropertiesExtractor.tagFeatCatering) > 0)
		{
			// add catering feature
			this.result.add(this.uriRestaurant,Prefixes.RES_FEAT,
					FeatureCodes.OffsiteCatering,DataProviderID.RESTCOM);
		}
		if (page.indexOf(RestComPropertiesExtractor.tagFeatGarden) > 0)
		{
			// add garden feature
			this.result.add(this.uriRestaurant,Prefixes.RES_FEAT,
					FeatureCodes.Garden,DataProviderID.RESTCOM);
		}
		if (page.indexOf(RestComPropertiesExtractor.tagFeatLiveMuisc) > 0)
		{
			// add live music feature
			this.result.add(this.uriRestaurant,Prefixes.RES_FEAT,
					FeatureCodes.LiveMusic,DataProviderID.RESTCOM);
		}
		if (page.indexOf(RestComPropertiesExtractor.tagFeatParking) > 0)
		{
			// add parking description
			this.result.add(this.uriRestaurant,Prefixes.RES_PARK,
					ParkingCodes.SomeParking,DataProviderID.RESTCOM);
		}
		if (page.indexOf(RestComPropertiesExtractor.tagFeatParties) > 0)
		{
			// add private parties feature
			this.result.add(this.uriRestaurant,Prefixes.RES_FEAT,
					FeatureCodes.PrivateParties,DataProviderID.RESTCOM);
		}
		if (page.indexOf(RestComPropertiesExtractor.tagFeatRegularCustomerCard) > 0)
		{
			// add regular customer card feature
			this.result.add(this.uriRestaurant,Prefixes.RES_FEAT,
					FeatureCodes.RegularCustomerCard,DataProviderID.RESTCOM);
		}
		if (page.indexOf(RestComPropertiesExtractor.tagFeatReservations) > 0)
		{
			// add reservations description
			this.result.add(this.uriRestaurant,Prefixes.RES_RESERV,
					ReservationCodes.AcceptedReservations,DataProviderID.RESTCOM);
		}

		// extract foreign languages
		int idxLanguages = page
				.indexOf(RestComPropertiesExtractor.headerForeignLanguages);
		if (idxLanguages < 0)
		{
			throw new WebpageParseException(
					"Can't find foreign languages header");
		}
		page = page.substring(idxLanguages
				+ RestComPropertiesExtractor.headerForeignLanguages.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strLanguages = page.substring(
				idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2).trim()
				.toLowerCase();
		if (strLanguages.compareTo("brak") != 0
				&& strLanguages.compareTo("brak informacji") != 0
				&& strLanguages.compareTo("tak") != 0)
		{
			strLanguages = strLanguages.replaceAll("\n",",");
			strLanguages = strLanguages.replaceAll(" ",",");
			strLanguages = strLanguages.replaceAll("/",",");
			strLanguages = strLanguages.replaceAll("\t",",");
			strLanguages = strLanguages.replaceAll(";",",");
			String[] langArr = strLanguages.split(",");
			for (int i = 0; i < langArr.length; i++)
			{
				String lang = langArr[i].trim().toLowerCase();
				if (lang.length() == 0)
				{
					continue;
				}
				Object obj = this.languageMap.get(lang);
				if (obj != null)
				{
					// add service language
					this.result.add(this.uriRestaurant,Prefixes.RES_LANG,
							(String)obj,DataProviderID.RESTCOM);
				}
			}
		}
		if (!this.result.containsStatement(this.uriRestaurant,Prefixes.RES_LANG,
				"Polish",DataProviderID.RESTCOM))
		{
			this.result.add(this.uriRestaurant,Prefixes.RES_LANG,"Polish",
					DataProviderID.RESTCOM);
		}

		// extract bank cards
		int idxBankCards = page
				.indexOf(RestComPropertiesExtractor.headerBankCards);
		if (idxBankCards < 0)
		{
			throw new WebpageParseException("Can't find bank cards header");
		}
		page = page.substring(idxBankCards
				+ RestComPropertiesExtractor.headerBankCards.length());
		idx1 = page.indexOf(RestComPropertiesExtractor.tagTDLine);
		idx2 = page.indexOf("</td>");
		String strBankCards = page.substring(
				idx1 + RestComPropertiesExtractor.tagTDLine.length(),idx2).trim()
				.toLowerCase();
		if (strBankCards.length() > 0)
		{
			if (strBankCards.compareTo("tak") == 0
					|| strBankCards.compareTo("akceptujemy") == 0
					|| strBankCards.compareTo("wszystkie") == 0
					|| strBankCards.compareTo("akceptacja") == 0
					|| strBankCards.compareTo("przyjmujemy patnoci kartami") == 0
					|| strBankCards.compareTo("wszystkie rodzaje") == 0
					|| strBankCards.compareTo("tak, wszystkie") == 0
					|| strBankCards.compareTo("realizowane") == 0
					|| strBankCards.compareTo("all") == 0
					|| strBankCards
							.compareTo("akceptujemy wszystkie karty kredytowe") == 0
					|| strBankCards.compareTo("wszytkie rodzaje kart kredytowych") == 0
					|| strBankCards.compareTo("respektujemy") == 0
					|| strBankCards.compareTo("tak wszystkie") == 0
					|| strBankCards.compareTo("honorujemy wszystkie karty") == 0
					|| strBankCards.compareTo("all card") == 0
					|| strBankCards.compareTo("wszelkie") == 0
					|| strBankCards.compareTo("przyjmuje") == 0)
			{
				// wszystkie obsugiwane przez polcard

				// add bank cards accepted information
				this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
						AcceptedPaymentMethods.BankCard,DataProviderID.RESTCOM);
			}
			else if (strBankCards.compareTo("nie") == 0
					|| strBankCards.compareTo("nie akceptujemy") == 0
					|| strBankCards.compareTo("niestety nie") == 0
					|| strBankCards.compareTo("brak") == 0
					|| strBankCards.compareTo("-") == 0)
			{
				// add bank cards not accepted information
				this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
						AcceptedPaymentMethods.NoBankCard,DataProviderID.RESTCOM);
			}
			else if (strBankCards.compareTo("brak informacji") == 0)
			{
				// do nothing
			}
			else
			{
				String[] cardNames = strBankCards.split(",");
				for (int i = 0; i < cardNames.length; i++)
				{
					String name = cardNames[i].trim();
					if (name.compareTo("visa") == 0
							|| name.compareTo("visa electron") == 0
							|| name.compareTo("visa elektron") == 0
							|| name.compareTo("visa classic") == 0
							|| name.compareTo("visa-elektron") == 0
							|| name.compareTo("visa-electron") == 0
							|| strBankCards.indexOf("<img height=25 alt=\"visa\"") >= 0)
					{
						// add visa as accepted card
						this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
								AcceptedPaymentMethods.VisaCard,DataProviderID.RESTCOM);
					}
					else if (name.compareTo("mastercard") == 0
							|| name.compareTo("master card") == 0
							|| name.compareTo("eurocard") == 0
							|| name.compareTo("ec") == 0
							|| name.compareTo("mc") == 0
							|| name.compareTo("mastercard electroniq") == 0
							|| name.compareTo("mastercard electronic") == 0
							|| name.compareTo("mastercard electr.") == 0
							|| strBankCards
									.indexOf("<img height=25 alt=\"mastercard\"") >= 0)
					{
						// add MasterCard/EuroCard as accepted card
						this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
								AcceptedPaymentMethods.MasterCardEuroCard,
								DataProviderID.RESTCOM);
					}
					else if (name.compareTo("american express") == 0
							|| name.compareTo("amexp") == 0
							|| name.compareTo("am.express") == 0
							|| name.compareTo("ax") == 0
							|| name.compareTo("ae") == 0
							|| name.compareTo("am-ex") == 0
							|| name.compareTo("amexp") == 0
							|| name.compareTo("amex") == 0
							|| name.compareTo("americanexpress") == 0
							|| strBankCards
									.indexOf("<img height=25 alt=\"american express\"") >= 0)
					{
						// add American Express as accepted card
						this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
								AcceptedPaymentMethods.AmericanExpressCard,
								DataProviderID.RESTCOM);
					}
					else if (name.compareTo("maestro") == 0
							|| strBankCards.indexOf("<img height=25 alt=\"maestro\"") >= 0)
					{
						// add Maestro Card as accepted card
						this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
								AcceptedPaymentMethods.MaestroCard,
								DataProviderID.RESTCOM);
					}
					else if (name.compareTo("dinners club") == 0
							|| name.compareTo("dinner") == 0
							|| name.compareTo("diners club") == 0
							|| name.compareTo("diners") == 0
							|| strBankCards.indexOf("<img height=25 alt=\"maestro\"") >= 0)
					{
						// add Diners Club as accepted card
						this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
								AcceptedPaymentMethods.DinersClubCard,
								DataProviderID.RESTCOM);
					}
					else if (name.compareTo("jcb") == 0
							|| name.compareTo("jcb card") == 0)
					{
						// add Japan Credit Bureau as accepted card
						this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
								AcceptedPaymentMethods.JapanCreditBureauCard,
								DataProviderID.RESTCOM);
					}
					else if (name.compareTo("polcard") == 0
							|| strBankCards.indexOf("<img height=25 alt=\"polcard\"") >= 0)
					{
						// add Polcard as accepted card
						this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
								AcceptedPaymentMethods.Polcard,
								DataProviderID.RESTCOM);
					}
					else
					{
						// ignore
					}
				}
			}
			// add cash accepted information
			this.result.add(this.uriRestaurant,Prefixes.RES_ACC,
					AcceptedPaymentMethods.Cash,DataProviderID.RESTCOM);
		}
		else
		{
			System.out.println("Empty bank cards: " + url);
		}
		return null;
	}

	/**
	 * @param result
	 *           The result to set.
	 */
	public void setResult (QuadsList result)
	{
		this.result = result;
	}

	/**
	 * @return Returns the result.
	 */
	public QuadsList getResult ()
	{
		return this.result;
	}
}
