package logrus.agents.datasources.wrapperAgent.behaviours;

import jade.core.AID;
import jade.core.Agent;
import jade.lang.acl.ACLMessage;
import jade.util.leap.ArrayList;
import logrus.agents.OSMessage;
import logrus.agents.base.behaviours.OSOneShotBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.datasources.wrapperAgent.WrapperAgent;

/**
 * Behaviour intened to be used as a subbehaviour by wrapper agent. Sends single object information
 * to specified agent.
 */
public class SendLinksSetWABehaviour extends OSOneShotBehaviour
{
	private WrapperAgent agentWA;

	// AID of agent to which information should be transfered
	private AID aidClient;

	// identifer of conversation with agent of AID equal aidClient
	private String conversationID;

	// links extracted from
	private ArrayList links;

	public SendLinksSetWABehaviour (Agent a, AID aidClient,
			String conversationID, String[] links)
	{
		super(a);
		this.conversationID = conversationID;
		this.agentWA = (WrapperAgent)a;
		this.links = new ArrayList();
		for (int i = 0; i < links.length; i++)
		{
			this.links.add(links[i]);
		}

		// if no AID for client specified then sent it to SCA
		if (aidClient != null)
		{
			this.aidClient = aidClient;
		}
		else
		{
			this.aidClient = this.agentWA.getAidSCA();
		}
	}

	public void action ()
	{
		OSMessage msg = new OSMessage();
		msg.setMessageCode(OSMessage.WA_Crawl_Result);
		msg.setUris(this.links);
		try
		{
			if (this.conversationID != null)
			{
				this.agentWA.sendMessage(ACLMessage.INFORM,msg,this.agentWA.getAidSCA(),
						this.conversationID);
			}
			else
			{
				this.agentWA.sendMessage(ACLMessage.INFORM,msg,this.agentWA.getAidSCA());
			}
		}
		catch (SendingMessageFailed exc)
		{
			// the only thing we can do is to log error
			this.osAgent.onSendMessgeFailed(this.aidClient,msg.getMessageCode(),
					exc,msg);
		}
	}
}
