package logrus.agents.datasources.wrapperAgent.behaviours;

import jade.core.AID;
import jade.core.Agent;
import jade.lang.acl.ACLMessage;
import logrus.agents.OSMessage;
import logrus.agents.base.behaviours.OSOneShotBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.datasources.wrapperAgent.WrapperAgent;
import logrus.ontmodel.QuadsList;

/**
 * Behaviour intened to be used as a subbehaviour by wrapper agent. Sends single object information
 * to specified agent.
 */
public class SendObjectInfoWABehaviour extends OSOneShotBehaviour
{
	private WrapperAgent agentWA;

	// list of quads to be sent
	private QuadsList quads;

	// AID of agent to which information should be transfered
	private AID aidClient;

	// identifer of conversation with agent of AID equal aidClient
	private String conversationID;

	/**
	 * @param a
	 *           Agent running this behaviour
	 * @param quads
	 *           Object information
	 * @param aidClient
	 *           AID of agent to which information should be transfered
	 * @param conversationID
	 *           Identifer of conversation with agent of AID equal aidClient
	 */
	public SendObjectInfoWABehaviour (Agent a, QuadsList quads, AID aidClient,
			String conversationID)
	{
		super(a);
		this.agentWA = (WrapperAgent)a;
		this.quads = quads;
		this.conversationID = conversationID;

		// if no AID for client specified then sent it to SCA
		if (aidClient != null)
		{
			this.aidClient = aidClient;
		}
		else
		{
			this.aidClient = this.agentWA.getAidSCA();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		OSMessage msg = new OSMessage();
		if (this.quads!=null)
		{
			msg.setMessageCode(OSMessage.WA_Extract_Object_Info_Result);
			msg.setObjectInfo(this.quads);
		}
		else
		{
			msg.setMessageCode(OSMessage.WA_Extract_Object_Info_Failed);
		}
		try
		{
			this.agentWA.sendMessage(ACLMessage.INFORM,msg,this.aidClient,
						this.conversationID);
		}
		catch (SendingMessageFailed exc)
		{
			// the only thing we can do is to log error
			this.osAgent.onSendMessgeFailed(this.aidClient,msg.getMessageCode(),
					exc,msg);
		}
	}
}
