package logrus.agents.gui.mainDialogAgent;

import java.util.logging.Level;

import org.eclipse.swt.widgets.Display;

import jade.content.onto.OntologyException;
import jade.core.AID;
import jade.domain.FIPAException;
import logrus.agents.OntoStorageAgent;
import logrus.agents.ServiceNames;
import logrus.agents.gui.mainDialogAgent.behaviours.BootGUISequenceBehaviour;
import logrus.agents.gui.mainDialogAgent.behaviours.SendReportRequestBehaviour;
import logrus.agents.gui.mainDialogAgent.behaviours.CrawlRequestBehaviour;
import logrus.agents.gui.mainDialogAgent.behaviours.SaveModelRequestBehaviour;
import logrus.agents.gui.mainDialogAgent.behaviours.SendUpdateRequestBehaviour;
import logrus.agents.ontology.OntoStorageOntology;
import logrus.gui.ManagerGUI;
import logrus.tools.logging.InitializeLoggerFailedException;

/**
 * Agent responsible for communication between main GUI Application and rest of the system
 */
public class MainDialogAgent extends OntoStorageAgent implements
		MainDialogAgentExternal
{
	ManagerGUI dialog;
	
	private AID aidMA;
	
	/**
	 * @throws InitializeLoggerFailedException
	 * @throws OntologyException
	 */
	public MainDialogAgent (ManagerGUI dialog) throws InitializeLoggerFailedException,
			OntologyException
	{
		super();
		this.setDialog(dialog);
	}

	/* (non-Javadoc)
	 * @see logrus.agents.gui.mainDialogAgent.MainDialogAgentExternal#bootSystem(java.lang.String)
	 */
	public void bootSystem (String databaseName)
	{
		this.addBehaviour(new BootGUISequenceBehaviour(this,databaseName));
	}
	
	public void setup ()
	{
		this.logg(Level.INFO,"Main GUI agent booted");
		try
		{
			this.registerService(ServiceNames.ServiceGUI, OntoStorageOntology.ONTOLOGY_NAME);
		}
		catch (FIPAException exc)
		{
			this.logg("GUI agent couldn't register service",exc);
		}			
	}	
	
	public void takeDown()
	{
		this.logg(Level.INFO,"Main GUI agent terminating");
	}

	/**
	 * @param dialog The dialog to set.
	 */
	public void setDialog (ManagerGUI dialog)
	{
		this.dialog = dialog;
	}

	/**
	 * @return Returns the dialog.
	 */
	public ManagerGUI getDialog ()
	{
		return this.dialog;
	}

	/**
	 * @param aidMA The aidMA to set.
	 */
	public void setAidMA (AID aidMA)
	{
		this.aidMA = aidMA;
	}

	/**
	 * @return Returns the aidMA.
	 */
	public AID getAidMA ()
	{
		return this.aidMA;
	}

	/* (non-Javadoc)
	 * @see logrus.agents.gui.mainDialogAgent.sendCrawlMessage#bootSystem(int,java.lang.String)
	 */
	public void sendCrawlMessage (String providerID, String link, int options)
	{
		this.addBehaviour(new CrawlRequestBehaviour(this,providerID,link,options));
	}
	
	public void displayMessage (final String strMessage)
	{
		final Display display=this.dialog.getDisplay();
		final String message = strMessage;
		Runnable guiDisplay = new Runnable()
		{
			public void run ()
			{
				MainDialogAgent.this.dialog.displayMessage(message);
			}
		};
		display.asyncExec(guiDisplay);		
	}

	public void saveModel (String path)
	{
		this.addBehaviour(new SaveModelRequestBehaviour(this,path));
	}

	public void sendBuildReportRequest (String ontClass)
	{
		this.addBehaviour(new SendReportRequestBehaviour(this,ontClass));
	}

	public void sendUpdateRequest ()
	{
		this.addBehaviour(new SendUpdateRequestBehaviour(this));
	}
}
