package logrus.agents.gui.mainDialogAgent.behaviours;

import jade.content.OntoACLMessage;
import jade.content.onto.OntologyException;
import jade.core.Agent;
import jade.lang.acl.MessageTemplate;
import jade.wrapper.StaleProxyException;

import java.util.logging.Level;

import logrus.AgentsPlatform;
import logrus.agents.AgentNames;
import logrus.agents.MessageTimeoutException;
import logrus.agents.OSMessage;
import logrus.agents.OSMessageMatcher;
import logrus.agents.Timeouts;
import logrus.agents.base.behaviours.OSOneShotBehaviour;
import logrus.agents.gui.mainDialogAgent.MainDialogAgent;
import logrus.agents.managerAgent.ManagerAgent;
import logrus.tools.logging.InitializeLoggerFailedException;

import org.eclipse.swt.widgets.Display;

public class BootGUISequenceBehaviour extends OSOneShotBehaviour
{
	private String databaseName;

	MainDialogAgent agentGA;

	public BootGUISequenceBehaviour (Agent a, String databaseName)
	{
		super(a);
		this.databaseName = databaseName;
		this.agentGA = (MainDialogAgent)a;
	}

	private void onBootFailed (String announcement)
	{
		this.osAgent.logg(Level.SEVERE,announcement);

		this.osAgent.doDelete();
		return;
	}

public void action ()
	{
		ManagerAgent agentManager = null; 
		try
		{
			agentManager = new ManagerAgent(this.osAgent.getAID(),this.databaseName);
			AgentsPlatform.getInstance().addAgent(agentManager,AgentNames.ManagerAgent);
			this.agentGA.setAidMA(agentManager.getAID());
		}
		catch (OntologyException e)
		{
			this.onBootFailed("Creating MA failed "+e);
			return;
		}
		catch (InitializeLoggerFailedException e)
		{
			this.onBootFailed("Creating MA failed "+e);
			return;
		}
		catch (StaleProxyException e)
		{
			this.onBootFailed("Creating MA failed "+e);
			return;
		}
		
		// wait for confirmation from MA
		OSMessageMatcher matcher = new OSMessageMatcher(this.osAgent
				.getContentManager());
		matcher.addAcceptedMsgCode(OSMessage.MA_Boot_OK);
		matcher.addAcceptedMsgCode(OSMessage.MA_Boot_Failed);
		OntoACLMessage msg;

		try
		{
			msg = this.osAgent.waitForMessage(new MessageTemplate(matcher),this,
					10,500,Timeouts.getInstance().MA_Boot_Timeout);
		}
		catch (MessageTimeoutException e)
		{
			this.onBootFailed("MA boot time limit exceeded: "+e);
			return;
		}
		OSMessage osmsg = OSMessageMatcher.extractMessage(msg,this.osAgent
				.getContentManager());
		if (osmsg.getMessageCode() == OSMessage.MA_Boot_Failed)
		{
			this.onBootFailed("MA boot time limit exceeded: "+osmsg.getAnnouncement());
			return;
		}
		final Display display=this.agentGA.getDialog().getDisplay();
		Runnable guiBootSuccessful = new Runnable()
		{
			public void run ()
			{
				BootGUISequenceBehaviour.this.agentGA.getDialog().onBootSuccessful();
			}
		};
		display.asyncExec(guiBootSuccessful);
		//TestAgent.runTests();
	}}
