package logrus.agents.gui.mainDialogAgent.behaviours;

import jade.content.OntoACLMessage;
import jade.core.AID;
import jade.core.Agent;
import jade.domain.FIPAException;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import java.util.logging.Level;

import logrus.agents.OSMessage;
import logrus.agents.OSMessageMatcher;
import logrus.agents.base.behaviours.OSBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.gui.mainDialogAgent.MainDialogAgent;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.IdentifierGenerator;
import logrus.tools.Utils;

/**
 * 
 */
public class SaveModelRequestBehaviour extends OSBehaviour
{
	// path of file where model should be saved
	private String filePath;
	
	// GUI agent reference
	private MainDialogAgent agentGUI;

	// conversation ID for whole process
	private String conversationID;

	// flag determining if this behaviour is finished
	private boolean finished;

	// current stage of behaviour
	private int stage;

	// index of stage responsible for sending crawl request to DBA
	private static final int StageSendSaveModelRequest = 0;

	// index of stage responsible for waiting for reply from DBA
	private static final int StageWaitForReply = 1;

	// index of final stage of this behaviour
	private static final int StageFinal = 10;

	// error message
	private String strErrorMessage;

	
	/**
	 * @param a
	 */
	public SaveModelRequestBehaviour (Agent a, String filePath)
	{
		super(a);
		this.agentGUI = (MainDialogAgent)a;
		this.conversationID = null;
		this.finished = false;
		this.stage = StageSendSaveModelRequest;
		this.filePath = filePath;
		this.strErrorMessage = null;
	}

	/* (non-Javadoc)
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		switch (this.stage)
		{
			case StageSendSaveModelRequest:
				this.sendSaveQuery();
				break;
			case StageWaitForReply:
				this.waitForReply();
				break;
			case StageFinal:
				this.finalizeBehaviour();
				break;
		}
	}
	
	private void sendSaveQuery ()
	{
		AID aidDBA = null;
		OSMessage msg = null;
		try
		{
			// find AID of DBA agent
			//aidDBA = this.osAgent.findAgentPerformingService(ServiceNames.ServiceDBA);
			aidDBA = Utils.getInstance().tossDBARead(this.osAgent);
			if (aidDBA == null)
			{
				this.strErrorMessage = "No DBA service available";
				this.stage = StageFinal;
			}

			// generate conversation ID
			this.conversationID = IdentifierGenerator.getInstance().GenerateID();

			msg = new OSMessage();
			msg.setMessageCode(OSMessage.DBA_Save_Model);
			msg.setAnnouncement(this.filePath);
			this.agentGUI.sendMessage(ACLMessage.REQUEST,msg,aidDBA,
					this.conversationID);
			this.stage = StageWaitForReply;
		}
		catch (FIPAException exc)
		{
			this.strErrorMessage = "Error while searching for DBA service "
					+ exc.getMessage();
			this.stage = StageFinal;
		}
		catch (IdentifierGenerationFailedException exc)
		{
			this.strErrorMessage = "Couldn't generate conversation ID "
					+ exc.getMessage();
			this.stage = StageFinal;
		}
		catch (SendingMessageFailed exc)
		{
			this.strErrorMessage = "Couldn't send DBA_Save_Model to DBA "
					+ exc.getMessage();
			this.agentGUI.onSendMessgeFailed(aidDBA,OSMessage.DBA_Execute_RDQL,
					exc,msg);
			this.stage = StageFinal;
		}
	}	
	
	private void waitForReply ()
	{
		// prepare accepted message templates
		OSMessageMatcher matcher = new OSMessageMatcher(this.osAgent
				.getContentManager());
		matcher.addAcceptedMsgCode(OSMessage.DBA_Save_Model_OK);
		matcher.addAcceptedMsgCode(OSMessage.DBA_Save_Model_Failed);
		matcher.setAcceptedConversationID(this.conversationID);
		MessageTemplate pattern = new MessageTemplate(matcher);

		// recive message
		OntoACLMessage jmsg = (OntoACLMessage)this.osAgent.receive(pattern);
		if (jmsg != null)
		{
			OSMessage msg = OSMessageMatcher.extractMessage(jmsg,this.osAgent
					.getContentManager());
			switch (msg.getMessageCode())
			{
				case OSMessage.DBA_Save_Model_OK:
					this.agentGUI.displayMessage("Model saved");
					break;
				case OSMessage.DBA_Save_Model_Failed:
					this.agentGUI
							.displayMessage("Error while saving model: "
									+ msg.getAnnouncement() != null ? msg
									.getAnnouncement() : "");
					break;
			}
			this.stage = StageFinal;
		}
		else
		{
			// block behaviour until message arrives
			block();
		}
	}
	
	
	/**
	 * Perform final actions before finishing behaviour
	 */
	private void finalizeBehaviour ()
	{
		// if there were some errors then log them
		if (this.strErrorMessage != null)
		{
			this.agentGUI.logg(Level.SEVERE,this.strErrorMessage);
			this.agentGUI.displayMessage(this.strErrorMessage);
		}

		// mark behaviour as finished
		this.finished = true;
	}	

	/* (non-Javadoc)
	 * @see jade.core.behaviours.Behaviour#done()
	 */
	public boolean done ()
	{
		return this.finished;
	}
}
