package logrus.agents.gui.mainDialogAgent.behaviours;

import jade.content.OntoACLMessage;
import jade.core.AID;
import jade.core.Agent;
import jade.domain.FIPAException;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import java.util.logging.Level;

import logrus.agents.OSMessage;
import logrus.agents.OSMessageMatcher;
import logrus.agents.base.behaviours.OSBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.gui.mainDialogAgent.MainDialogAgent;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.IdentifierGenerator;
import logrus.tools.Utils;

/**
 * 
 */
public class SendReportRequestBehaviour extends OSBehaviour
{
	// GUI agent reference
	private MainDialogAgent agentGUI;

	// conversation ID for whole process
	private String conversationID;

	// flag determining if this behaviour is finished
	private boolean finished;

	// current stage of behaviour
	private int stage;

	// index of stage responsible for sending crawl request to MA
	private static final int StageSendRequest = 0;

	// index of stage responsible for waiting for reply from MA
	private static final int StageWaitForReply = 1;

	// index of final stage of this behaviour
	private static final int StageFinal = 10;

	// error message
	private String strErrorMessage;
	
	// ontological class used for filtering objects
	private String ontClass;
	
	/**
	 * @param a
	 */
	public SendReportRequestBehaviour (Agent a, String ontClass)
	{
		super(a);
		this.ontClass = ontClass;
		this.agentGUI = (MainDialogAgent)a;
		this.stage = StageSendRequest;
		this.strErrorMessage = null;
		this.conversationID = null;
	}

	/* (non-Javadoc)
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		switch (this.stage)
		{
			case StageSendRequest:
				this.sendRequest();
				break;
			case StageWaitForReply:
				this.waitForReply();
				break;
			case StageFinal:
				this.finalizeBehaviour();
				break;
		}
	}
	
	/**
	 * Send MA_Crawl_Request to MA
	 */
	private void sendRequest ()
	{
		
		// prepare conversationID
		try
		{
			this.conversationID = IdentifierGenerator.getInstance().GenerateID();
		}
		catch (IdentifierGenerationFailedException exc)
		{
			this.strErrorMessage = "Couldn't generate ID for conversation "
					+ exc.getMessage();
			this.stage = StageFinal;
		}
		
		// find AID of DBA agent
		AID aidDBA = null;
		try
		{
			//aidDBA = this.osAgent.findAgentPerformingService(ServiceNames.ServiceDBA);
			aidDBA = Utils.getInstance().tossDBARead(this.osAgent);
			if (aidDBA == null)
			{
				this.strErrorMessage = "No DBA service available";
				this.stage = StageFinal;
			}			
		}
		catch (FIPAException exc)
		{
			this.strErrorMessage = "Error while searching for DBA service "
					+ exc.getMessage();
			this.stage = StageFinal;
		}
		

		// prepare message
		OSMessage msg = new OSMessage();
		msg.setMessageCode(OSMessage.DBA_Build_Report);
		msg.setAnnouncement(this.ontClass);

		// send message
		try
		{
			this.agentGUI.sendMessage(ACLMessage.REQUEST,msg,aidDBA,this.conversationID);
			this.stage = StageWaitForReply;
		}
		catch (SendingMessageFailed exc)
		{
			this.agentGUI.onSendMessgeFailed(this.agentGUI.getAidMA(),
					OSMessage.DBA_Build_Report,exc,msg);
			this.strErrorMessage = "Couldn't send message DBA_Build_Report to DBA"
					+ exc.getMessage();
			this.stage = StageFinal;
		}
	}	
	
	private void waitForReply ()
	{
		// prepare accepted message templates
		OSMessageMatcher matcher = new OSMessageMatcher(this.osAgent
				.getContentManager());
		matcher.addAcceptedMsgCode(OSMessage.DBA_Build_Report_OK);
		matcher.addAcceptedMsgCode(OSMessage.DBA_Build_Report_Failed);
		matcher.setAcceptedConversationID(this.conversationID);
		MessageTemplate pattern = new MessageTemplate(matcher);

		// recive message
		OntoACLMessage jmsg = (OntoACLMessage)this.osAgent.receive(pattern);
		if (jmsg != null)
		{
			OSMessage msg = OSMessageMatcher.extractMessage(jmsg,this.osAgent
					.getContentManager());
			switch (msg.getMessageCode())
			{
				case OSMessage.DBA_Build_Report_OK:
					this.agentGUI.displayMessage(msg.getAnnouncement());
					break;
				case OSMessage.DBA_Build_Report_Failed:
					this.agentGUI
							.displayMessage("Error while building report. "
									+ msg.getAnnouncement() != null ? msg
									.getAnnouncement() : "");
					break;
			}
			this.stage = StageFinal;
		}
		else
		{
			// block behaviour until message arrives
			block();
		}
	}	

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#done()
	 */
	public boolean done ()
	{
		return this.finished;
	}

	/**
	 * Perform final actions before finishing behaviour
	 */
	private void finalizeBehaviour ()
	{
		// if there were some errors then log them
		if (this.strErrorMessage != null)
		{
			this.agentGUI.logg(Level.SEVERE,this.strErrorMessage);
			this.agentGUI.displayMessage(this.strErrorMessage);
		}

		// mark behaviour as finished
		this.finished = true;
	}
}
