package logrus.agents.gui.mainDialogAgent.behaviours;

import jade.content.OntoACLMessage;
import jade.core.AID;
import jade.core.Agent;
import jade.domain.FIPAException;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import java.util.logging.Level;

import logrus.agents.OSMessage;
import logrus.agents.OSMessageMatcher;
import logrus.agents.ServiceNames;
import logrus.agents.base.behaviours.OSBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.gui.mainDialogAgent.MainDialogAgent;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.IdentifierGenerator;

public class SendUpdateRequestBehaviour extends OSBehaviour
{
	// GUI agent reference
	private MainDialogAgent agentGUI;

	// conversation ID for whole process
	private String conversationID;

	// flag determining if this behaviour is finished
	private boolean finished;

	// current stage of behaviour
	private int stage;

	// index of stage responsible for sending crawl request to DSA
	private static final int StageSendRequest = 0;

	// index of stage responsible for waiting for reply from DSA
	private static final int StageWaitForReply = 1;

	// index of final stage of this behaviour
	private static final int StageFinal = 10;

	// error message
	private String strErrorMessage;

	/**
	 * @param a
	 */
	public SendUpdateRequestBehaviour (Agent a)
	{
		super(a);
		this.agentGUI = (MainDialogAgent)a;
		this.stage = StageSendRequest;
		this.strErrorMessage = null;
		this.conversationID = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		switch (this.stage)
		{
			case StageSendRequest:
				this.sendRequest();
				break;
			case StageWaitForReply:
				this.waitForReply();
				break;
			case StageFinal:
				this.finalizeBehaviour();
				break;
		}
	}

	/**
	 * Send MA_Crawl_Request to MA
	 * 
	 * @throws FIPAException
	 */
	private void sendRequest ()
	{
		// prepare conversationID
		try
		{
			this.conversationID = IdentifierGenerator.getInstance().GenerateID();
		}
		catch (IdentifierGenerationFailedException exc)
		{
			this.strErrorMessage = "Couldn't generate ID for conversation "
					+ exc.getMessage();
			this.stage = StageFinal;
		}

		// prepare message
		OSMessage msg = new OSMessage();
		msg.setMessageCode(OSMessage.DSA_Update);

		AID aidDSA = null;
		try
		{
			aidDSA = this.osAgent
					.findAgentPerformingService(ServiceNames.ServiceDSA);
		}
		catch (FIPAException e)
		{
			this.strErrorMessage = "No DSA running";
			this.stage = StageFinal;
			return;
		}

		// send message
		try
		{
			this.agentGUI.sendMessage(ACLMessage.REQUEST,msg,aidDSA,
					this.conversationID);
			this.stage = StageWaitForReply;
		}
		catch (SendingMessageFailed exc)
		{
			this.agentGUI.onSendMessgeFailed(aidDSA,OSMessage.DSA_Update,exc,msg);
			this.strErrorMessage = "Couldn't send message DSA_Update to DSA"
					+ exc.getMessage();
			this.stage = StageFinal;
		}
	}
	
	private void waitForReply ()
	{
		// prepare accepted message templates
		OSMessageMatcher matcher = new OSMessageMatcher(this.osAgent
				.getContentManager());
		matcher.addAcceptedMsgCode(OSMessage.DSA_Update_OK);
		matcher.addAcceptedMsgCode(OSMessage.DSA_Update_Failed);
		matcher.setAcceptedConversationID(this.conversationID);
		MessageTemplate pattern = new MessageTemplate(matcher);

		// recive message
		OntoACLMessage jmsg = (OntoACLMessage)this.osAgent.receive(pattern);
		if (jmsg != null)
		{
			OSMessage msg = OSMessageMatcher.extractMessage(jmsg,this.osAgent
					.getContentManager());
			switch (msg.getMessageCode())
			{
				case OSMessage.DSA_Update_OK:
					this.agentGUI.displayMessage("Update complete");
					break;
				case OSMessage.DSA_Update_Failed:
					this.agentGUI
							.displayMessage("Error while updating: "
									+ msg.getAnnouncement() != null ? msg
									.getAnnouncement() : "");
					break;
			}
			this.stage = StageFinal;
		}
		else
		{
			// block behaviour until message arrives
			block();
		}
	}		
		
	/**
	 * Perform final actions before finishing behaviour
	 */
	private void finalizeBehaviour ()
	{
		// if there were some errors then log them
		if (this.strErrorMessage != null)
		{
			this.agentGUI.logg(Level.SEVERE,this.strErrorMessage);
			this.agentGUI.displayMessage(this.strErrorMessage);
		}

		// mark behaviour as finished
		this.finished = true;
	}
	
	public boolean done ()
	{
		return this.finished;
	}
}
