package logrus.agents.gui.queryDialogAgent;

import jade.content.onto.OntologyException;
import logrus.agents.OntoStorageAgent;
import logrus.agents.base.behaviours.EmptyBehaviour;
import logrus.agents.gui.queryDialogAgent.behaviours.PerformRDQLBehaviour;
import logrus.gui.RDQLQueryDialog;
import logrus.tools.logging.InitializeLoggerFailedException;

import org.eclipse.swt.widgets.Display;

/**
 * Agent used for communcation with dialog supporting query execution
 */
public class RDQLDialogAgent extends OntoStorageAgent implements
		RDQLDialogAgentExternal
{
	// dialog with which this agent is communicating
	RDQLQueryDialog dialog;

	/**
	 * @param dialog
	 *           dialog with which this agent is communicating
	 * @throws InitializeLoggerFailedException
	 * @throws OntologyException
	 */
	public RDQLDialogAgent (RDQLQueryDialog dialog) throws OntologyException,
			InitializeLoggerFailedException
	{
		super();
		this.setDialog(dialog);
	}

	public void setup ()
	{
		this.addBehaviour(new EmptyBehaviour(this));
	}

	/**
	 * @param dialog
	 *           The dialog to set.
	 */
	public void setDialog (RDQLQueryDialog dialog)
	{
		this.dialog = dialog;
	}

	/**
	 * @return Returns the dialog.
	 */
	public RDQLQueryDialog getDialog ()
	{
		return this.dialog;
	}

	public void executeQuery (String query)
	{
		this.addBehaviour(new PerformRDQLBehaviour(this,query)); 
	}

	public void displayMessage (String strMessage)
	{
		final Display display = this.dialog.getDisplay();
		final String message = strMessage;
		Runnable guiDisplay = new Runnable()
		{
			public void run ()
			{
				RDQLDialogAgent.this.dialog.displayMessage(message);
			}
		};
		display.asyncExec(guiDisplay);
	}

	public void displayResults (String results)
	{
		final Display display = this.dialog.getDisplay();
		final String resultsText = results;
		Runnable guiDisplay = new Runnable()
		{
			public void run ()
			{
				RDQLDialogAgent.this.dialog.displayRDQLResults(resultsText);
			}
		};
		display.asyncExec(guiDisplay);
	}
}
