package logrus.agents.gui.queryDialogAgent.behaviours;

import jade.content.OntoACLMessage;
import jade.core.AID;
import jade.core.Agent;
import jade.domain.FIPAException;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import java.util.logging.Level;

import logrus.agents.OSMessage;
import logrus.agents.OSMessageMatcher;
import logrus.agents.base.behaviours.OSBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.gui.queryDialogAgent.RDQLDialogAgent;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.IdentifierGenerator;
import logrus.tools.Utils;

/**
 * 
 */
public class PerformRDQLBehaviour extends OSBehaviour
{
	// RDQL dialog agent reference
	private RDQLDialogAgent agentRDQL;

	// conversation ID for whole process
	private String conversationID;

	// flag determining if this behaviour is finished
	private boolean finished;

	// current stage of behaviour
	private int stage;

	// index of stage responsible for sending crawl request to MA
	private static final int StageSendRDQLQuery = 0;

	// index of stage responsible for waiting for reply from MA
	private static final int StageWaitForReply = 1;

	// index of final stage of this behaviour
	private static final int StageFinal = 10;

	// error message
	private String strErrorMessage;

	// RDQL query string
	private String queryString;

	/**
	 * @param a
	 */
	public PerformRDQLBehaviour (Agent a, String queryString)
	{
		super(a);
		this.agentRDQL = (RDQLDialogAgent)a;
		this.conversationID = null;
		this.finished = false;
		this.stage = StageSendRDQLQuery;
		this.strErrorMessage = null;
		this.queryString = queryString;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		switch (this.stage)
		{
			case StageSendRDQLQuery:
				this.sendRDQLQuery();
				break;
			case StageWaitForReply:
				this.waitForReply();
				break;
			case StageFinal:
				this.finalizeBehaviour();
				break;
		}
	}

	private void waitForReply ()
	{
		// prepare accepted message templates
		OSMessageMatcher matcher = new OSMessageMatcher(this.osAgent
				.getContentManager());
		matcher.addAcceptedMsgCode(OSMessage.DBA_Execute_RDQL_Result);
		matcher.addAcceptedMsgCode(OSMessage.DBA_Execute_RDQL_Failed);
		matcher.setAcceptedConversationID(this.conversationID);
		MessageTemplate pattern = new MessageTemplate(matcher);

		// recive message
		OntoACLMessage jmsg = (OntoACLMessage)this.osAgent.receive(pattern);
		if (jmsg != null)
		{
			OSMessage msg = OSMessageMatcher.extractMessage(jmsg,this.osAgent
					.getContentManager());
			switch (msg.getMessageCode())
			{
				case OSMessage.DBA_Execute_RDQL_Result:
					this.agentRDQL.displayResults(msg.getAnnouncement());
					this.agentRDQL.displayMessage("Done RDQL query");
					break;
				case OSMessage.DBA_Execute_RDQL_Failed:
					this.agentRDQL
							.displayMessage("Error while performing RDQL Query: "
									+ msg.getAnnouncement() != null ? msg
									.getAnnouncement() : "");
					break;
			}
			this.stage = StageFinal;
		}
		else
		{
			// block behaviour until message arrives
			block();
		}
	}

	private void sendRDQLQuery ()
	{
		AID aidDBA = null;
		OSMessage msg = null;
		try
		{
			// find AID of DBA agent
			//aidDBA = this.osAgent.findAgentPerformingService(ServiceNames.ServiceDBA);
			aidDBA = Utils.getInstance().tossDBARead(this.osAgent);
			if (aidDBA == null)
			{
				this.strErrorMessage = "No DBA service available";
				this.stage = StageFinal;
			}

			// generate conversation ID
			this.conversationID = IdentifierGenerator.getInstance().GenerateID();

			msg = new OSMessage();
			msg.setMessageCode(OSMessage.DBA_Execute_RDQL);
			msg.setAnnouncement(this.queryString);
			this.agentRDQL.sendMessage(ACLMessage.REQUEST,msg,aidDBA,
					this.conversationID);
			this.stage = StageWaitForReply;
		}
		catch (FIPAException exc)
		{
			this.strErrorMessage = "Error while searching for DBA service "
					+ exc.getMessage();
			this.stage = StageFinal;
		}
		catch (IdentifierGenerationFailedException exc)
		{
			this.strErrorMessage = "Couldn't generate conversation ID "
					+ exc.getMessage();
			this.stage = StageFinal;
		}
		catch (SendingMessageFailed exc)
		{
			this.strErrorMessage = "Couldn't send DBA_Execute_RDQL to DBA "
					+ exc.getMessage();
			this.agentRDQL.onSendMessgeFailed(aidDBA,OSMessage.DBA_Execute_RDQL,
					exc,msg);
			this.stage = StageFinal;
		}
	}
	
	/**
	 * Perform final actions before finishing behaviour
	 */
	private void finalizeBehaviour ()
	{
		// if there were some errors then log them
		if (this.strErrorMessage != null)
		{
			this.agentRDQL.logg(Level.SEVERE,this.strErrorMessage);
			this.agentRDQL.displayMessage(this.strErrorMessage);
		}

		// mark behaviour as finished
		this.finished = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#done()
	 */
	public boolean done ()
	{
		return this.finished;
	}
}
