package logrus.agents.indexingAgent.behaviours;

import jade.core.Agent;
import logrus.agents.base.behaviours.OSCyclicBehaviour;
import logrus.agents.indexingAgent.IndexingAgent;
import logrus.agents.indexingAgent.IndexingTask;

/**
 * 
 */
public class SchedulerBehaviour extends OSCyclicBehaviour
{
	private IndexingAgent agentIA;

	/**
	 * @param a
	 */
	public SchedulerBehaviour (Agent a)
	{
		super(a);
		this.agentIA = (IndexingAgent)a;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		// check if there are some tasks to perform and some free slots
		if (this.agentIA.isTaskToPerform() && this.agentIA.isFreeSubBehaviour())
		{
			// calculate number of indexing processes that can be launched as lower value of number of
			// free slots for behaviours and number of tasks to be performed
			int nToLaunch = Math.min(this.agentIA.getNumberAwaitingTasks(),
					this.agentIA.getFreeSlots());
			for (int i = 0; i < nToLaunch; i++)
			{
				// take the oldest task from queue
				IndexingTask task = this.agentIA.dequeueTask();

				switch (task.getType())
				{
					case IndexingTask.InsertUpdateSingle:
						// mark slot as taken
						this.agentIA.decreaseFreeSlots();
						//	add appropriate behaviour responsible for performing indexing process
						this.agentIA.addBehaviour(new IndexingBehaviour(this.agentIA,task));
						break;
				}
			}
		}

		// block behaviour
		block();
	}
}
