package logrus.agents.indexingAgent.behaviours;

import jade.content.OntoACLMessage;
import jade.core.AID;
import jade.core.Agent;
import jade.lang.acl.MessageTemplate;
import jade.util.leap.ArrayList;
import jade.util.leap.Iterator;
import logrus.agents.OSMessage;
import logrus.agents.OSMessageMatcher;
import logrus.agents.base.behaviours.OSCyclicBehaviour;
import logrus.agents.indexingAgent.IndexingAgent;
import logrus.agents.indexingAgent.IndexingTask;
import logrus.ontmodel.QuadsList;

/**
 * Behaviour performing serivce requests listening for Indexing Agent
 */
public class ServiceIABehaviour extends OSCyclicBehaviour
{
	private IndexingAgent agentIA;

	/**
	 * @param a
	 */
	public ServiceIABehaviour (Agent a)
	{
		super(a);
		this.agentIA = (IndexingAgent)a;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		// prepare accepted message templates
		OSMessageMatcher matcher = new OSMessageMatcher(this.osAgent
				.getContentManager());
		matcher.addAcceptedMsgCode(OSMessage.IA_Mass_Index);
		MessageTemplate pattern = new MessageTemplate(matcher);

		// recive message
		OntoACLMessage jmsg = (OntoACLMessage)this.osAgent.receive(pattern);
		if (jmsg != null)
		{
			// extract information form message
			AID aidClient = jmsg.getSender();
			String conversationID = jmsg.getConversationId();
			OSMessage msg = OSMessageMatcher.extractMessage(jmsg,this.osAgent
					.getContentManager());

			// check which operation is requested and perform it
			switch (msg.getMessageCode())
			{
				case OSMessage.IA_Mass_Index:
					this.onIA_Mass_Index(msg,aidClient,conversationID);
					break;
				default:
					this.osAgent.onUnexpectedMessage("IA",msg,aidClient);
					return;					
			}
		}
		else
		{
			block();
		}
	}

	private void onIA_Mass_Index (OSMessage msg, AID aidClient,
			String conversationID)
	{
		// generate and enqueue all indexing tasks
		ArrayList objectsCollection = msg.getObjectsCollection();
		for (Iterator iter = objectsCollection.iterator(); iter.hasNext();)
		{
			IndexingTask task = new IndexingTask(IndexingTask.InsertUpdateSingle,aidClient,conversationID);
			task.setObjIncl((QuadsList)iter.next());

			this.agentIA.enqueueTask(task);
		}
		
		// wait scheduler behaviour
		this.agentIA.wakeTasksScheduler();
	}
}
