package logrus.agents.searchCoordinatorAgent;

import jade.content.Concept;
import jade.content.ContentElement;
import logrus.agents.datasources.DataProviderID;

public class SearchTask implements ContentElement, Concept
{
	// extract information about object form URL specified by urlToParse field
	public static final int EXTRACT_INFO = 1;

	// extract links form specified page
	public static final int EXTRACT_LINKS = 2;

	// (mandatory) searchType of search to be performed
	private int searchType;

	// (mandatory) ID of data source which should be searched
	private String dataSource;

	// (optional) URL to be parsed
	private String urlToParse;

	// (optional) Special number of conversationID that should be used for exchanging messages
	// considering this search request
	private String conversationID;

	// (optional) Special data for searcher
	private String commandOptions;

	public SearchTask ()
	{
		super();
	}

	public String toString ()
	{
		StringBuffer buff = new StringBuffer();
		if (this.searchType == SearchTask.EXTRACT_INFO)
		{
			buff.append("Extract info from ");
		}
		else if (this.searchType == SearchTask.EXTRACT_LINKS)
		{
			buff.append("Extract links, start from ");
		}

		if (DataProviderID.CHEFMOZ.compareTo(this.dataSource) == 0)
		{
			buff.append("CHEFMOZ, webpage ");
		}
		else if (DataProviderID.RESTCOM.compareTo(this.dataSource) == 0)
		{
			buff.append("restauracje.com, webpage ");
		}
		buff.append(this.urlToParse);
		buff.append('\n');

		if (this.commandOptions != null)
		{
			buff.append("Command options: ");
			buff.append(this.commandOptions);
			buff.append('\n');
		}
		
//		if (this.conversationID != null)
//		{
//			buff.append("Conversation ID: ");
//			buff.append(this.conversationID);
//			buff.append('\n');
//		}		
		return buff.toString();
	}

	public SearchTask (int type, String dataSource, String urlToParse)
	{
		super();
		this.searchType = type;
		this.dataSource = dataSource;
		this.urlToParse = urlToParse;
	}

	/**
	 * @param searchType
	 *           The searchType to set.
	 */
	public void setSearchType (int type)
	{
		this.searchType = type;
	}

	/**
	 * @return Returns the searchType.
	 */
	public int getSearchType ()
	{
		return this.searchType;
	}

	/**
	 * @param dataSource
	 *           The dataSource to set.
	 */
	public void setDataSource (String dataSource)
	{
		this.dataSource = dataSource;
	}

	/**
	 * @return Returns the dataSource.
	 */
	public String getDataSource ()
	{
		return this.dataSource;
	}

	/**
	 * @param urlToParse
	 *           The urlToParse to set.
	 */
	public void setUrlToParse (String url)
	{
		this.urlToParse = url;
	}

	/**
	 * @return Returns the urlToParse.
	 */
	public String getUrlToParse ()
	{
		return this.urlToParse;
	}

	/**
	 * @param commandOptions
	 *           The commandOptions to set.
	 */
	public void setCommandOptions (String command)
	{
		this.commandOptions = command;
	}

	/**
	 * @return Returns the commandOptions.
	 */
	public String getCommandOptions ()
	{
		return this.commandOptions;
	}

	/**
	 * @param conversationID
	 *           The conversationID to set.
	 */
	public void setConversationID (String conversationID)
	{
		this.conversationID = conversationID;
	}

	/**
	 * @return Returns the conversationID.
	 */
	public String getConversationID ()
	{
		return this.conversationID;
	}
}
