package logrus.agents.searchCoordinatorAgent.behaviours;

import java.util.logging.Level;

import jade.content.OntoACLMessage;
import jade.core.AID;
import jade.core.Agent;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import logrus.agents.OSMessage;
import logrus.agents.OSMessageMatcher;
import logrus.agents.base.behaviours.OSBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.searchCoordinatorAgent.SearchCoordinatorAgent;

public class ExtractObjectInfoSupervisonBehaviour extends OSBehaviour
{
	// behaviour should listen for messages wuth this conversation ID
	private String conversationID;

	// AID of agent that commisoned supervised search
	private AID aidClient;

	// SCA agent reference
	private SearchCoordinatorAgent agentSCA;

	// flag determining if behaviour should be considered as finished
	private boolean endFlag;

	public ExtractObjectInfoSupervisonBehaviour (Agent a, AID aidClient,
			String conversationID)
	{
		super(a);
		this.conversationID = conversationID;
		this.aidClient = aidClient;
		this.agentSCA = (SearchCoordinatorAgent)a;
		this.endFlag = false;
	}

	public void action ()
	{
		// prepare accepted message templates
		OSMessageMatcher matcher = new OSMessageMatcher(this.osAgent
				.getContentManager());
		matcher.addAcceptedMsgCode(OSMessage.WA_Extract_Object_Info_Result);
		matcher.addAcceptedMsgCode(OSMessage.WA_Extract_Object_Info_Failed);
		matcher.setAcceptedConversationID(this.conversationID);
		MessageTemplate pattern = new MessageTemplate(matcher);

		// receive message
		OntoACLMessage jmsg = (OntoACLMessage)this.osAgent.receive(pattern);
		if (jmsg != null)
		{
			OSMessage msg = OSMessageMatcher.extractMessage(jmsg,this.osAgent
					.getContentManager());
			switch (msg.getMessageCode())
			{
				case OSMessage.WA_Extract_Object_Info_Result:
					// send result back to the agent that has commissioned this search
					OSMessage replyOK = new OSMessage();
					replyOK.setMessageCode(OSMessage.SCA_Extract_Object_Info_Result);
					replyOK.setObjectInfo(msg.getObjectInfo());
					try
					{
						this.agentSCA.sendMessage(ACLMessage.INFORM,replyOK,
								this.aidClient,this.conversationID);
					}
					catch (SendingMessageFailed exc)
					{
						this.agentSCA.onSendMessgeFailed(this.aidClient,
								OSMessage.SCA_Extract_Object_Info_Result,exc,replyOK);
					}
					break;
				case OSMessage.WA_Extract_Object_Info_Failed:
					// send information about failure back to the agent that has commissioned this search
					OSMessage replyFailure = new OSMessage();
					replyFailure
							.setMessageCode(OSMessage.SCA_Extract_Object_Info_Failed);
					try
					{
						this.agentSCA.sendMessage(ACLMessage.INFORM,replyFailure,
								this.aidClient,this.conversationID);
					}
					catch (SendingMessageFailed exc)
					{
						this.agentSCA.onSendMessgeFailed(this.aidClient,
								OSMessage.SCA_Extract_Object_Info_Result,exc,
								replyFailure);
					}
					break;
				default:
					this.agentSCA.logg(Level.WARNING,"Unknown message code "
							+ msg.getMessageCode() + " from" + jmsg.getSender());
					break;
			}
			// whatever has happend register WA as a free one
			this.agentSCA.registerFreeAgent(jmsg.getSender());
			// finish behaviour
			this.endFlag = true;
		}
		else
		{
			// no message from appropriate WA so switch to inactive mode
			this.block();
		}
	}

	public boolean done ()
	{
		return this.endFlag;
	}

}
