package logrus.agents.searchCoordinatorAgent.behaviours;

import jade.content.OntoACLMessage;
import jade.core.AID;
import jade.core.Agent;
import jade.lang.acl.MessageTemplate;

import java.util.logging.Level;

import logrus.agents.OSMessage;
import logrus.agents.OSMessageMatcher;
import logrus.agents.base.behaviours.OSCyclicBehaviour;
import logrus.agents.searchCoordinatorAgent.SearchCoordinatorAgent;
import logrus.agents.searchCoordinatorAgent.SearchTask;

public class ServiceSCABehaviour extends OSCyclicBehaviour
{
	private SearchCoordinatorAgent agentSCA;

	public ServiceSCABehaviour (Agent a)
	{
		super(a);
		this.agentSCA = (SearchCoordinatorAgent)a;
	}

	public void action ()
	{
		// prepare accepted message templates
		OSMessageMatcher matcher = new OSMessageMatcher(this.osAgent
				.getContentManager());
		matcher.addAcceptedMsgCode(OSMessage.SCA_Extract_Object_Info);
		matcher.addAcceptedMsgCode(OSMessage.SCA_Crawl);
		MessageTemplate pattern = new MessageTemplate(matcher);

		// recive message
		OntoACLMessage jmsg = (OntoACLMessage)this.osAgent.receive(pattern);
		if (jmsg != null)
		{
			// extract information form message
			AID aidClient = jmsg.getSender();
			String conversationID = jmsg.getConversationId();
			OSMessage msg = OSMessageMatcher.extractMessage(jmsg,this.osAgent
					.getContentManager());

			// check which operation is requested and perform it
			switch (msg.getMessageCode())
			{
				case OSMessage.SCA_Extract_Object_Info:
					this.onSCA_Extract_Object_Info(msg,aidClient,conversationID);
					break;
				case OSMessage.SCA_Crawl:
					this.onSCA_Crawl(msg,aidClient,conversationID);
					break;
				default:
					this.osAgent.onUnexpectedMessage("SCA",msg,aidClient);
					return;
			}
		}
		else
		{
			block();
		}
	}

	private void onSCA_Crawl (OSMessage msg, AID aidClient, String conversationID)
	{
		// check if there really is search task sent in message
		SearchTask task = msg.getSearchCommand();
		if (task == null)
		{
			// error: there is on command for search in message - log message error and stop
			this.agentSCA.logg(Level.WARNING,
					"Empty SCA_Crawl message received from " + aidClient);
			return;
		}

		// add behaviour responsible for supervison on search process
		this.agentSCA.addBehaviour(new CrawlSupervisionBehaviour(this.agentSCA,
				aidClient,conversationID,task.getDataSource()));

		// place search task in queue
		this.agentSCA.enqueueSearchTask(task);

		// wake scheduler behaviour
		this.agentSCA.getTasksScheduler().restart();
	}

	private void onSCA_Extract_Object_Info (OSMessage msg, AID aidClient,
			String conversationID)
	{
		// check if there really is search task sent in message
		SearchTask task = msg.getSearchCommand();
		if (task == null)
		{
			// error: there is on command for search in message - log message error and stop
			this.agentSCA.logg(Level.WARNING,
					"Empty SCA_Extract_Object_Info message received from "
							+ aidClient);
			return;
		}

		// add behaviour responsible for supervison on search process
		this.agentSCA.addBehaviour(new ExtractObjectInfoSupervisonBehaviour(
				this.agentSCA,aidClient,conversationID));

		// place search task in queue
		this.agentSCA.enqueueSearchTask(task);

		// wake scheduler behaviour
		this.agentSCA.getTasksScheduler().restart();
	}
}
