package logrus.agents.searchCoordinatorAgent.behaviours;

import jade.core.AID;
import jade.core.Agent;
import jade.lang.acl.ACLMessage;
import logrus.agents.OSMessage;
import logrus.agents.base.behaviours.OSCyclicBehaviour;
import logrus.agents.base.exceptions.SendingMessageFailed;
import logrus.agents.searchCoordinatorAgent.SearchCoordinatorAgent;
import logrus.agents.searchCoordinatorAgent.SearchTask;

/**
 * 
 */
public class TasksSchedulerBehaviour extends OSCyclicBehaviour
{
	private SearchCoordinatorAgent agentSCA;

	/**
	 * @param a
	 */
	public TasksSchedulerBehaviour (Agent a)
	{
		super(a);
		this.agentSCA = (SearchCoordinatorAgent)a;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	public void action ()
	{
		if (this.agentSCA.isSearchToPerform() && this.agentSCA.isFreeAgent())
		{
			// calculate the number of agents that can be activated as a lower number form number of
			// free agents and number of tasks awaiting in queue
			int nToLaunch = Math.min(this.agentSCA.getNumberOfFreeAgents(),
					this.agentSCA.getNumberOfTasks());
			for (int i = 0; i < nToLaunch; i++)
			{
				// take the oldest task from queue
				SearchTask task = this.agentSCA.dequeueSearchTask();

				// take first free agent
				AID aidFreeWA = this.agentSCA.getFirstFreeAgent();

				// prepare message to WA
				OSMessage msg = new OSMessage();
				msg.setSearchCommand(task);
				switch (task.getSearchType())
				{
					case SearchTask.EXTRACT_INFO:
						msg.setMessageCode(OSMessage.WA_Extract_Object_Info);
						break;
					case SearchTask.EXTRACT_LINKS:
						msg.setMessageCode(OSMessage.WA_Crawl);
						break;
					default:
						break;
				}

				// try to send message to WA
				try
				{
					this.agentSCA.sendMessage(ACLMessage.REQUEST,msg,aidFreeWA,task
							.getConversationID());
				}
				catch (SendingMessageFailed exc)
				{
					// sending message failed so mark agent as free again, discard task and log error
					// information
					this.agentSCA.markAgentAsFree(aidFreeWA);
					this.agentSCA.onSendMessgeFailed(aidFreeWA,msg.getMessageCode(),
							exc,msg);
				}
			}
		}

		// no tasks to initiate or all available agents are busy so go to sleep
		this.block(1000);
//		System.out.println("Free agents: "
//				+ this.agentSCA.getNumberOfFreeAgents() + " | Tasks: "
//				+ this.agentSCA.getNumberOfTasks());
	}
}
