package logrus.gui;

import logrus.gui.tools.ImagesManager;
import logrus.tools.Utils;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

/**
 * General wrapper for SWT GUI object (window, dialog, etc.)
 */
public class GUIObject
{
	// display used by shell
	protected Display display = null;

	// dialog shell
	protected Shell shell = null;

	// icons manager
	protected ImagesManager manger;

	/**
	 * @param parent
	 *           Display used by shell
	 */
	public GUIObject (Display parent)
	{
		super();
		this.display = parent;
		this.manger = ImagesManager.getInstance();
	}

	/**
	 * Run GUI object - start messages loop
	 */
	public void runGUI ()
	{
		this.shell.open();
		while (!this.shell.isDisposed())
		{
			if (!this.getDisplay().readAndDispatch())
			{
				this.getDisplay().sleep();
			}
		}
	}

	/**
	 * Clean allocated resources
	 */
	public void clean ()
	{
		this.shell.dispose();
	}

	/**
	 * @return Returns the display.
	 */
	public Display getDisplay ()
	{
		return this.display;
	}

	/**
	 * @return Returns the display.
	 */
	public Shell getShell ()
	{
		return this.shell;
	}
	
	public void displayMessage (String strMessage)
	{
		Utils.getInstance().showInfoMessage(strMessage,this.shell);
	}	
}
