package logrus.gui;

import jade.content.onto.OntologyException;
import jade.wrapper.StaleProxyException;
import logrus.AgentsPlatform;
import logrus.agents.AgentNames;
import logrus.agents.datasources.DataProviderID;
import logrus.agents.datasources.chefmoz.ChefMozPageTypes;
import logrus.agents.datasources.restauracjecom.RestComPageTypes;
import logrus.agents.gui.mainDialogAgent.MainDialogAgent;
import logrus.agents.gui.mainDialogAgent.MainDialogAgentExternal;
import logrus.ontmodel.Prefixes;
import logrus.tools.Utils;
import logrus.tools.XMLAnalysisException;
import logrus.tools.logging.InitializeLoggerFailedException;
import logrus.tools.settings.SettingsManager;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

/**
 * Main GUI of OntoStorage manager
 */
public class ManagerGUI extends GUIObject
{
	/**
	 * Stores functionality codes
	 */
	private interface GUIFunctionality
	{
		public static final int SYSTEM_RUN_PLATFORM = 1;

		public static final int SYSTEM_STOP_PLATFORM = 2;

		public static final int TOOLS_RDQL_QUERY = 101;
	}

	private interface ToolTipsStrings
	{
		public static final String STR_SYSTEM_RUN_PLATFORM = "Create new JADE container and run OntoStorage";

		public static final String STR_SYSTEM_STOP_PLATFORM = "Stop OntoStorage and JADE container";

		public static final String STR_TOOLS_RDQL_QUERY = "Open RDQL quering dialog";

		public static final String STR_TOOLS_SAVE_MODEL = "Save model to RDF file";

		public static final String STR_TOOLS_REPORT = "Build report about objects in model";

		public static final String STR_TOOLS_UPDATE = "Initiate data update process";
	}

	// menu-related fields
	// main menu of application
	private Menu mainMenu;

	// file submenu
	private MenuItem fileGroup;

	private Menu fileMenu;

	private MenuItem miFile_Exit; // File -> Exit

	// system submenu
	private MenuItem systemGroup;

	private Menu systemMenu;

	private MenuItem miSystem_RunPlatform; // System -> Run platform

	private MenuItem miSystem_StopPlatform; // System -> Stop platform

	// tools submenu
	private MenuItem toolsGroup;

	private Menu toolsMenu;

	private MenuItem miTools_RDQLQuery; // Tools -> RDQL Query...

	private MenuItem miTools_SaveModel; // Tools -> Save model...

	private MenuItem miTools_Report; // Tools -> Report

	private MenuItem miTools_Update; // Tools -> Update process

	// actions submenu
	private MenuItem actionsGroup;

	private Menu actionsMenu;

	private MenuItem miActions_Crawl; // Actions -> Crawl item

	private Menu actionsCrawlSubmenu; // Actions -> Crawl submenu

	private MenuItem miActions_Crawl_Warsaw; // Actions -> Crawl -> Get data about Warsaw

	private MenuItem miActions_Crawl_Cracow; // Actions -> Crawl -> Get data about Cracow

	// help submenu
	private MenuItem helpGroup;

	private Menu helpMenu;

	// toolbar-related fields
	private CoolBar toolbar;

	// buttons related to menu labeled "System"
	private CoolItem barSystemItem;

	private ToolBar tbSystem;

	private ToolItem tiSystem_RunPlatform; // System -> Run platform

	private ToolItem tiSystem_StopPlatform; // System -> Stop platform

	// buttons related to menu labeled "Tools"
	private CoolItem barToolsItem;

	private ToolBar barTools;

	private ToolItem tiTools_SaveModel; // Tools -> Save model...

	private ToolItem tiTools_RDQLQuery; // Tools -> RDQL Query...

	private ToolItem tiTools_Report; // Tools -> Build Report

	private ToolItem tiTools_Update; // Tools ->Update Process

	// ---------------------------------------------------------

	// wrapper for JADE agents platform
	private AgentsPlatform platform = null;

	private MainDialogAgentExternal agentGUI = null;

	/**
	 * Constructor
	 */
	public ManagerGUI (Display parent)
	{
		super(parent);
		SettingsManager sManager = SettingsManager.getInstance();
		try
		{
			sManager.loadSettings(Utils.getInstance().getCWD()
					+ "\\xml\\Settings.xml");
		}
		catch (XMLAnalysisException e)
		{
			System.out.println(e.getMessage());
			sManager.setSettings(sManager.getDefaultSettings());
		}
		this.display = parent;
		this.init();
	}

	public void runGUI ()
	{
		this.shell.open();
		this.onSystem_RunPlatform(null);
		while (!this.shell.isDisposed())
		{
			if (!this.getDisplay().readAndDispatch())
			{
				this.getDisplay().sleep();
			}
		}
	}

	/**
	 * Init GUI
	 */
	public void init ()
	{
		this.createShell();
		this.createMenu();
		this.createToolBar();
		this.setGUIStateNotRunning();
		this.shell.addListener(SWT.Close,new Listener()
		{
			public void handleEvent (Event e)
			{
				onClose(e);
			}
		});
	}

	private void createShell ()
	{
		this.shell = new Shell(this.getDisplay());
		this.shell.setText("OntoStorage manager");
		this.shell.setSize(800,600);
	}

	private void createMenu ()
	{
		this.mainMenu = new Menu(this.shell,SWT.BAR);

		// file group
		this.fileGroup = new MenuItem(this.mainMenu,SWT.CASCADE);
		this.fileGroup.setText("File");
		this.fileMenu = new Menu(this.shell,SWT.DROP_DOWN);
		this.fileGroup.setMenu(this.fileMenu);
		// file menu items

		// File -> Exit
		this.miFile_Exit = new MenuItem(this.fileMenu,SWT.PUSH);
		this.miFile_Exit.setText("Exit");
		this.miFile_Exit.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onFile_Exit(e);
			}
		});

		// system group
		this.systemGroup = new MenuItem(this.mainMenu,SWT.CASCADE);
		this.systemGroup.setText("System");
		this.systemMenu = new Menu(this.shell,SWT.DROP_DOWN);
		this.systemGroup.setMenu(this.systemMenu);
		// system menu items

		// System -> Run platform
		this.miSystem_RunPlatform = new MenuItem(this.systemMenu,SWT.PUSH);
		this.miSystem_RunPlatform.setText("Run platform");
		this.miSystem_RunPlatform.setImage(this.manger.getImage(
				"System-RunPlatform",this.getDisplay()));
		this.miSystem_RunPlatform.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onSystem_RunPlatform(e);
			}
		});

		// System -> Stop platform
		this.miSystem_StopPlatform = new MenuItem(this.systemMenu,SWT.PUSH);
		this.miSystem_StopPlatform.setText("Stop platform");
		this.miSystem_StopPlatform.setImage(this.manger.getImage(
				"System-StopPlatform",this.getDisplay()));
		this.miSystem_StopPlatform.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onSystem_StopPlatform(e);
			}
		});

		// tools group
		this.toolsGroup = new MenuItem(this.mainMenu,SWT.CASCADE);
		this.toolsGroup.setText("Tools");
		this.toolsMenu = new Menu(this.shell,SWT.DROP_DOWN);
		this.toolsGroup.setMenu(this.toolsMenu);

		// Tools -> RDQL Query...
		this.miTools_RDQLQuery = new MenuItem(this.toolsMenu,SWT.PUSH);
		this.miTools_RDQLQuery.setText("RDQL Query...");
		this.miTools_RDQLQuery.setImage(this.manger.getImage("Tools-RDQLQuery",
				this.getDisplay()));
		this.miTools_RDQLQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onTools_RDQLQuery(e);
			}
		});

		// Tools -> Save model...
		this.miTools_SaveModel = new MenuItem(this.toolsMenu,SWT.PUSH);
		this.miTools_SaveModel.setText("Save model...");
		this.miTools_SaveModel.setImage(this.manger.getImage(
				"RDQLTool-SaveResults",this.getDisplay()));
		this.miTools_SaveModel.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onTools_SaveModel(e);
			}
		});

		// Tools -> Report...
		this.miTools_Report = new MenuItem(this.toolsMenu,SWT.PUSH);
		this.miTools_Report.setText("Send build report request");
		this.miTools_Report.setImage(this.manger.getImage("Tools-Report",this
				.getDisplay()));
		this.miTools_Report.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onTools_Report(e);
			}
		});

		// Tools -> Initiate update process
		this.miTools_Update = new MenuItem(this.toolsMenu,SWT.PUSH);
		this.miTools_Update.setText("Initiate update process");
		this.miTools_Update.setImage(this.manger.getImage("Tools-UpdateProcess",
				this.getDisplay()));
		this.miTools_Update.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onTools_Update(e);
			}
		});

		// actions group
		this.actionsGroup = new MenuItem(this.mainMenu,SWT.CASCADE);
		this.actionsGroup.setText("Actions");
		this.actionsMenu = new Menu(this.shell,SWT.DROP_DOWN);
		this.actionsGroup.setMenu(this.actionsMenu);
		// Actions -> Crawl submenu
		this.miActions_Crawl = new MenuItem(this.actionsMenu,SWT.CASCADE);
		this.miActions_Crawl.setText("Crawl...");
		this.actionsCrawlSubmenu = new Menu(this.shell,SWT.DROP_DOWN);
		this.miActions_Crawl.setMenu(this.actionsCrawlSubmenu);

		// Actions -> Crawl... -> Get data about Warsaw
		this.miActions_Crawl_Warsaw = new MenuItem(this.actionsCrawlSubmenu,
				SWT.PUSH);
		this.miActions_Crawl_Warsaw.setText("Get data about Warsaw");
		this.miActions_Crawl_Warsaw.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onActions_Crawl_Warsaw(e);
			}
		});

		// Actions -> Crawl... -> Get data about Cracow
		this.miActions_Crawl_Cracow = new MenuItem(this.actionsCrawlSubmenu,
				SWT.PUSH);
		this.miActions_Crawl_Cracow.setText("Get data about Cracow");
		this.miActions_Crawl_Cracow.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onActions_Crawl_Cracow(e);
			}
		});

		// help group
		this.helpGroup = new MenuItem(this.mainMenu,SWT.CASCADE);
		this.helpGroup.setText("Help");
		this.helpMenu = new Menu(this.shell,SWT.DROP_DOWN);
		this.helpGroup.setMenu(this.helpMenu);

		// attach menu to the shell
		this.shell.setMenuBar(this.mainMenu);
	}

	private void createToolBar ()
	{
		Point itemSize;
		this.toolbar = new CoolBar(this.shell,SWT.FLAT);

		// buttons related to menu labeled "System"
		this.barSystemItem = new CoolItem(this.toolbar,SWT.NONE);
		this.tbSystem = new ToolBar(this.toolbar,SWT.FLAT);
		// System -> Run platform
		this.tiSystem_RunPlatform = new ToolItem(this.tbSystem,SWT.FLAT);
		this.tiSystem_RunPlatform.setImage(this.manger.getImage(
				"System-RunPlatform",this.getDisplay()));
		this.tiSystem_RunPlatform
				.setToolTipText(ToolTipsStrings.STR_SYSTEM_RUN_PLATFORM);
		this.tiSystem_RunPlatform.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onSystem_RunPlatform(e);
			}
		});
		// System -> Stop platform
		this.tiSystem_StopPlatform = new ToolItem(this.tbSystem,SWT.FLAT);
		this.tiSystem_StopPlatform.setImage(this.manger.getImage(
				"System-StopPlatform",this.getDisplay()));
		this.tiSystem_StopPlatform
				.setToolTipText(ToolTipsStrings.STR_SYSTEM_STOP_PLATFORM);
		this.tiSystem_StopPlatform.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onSystem_StopPlatform(e);
			}
		});
		// finish creating "System" toolbar
		this.tbSystem.pack();
		itemSize = this.tbSystem.getSize();
		this.barSystemItem.setControl(this.tbSystem);
		this.barSystemItem.setSize(this.barSystemItem.computeSize(itemSize.x,
				itemSize.y));
		this.barSystemItem.setMinimumSize(itemSize);

		// buttons related to menu labeled "Tools"
		this.barToolsItem = new CoolItem(this.toolbar,SWT.NONE);
		ToolBar tbTools = new ToolBar(this.toolbar,SWT.FLAT);

		// Tools -> RDQL Query
		this.tiTools_RDQLQuery = new ToolItem(tbTools,SWT.FLAT);
		this.tiTools_RDQLQuery.setImage(this.manger.getImage("Tools-RDQLQuery",
				this.getDisplay()));
		this.tiTools_RDQLQuery
				.setToolTipText(ToolTipsStrings.STR_TOOLS_RDQL_QUERY);
		this.tiTools_RDQLQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onTools_RDQLQuery(e);
			}
		});

		// Tools -> Save model...
		this.tiTools_SaveModel = new ToolItem(tbTools,SWT.FLAT);
		this.tiTools_SaveModel.setImage(this.manger.getImage(
				"RDQLTool-SaveResults",this.getDisplay()));
		this.tiTools_SaveModel
				.setToolTipText(ToolTipsStrings.STR_TOOLS_SAVE_MODEL);
		this.tiTools_SaveModel.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onTools_SaveModel(e);
			}
		});

		// Tools -> Build report
		this.tiTools_Report = new ToolItem(tbTools,SWT.FLAT);
		this.tiTools_Report.setImage(this.manger.getImage("Tools-Report",this
				.getDisplay()));
		this.tiTools_Report.setToolTipText(ToolTipsStrings.STR_TOOLS_REPORT);
		this.tiTools_Report.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onTools_Report(e);
			}
		});

		// Tools -> Update
		this.tiTools_Update = new ToolItem(tbTools,SWT.FLAT);
		this.tiTools_Update.setImage(this.manger.getImage("Tools-UpdateProcess",
				this.getDisplay()));
		this.tiTools_Update.setToolTipText(ToolTipsStrings.STR_TOOLS_UPDATE);
		this.tiTools_Update.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onTools_Update(e);
			}
		});

		// finish creating "Tools" toolbar
		tbTools.pack();
		itemSize = tbTools.getSize();
		this.barToolsItem.setControl(tbTools);
		this.barToolsItem.setSize(this.barToolsItem.computeSize(itemSize.x,
				itemSize.y));
		this.barToolsItem.setMinimumSize(itemSize);

		// finish creating toolbar
		this.toolbar.pack();
	}

	/**
	 * Set initial GUI elements states
	 */
	private void setGUIStateNotRunning ()
	{

		this.miActions_Crawl.setEnabled(false);

		this.miTools_RDQLQuery.setEnabled(false);
		this.tiTools_RDQLQuery.setEnabled(false);

		this.miTools_Report.setEnabled(false);
		this.tiTools_Report.setEnabled(false);

		this.miTools_SaveModel.setEnabled(false);
		this.tiTools_SaveModel.setEnabled(false);

		this.miTools_Update.setEnabled(false);
		this.tiTools_Update.setEnabled(false);

		this.miSystem_RunPlatform.setEnabled(false);
		this.tiSystem_RunPlatform.setEnabled(false);

		this.miSystem_StopPlatform.setEnabled(false);
		this.tiSystem_StopPlatform.setEnabled(false);
	}

	/**
	 * Set initial GUI elements states
	 */
	private void setGUIStateBooting ()
	{
		this.miActions_Crawl.setEnabled(false);

		this.miTools_RDQLQuery.setEnabled(false);
		this.tiTools_RDQLQuery.setEnabled(false);

		this.miTools_Report.setEnabled(false);
		this.tiTools_Report.setEnabled(false);

		this.miTools_SaveModel.setEnabled(false);
		this.tiTools_SaveModel.setEnabled(false);

		this.miTools_Update.setEnabled(false);
		this.tiTools_Update.setEnabled(false);

		this.miSystem_RunPlatform.setEnabled(false);
		this.tiSystem_RunPlatform.setEnabled(false);

		this.miSystem_StopPlatform.setEnabled(true);
		this.tiSystem_StopPlatform.setEnabled(true);
	}

	/**
	 * Set initial GUI elements states
	 */
	private void setGUIStateRunning ()
	{

		this.miActions_Crawl.setEnabled(true);

		this.miTools_RDQLQuery.setEnabled(true);
		this.tiTools_RDQLQuery.setEnabled(true);

		this.miTools_Report.setEnabled(true);
		this.tiTools_Report.setEnabled(true);

		this.miTools_SaveModel.setEnabled(true);
		this.tiTools_SaveModel.setEnabled(true);

		this.miTools_Update.setEnabled(true);
		this.tiTools_Update.setEnabled(true);

		this.miSystem_RunPlatform.setEnabled(false);
		this.tiSystem_RunPlatform.setEnabled(false);

		this.miSystem_StopPlatform.setEnabled(true);
		this.tiSystem_StopPlatform.setEnabled(true);
	}

	/**
	 * Process request: close shell
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onFile_Exit (Event e)
	{
		this.onSystem_StopPlatform(e);
		this.shell.close();
	}

	/**
	 * Process request: stop OntoStorage
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onSystem_StopPlatform (Event e)
	{
		if (this.platform != null)
		{
			try
			{
				this.platform.shutdownPlatform();
				this.platform = null;
				this.setGUIStateNotRunning();
				Utils.getInstance().showInfoMessage("OntoStorage stopped",
						this.shell);
			}
			catch (StaleProxyException exc)
			{
				Utils.getInstance().showExceptionInfo(exc,this.shell);
			}
		}
	}

	/**
	 * Process request: startOntoStorage
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onSystem_RunPlatform (Event e)
	{
		if (this.platform == null)
		{
			this.platform = AgentsPlatform.getInstance();
			try
			{
				this.setGUIStateBooting();
				this.platform.runPlatform(this);
				MainDialogAgent agent = new MainDialogAgent(this);
				AgentsPlatform.getInstance()
						.addAgent(agent,AgentNames.MainGUIAgent);
				this.agentGUI = agent;

				// get filename of RDF file from standard FileDialog
				FileDialog fileDialog = new FileDialog(this.shell,SWT.OPEN);
				fileDialog.setText("Load RDF model from file");
				String[] filterExt = { "*.rdf" };
				fileDialog.setFilterExtensions(filterExt);
				String filePath = fileDialog.open();
				if (filePath == null || filePath.length() == 0)
				{
					filePath = Utils.getInstance().getCWD().replace('\\','/')
							+ "databases/default.rdf";
				}
				this.agentGUI.bootSystem(filePath);
			}
			catch (StaleProxyException exc)
			{
				Utils.getInstance().showExceptionInfo(exc,this.shell);
			}
			catch (OntologyException exc)
			{
				Utils.getInstance().showExceptionInfo(exc,this.shell);
			}
			catch (InitializeLoggerFailedException exc)
			{
				Utils.getInstance().showExceptionInfo(exc,this.shell);
			}
		}
	}

	/**
	 * Process request: display RDQL query dialog
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onTools_RDQLQuery (Event e)
	{
		RDQLQueryDialog queryDialog = new RDQLQueryDialog(this.getDisplay());
		queryDialog.runGUI();
	}

	protected void onActions_Crawl_Warsaw (Event e)
	{

		this.agentGUI.sendCrawlMessage(DataProviderID.CHEFMOZ,
				"http://chefmoz.org/Poland/MA/Warsaw/",ChefMozPageTypes.CityPage);

		this.agentGUI.sendCrawlMessage(DataProviderID.RESTCOM,
				"http://www.restauracje.com/search.php3?&F_miasto=Warszawa",
				RestComPageTypes.ListPage);
	}

	protected void onActions_Crawl_Cracow (Event e)
	{

		this.agentGUI.sendCrawlMessage(DataProviderID.CHEFMOZ,
				"http://chefmoz.org/Poland/MP/Cracow/",ChefMozPageTypes.CityPage);
		this.agentGUI.sendCrawlMessage(DataProviderID.RESTCOM,
				"http://www.restauracje.com/search.php3?&F_miasto=Krak%F3w",
				RestComPageTypes.ListPage);
//		this.agentGUI.sendCrawlMessage(DataProviderID.CHEFMOZ,
//				"http://chefmoz.org/Poland/ZP/Rewal/",ChefMozPageTypes.CityPage);
	}

	public void onBootFailed ()
	{
		this.onSystem_StopPlatform(null);
	}

	public void onBootSuccessful ()
	{
		this.setGUIStateRunning();
		Utils.getInstance().showInfoMessage("Boot successful",this.shell);
	}

	public void onTools_SaveModel (Event e)
	{
		// get filename of RDF file from standard FileDialog
		FileDialog fileDialog = new FileDialog(this.shell,SWT.SAVE);
		fileDialog.setText("Save model to the RDF file");
		String[] filterExt = { "*.rdf" };
		fileDialog.setFilterExtensions(filterExt);
		String filePath = fileDialog.open();
		if (filePath == null || filePath.length() == 0)
		{
			return;
		}
		if (!filePath.endsWith(".rdf"))
		{
			filePath = Utils.getInstance().strAppendExtension(filePath,"rdf");
		}
		this.agentGUI.saveModel(filePath);
	}

	public void onTools_Update (Event e)
	{
		this.agentGUI.sendUpdateRequest();
	}

	public void onClose (Event e)
	{
		this.onFile_Exit(e);
	}

	void onTools_Report (Event e)
	{
		this.agentGUI.sendBuildReportRequest(Prefixes.CLASS_RES);
	}
}
