package logrus.gui;

import jade.content.onto.OntologyException;
import jade.wrapper.StaleProxyException;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;

import logrus.AgentsPlatform;
import logrus.agents.gui.queryDialogAgent.RDQLDialogAgent;
import logrus.gui.localdata.FileExtensions;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.IdentifierGenerator;
import logrus.tools.Utils;
import logrus.tools.logging.InitializeLoggerFailedException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.TreeItem;

/**
 * Dialog servicing quering Ontostorage with RDQL
 */
public class RDQLQueryDialog extends GUIObject
{
	// agent servicing this dialog
	private RDQLDialogAgent agent;

	// main menu of this dialog
	private Menu mainMenu;

	// file submenu
	private MenuItem menugroupFile;

	private Menu menuFile;

	private MenuItem miFile_Exit; // File -> Exit

	private MenuItem miFile_Font; // File -> Font...

	// query submenu
	private MenuItem menugroupQuery;

	private Menu menuQuery;

	private MenuItem miQuery_NewQuery; // Query -> New query

	private MenuItem miQuery_OpenQuery; // Query -> Open query...

	private MenuItem miQuery_SaveQuery; // Query -> Save query...

	private MenuItem miQuery_RunQuery; // Query -> Run query

	private MenuItem miQuery_SaveResults; // Query -> Save results...

	// toolbar-related fields
	private CoolBar toolbar;

	// buttons related to menu labeled "Query"
	private CoolItem barQueryItem;

	private ToolBar tbQuery;

	private ToolItem tiFile_ChangeFont; // File -> Change font...

	private ToolItem tiQuery_NewQuery; // Query -> New query

	private ToolItem tiQuery_OpenQuery; // Query -> Open query...

	private ToolItem tiQuery_SaveQuery; // Query -> Save query...

	private ToolItem tiQuery_RunQuery; // Query -> Run query

	private ToolItem tiQuery_SaveResults; // Query -> Save results...

	// Tree items
	private ArrayList treeItems;

	// layout used by dialog
	private GridLayout gridLayout;

	// combo used for selecting models to be queried
	private Combo comboModelToQuery;

	// Text box with query string
	private Text textQueryString;

	// Text box with RDQL results
	private Text textRDQLResults;

	private Label labelSelectModel;

	private Label labelQueryString;

	private Label labelQueryResult;

	/**
	 *  
	 */
	public RDQLQueryDialog (Display parent)
	{
		super(parent);
		this.display = parent;
		this.treeItems = new ArrayList();
		this.init();
	}

	/**
	 * Init GUI
	 */
	public void init ()
	{
		this.createShell();
		this.createMenu();
		this.createToolBar();
		this.createControls();
		try
		{
			this.initAgent();
		}
		catch (StaleProxyException e)
		{
			Utils.getInstance().showExceptionInfo(e,this.shell);
			this.shell.close();
		}
		catch (OntologyException e)
		{
			Utils.getInstance().showExceptionInfo(e,this.shell);
			this.shell.close();

		}
		catch (InitializeLoggerFailedException e)
		{
			Utils.getInstance().showExceptionInfo(e,this.shell);
			this.shell.close();
		}
	}

	/**
	 * Initialize agent responsible for servicing this dialog
	 * 
	 * @throws StaleProxyException
	 * @throws InitializeLoggerFailedException
	 * @throws OntologyException
	 */
	private void initAgent () throws StaleProxyException, OntologyException,
			InitializeLoggerFailedException
	{
		this.agent = new RDQLDialogAgent(this);
		String agentPostfix = "";
		try
		{
			agentPostfix = IdentifierGenerator.getInstance().GenerateID();
		}
		catch (IdentifierGenerationFailedException e)
		{
			// ignore
		}
		AgentsPlatform.getInstance().addAgent(this.agent,
				"QueryDialogAgent_" + agentPostfix);
	}

	private void createShell ()
	{
		this.shell = new Shell(this.getDisplay());
		this.shell.setText("Quering OntoStorage");
		this.shell.setSize(800,600);
		this.shell.setImage(this.manger.getImage("Tools-RDQLQuery",this
				.getDisplay()));
	}

	private void createMenu ()
	{
		this.mainMenu = new Menu(this.shell,SWT.BAR);

		// file group
		this.menugroupFile = new MenuItem(this.mainMenu,SWT.CASCADE);
		this.menugroupFile.setText("File");
		this.menuFile = new Menu(this.shell,SWT.DROP_DOWN);
		this.menugroupFile.setMenu(this.menuFile);
		// file menu items
		// File -> Exit
		this.miFile_Exit = new MenuItem(this.menuFile,SWT.PUSH);
		this.miFile_Exit.setText("Exit");
		this.miFile_Exit.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onFile_Exit(e);
			}
		});

		// File -> Font...
		this.miFile_Font = new MenuItem(this.menuFile,SWT.PUSH);
		this.miFile_Font.setText("Change font...");
		this.miFile_Font.setImage(this.manger.getImage("RDQLTool-ChangeFont",this
				.getDisplay()));
		this.miFile_Font.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onFile_Font(e);
			}
		});

		// query group
		this.menugroupQuery = new MenuItem(this.mainMenu,SWT.CASCADE);
		this.menugroupQuery.setText("Query");
		this.menuQuery = new Menu(this.shell,SWT.DROP_DOWN);
		this.menugroupQuery.setMenu(this.menuQuery);
		// query menu items
		// Query -> New
		this.miQuery_NewQuery = new MenuItem(this.menuQuery,SWT.PUSH);
		this.miQuery_NewQuery.setText("New query");
		this.miQuery_NewQuery.setImage(this.manger.getImage("RDQLTool-NewQuery",
				this.getDisplay()));
		this.miQuery_NewQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_NewQuery(e);
			}
		});
		// Query -> Open...
		this.miQuery_OpenQuery = new MenuItem(this.menuQuery,SWT.PUSH);
		this.miQuery_OpenQuery.setText("Open query...");
		this.miQuery_OpenQuery.setImage(this.manger.getImage(
				"RDQLTool-OpenQuery",this.getDisplay()));
		this.miQuery_OpenQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_OpenQuery(e);
			}
		});
		// Query -> Save query...
		this.miQuery_SaveQuery = new MenuItem(this.menuQuery,SWT.PUSH);
		this.miQuery_SaveQuery.setText("Save query...");
		this.miQuery_SaveQuery.setImage(this.manger.getImage(
				"RDQLTool-SaveQuery",this.getDisplay()));
		this.miQuery_SaveQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_SaveQuery(e);
			}
		});
		// Query -> Run query
		this.miQuery_RunQuery = new MenuItem(this.menuQuery,SWT.PUSH);
		this.miQuery_RunQuery.setText("Run query");
		this.miQuery_RunQuery.setImage(this.manger.getImage("RDQLTool-RunQuery",
				this.getDisplay()));
		this.miQuery_RunQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_RunQuery(e);
			}
		});
		// Query -> Save results...
		this.miQuery_SaveResults = new MenuItem(this.menuQuery,SWT.PUSH);
		this.miQuery_SaveResults.setText("Save results...");
		this.miQuery_SaveResults.setImage(this.manger.getImage(
				"RDQLTool-SaveResults",this.getDisplay()));
		this.miQuery_SaveResults.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_SaveResults(e);
			}
		});

		// attach menu to the shell
		this.shell.setMenuBar(this.mainMenu);
	}

	private void createToolBar ()
	{
		Point itemSize;
		this.toolbar = new CoolBar(this.shell,SWT.FLAT);

		// buttons related to menu labeled "Query"
		this.barQueryItem = new CoolItem(this.toolbar,SWT.NONE);
		this.tbQuery = new ToolBar(this.toolbar,SWT.FLAT);

		// File -> Change Font...
		this.tiFile_ChangeFont = new ToolItem(this.tbQuery,SWT.FLAT);
		this.tiFile_ChangeFont.setImage(this.manger.getImage(
				"RDQLTool-ChangeFont",this.getDisplay()));
		this.tiFile_ChangeFont
				.setToolTipText("Change font used by results text box");
		this.tiFile_ChangeFont.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onFile_Font(e);
			}
		});

		// Query -> New query
		this.tiQuery_NewQuery = new ToolItem(this.tbQuery,SWT.FLAT);
		this.tiQuery_NewQuery.setImage(this.manger.getImage("RDQLTool-NewQuery",
				this.getDisplay()));
		this.tiQuery_NewQuery.setToolTipText("Create new RDQL query");
		this.tiQuery_NewQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_NewQuery(e);
			}
		});

		// Query -> Open query
		this.tiQuery_OpenQuery = new ToolItem(this.tbQuery,SWT.FLAT);
		this.tiQuery_OpenQuery.setImage(this.manger.getImage(
				"RDQLTool-OpenQuery",this.getDisplay()));
		this.tiQuery_OpenQuery.setToolTipText("Load RDQL query from a file");
		this.tiQuery_OpenQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_OpenQuery(e);
			}
		});

		// Query -> Save query
		this.tiQuery_SaveQuery = new ToolItem(this.tbQuery,SWT.FLAT);
		this.tiQuery_SaveQuery.setImage(this.manger.getImage(
				"RDQLTool-SaveQuery",this.getDisplay()));
		this.tiQuery_SaveQuery
				.setToolTipText("Save created RDQL query to a file");
		this.tiQuery_SaveQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_SaveQuery(e);
			}
		});

		// Query -> Run query
		this.tiQuery_RunQuery = new ToolItem(this.tbQuery,SWT.FLAT);
		this.tiQuery_RunQuery.setImage(this.manger.getImage("RDQLTool-RunQuery",
				this.getDisplay()));
		this.tiQuery_RunQuery.setToolTipText("Run created RDQL query");
		this.tiQuery_RunQuery.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_RunQuery(e);
			}
		});

		// Query -> Save results
		this.tiQuery_SaveResults = new ToolItem(this.tbQuery,SWT.FLAT);
		this.tiQuery_SaveResults.setImage(this.manger.getImage(
				"RDQLTool-SaveResults",this.getDisplay()));
		this.tiQuery_SaveResults
				.setToolTipText("Save results of a executed query");
		this.tiQuery_SaveResults.addListener(SWT.Selection,new Listener()
		{
			public void handleEvent (Event e)
			{
				onQuery_SaveResults(e);
			}
		});

		// finish creating "Query" toolbar
		this.tbQuery.pack();
		itemSize = this.tbQuery.getSize();
		this.barQueryItem.setControl(this.tbQuery);
		this.barQueryItem.setSize(this.barQueryItem.computeSize(itemSize.x,
				itemSize.y));
		this.barQueryItem.setMinimumSize(itemSize);

		// finish creating toolbar
		this.toolbar.pack();
		GridData dataToolbar = new GridData();
		dataToolbar.horizontalSpan = 2;
		this.toolbar.setLayoutData(dataToolbar);
	}

	private void createControls ()
	{
		// combo used for selecting models to be queried
		this.gridLayout = new GridLayout();
		this.gridLayout.numColumns = 2;
		this.gridLayout.marginHeight = 5;
		this.gridLayout.marginWidth = 5;
		this.shell.setLayout(this.gridLayout);
		// dialog controls
		this.labelSelectModel = new Label(this.shell,SWT.RIGHT);
		this.labelSelectModel.setText("Select model to be queried:");
		this.labelSelectModel.setVisible(false);
		GridData dataLabelSelectModel = new GridData();
		this.labelSelectModel.setLayoutData(dataLabelSelectModel);
		this.comboModelToQuery = new Combo(this.shell,SWT.DROP_DOWN
				| SWT.READ_ONLY);
		this.comboModelToQuery.setItems(new String[] { "Both models",
				"Only complete model", "Only temporary model" });
		this.comboModelToQuery.select(0);
		this.comboModelToQuery.setVisible(false);
		GridData dataModelToQueryCombo = new GridData();
		this.comboModelToQuery.setLayoutData(dataModelToQueryCombo);

		// Textbox with query string
		this.labelQueryString = new Label(this.shell,SWT.LEFT);
		this.labelQueryString.setText("Type the RDQL query:");
		GridData dataLabelQueryString = new GridData(GridData.FILL_HORIZONTAL);
		dataLabelQueryString.horizontalSpan = 2;
		this.labelQueryString.setLayoutData(dataLabelQueryString);
		this.textQueryString = new Text(this.shell,SWT.BORDER | SWT.V_SCROLL
				| SWT.H_SCROLL | SWT.MULTI);
		GridData dataQueryStringText = new GridData(GridData.FILL_BOTH);
		dataQueryStringText.horizontalSpan = 2;
		dataQueryStringText.heightHint = 80;
		this.textQueryString.setLayoutData(dataQueryStringText);

		// Textbox with query results
		this.labelQueryResult = new Label(this.shell,SWT.LEFT);
		this.labelQueryResult.setText("Browse results of the query:");
		GridData dataLabelQueryResult = new GridData(GridData.FILL_HORIZONTAL);
		dataLabelQueryResult.horizontalSpan = 2;
		this.labelQueryResult.setLayoutData(dataLabelQueryResult);
		this.textRDQLResults = new Text(this.shell,SWT.BORDER | SWT.V_SCROLL
				| SWT.H_SCROLL | SWT.MULTI);
		try
		{
			FontData fontData = new FontData(
					"1|Courier New|9|0|WINDOWS|1|-13|0|0|0|400|0|0|0|-18|3|2|1|49|Courier New");
			this.textRDQLResults.setFont(new Font(this.display,fontData));
		}
		catch (Exception exc)
		{
			// ignore
		}
		GridData dataTextRDQLResults = new GridData(GridData.FILL_HORIZONTAL);
		dataTextRDQLResults.horizontalSpan = 2;
		dataTextRDQLResults.heightHint = 320;
		this.textRDQLResults.setLayoutData(dataTextRDQLResults);
	}

	/**
	 * Process request: close shell
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onFile_Exit (Event e)
	{
		this.shell.close();
	}

	/**
	 * Process request: create new query
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onQuery_NewQuery (Event e)
	{
		// select default model
		this.comboModelToQuery.select(0);
		// clean query string
		this.textQueryString.setText("");
		// clean results tree
		for (Iterator iter = this.treeItems.iterator(); iter.hasNext();)
		{
			TreeItem element = (TreeItem)iter.next();
			element.dispose();
		}
	}

	/**
	 * Process request: open file with saved query
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onQuery_OpenQuery (Event e)
	{
		// first ask for a file
		FileDialog fileDlg = new FileDialog(this.shell,SWT.OPEN);
		fileDlg.setFilterExtensions(new String[] { "*."
				+ FileExtensions.RDQLQueryDocExt });
		String filename = fileDlg.open();

		// if file has been selected follow the procedure
		if (filename == null)
		{
			return;
		}
		FileReader input;
		StringBuffer strBuff = new StringBuffer();
		BufferedReader buffReader = null;
		try
		{
			input = new FileReader(filename);
			buffReader = new BufferedReader(input);
			for (String line = buffReader.readLine(); line != null;)
			{
				strBuff.append(line);
				strBuff.append('\n');
				line = buffReader.readLine();
			}
			this.textQueryString.setText(strBuff.toString());
		}
		catch (FileNotFoundException exc)
		{
			Utils.getInstance().showErrorMessage("File not found",this.shell);
		}
		catch (IOException exc)
		{
			Utils.getInstance().showErrorMessage(
					"Error reading file: " + exc.getMessage(),this.shell);
		}
		finally
		{
			if (buffReader != null)
			{
				try
				{
					buffReader.close();
				}
				catch (IOException exc)
				{
					Utils.getInstance().showErrorMessage(
							"Error reading file: " + exc.getMessage(),this.shell);
				}
			}
		}
	}

	/**
	 * Process request: save created query to a file
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onQuery_SaveQuery (Event e)
	{
		// first check if user has entered any query
		String query = this.validateEnteredQuery();
		if (query == null)
		{
			Utils.getInstance().showErrorMessage("No query string to save!",
					this.shell);
			return;
		}

		// ok, now prompt for file
		FileDialog fileDlg = new FileDialog(this.shell,SWT.SAVE);
		fileDlg.setFilterExtensions(new String[] { "*."
				+ FileExtensions.RDQLQueryDocExt });
		String filename = fileDlg.open();
		String extension = FileExtensions.RDQLQueryDocExt;

		// if file has been selected follow the procedure
		if (filename == null)
		{
			return;
		}
		// extend filename if necessary
		filename = Utils.getInstance().strAppendExtension(filename,extension);

		Writer output = null;
		try
		{
			File file = new File(filename);
			output = new BufferedWriter(new FileWriter(file));
			output.write(query);
		}
		catch (IOException exc)
		{
			Utils.getInstance().showErrorMessage(
					"Error while saving file: " + exc.getMessage(),this.shell);
		}
		finally
		{
			if (output != null)
				try
				{
					output.close();
				}
				catch (IOException exc)
				{
					Utils.getInstance().showErrorMessage(
							"Error while saving file: " + exc.getMessage(),this.shell);
				}
		}
	}

	/**
	 * Process request: run created query
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onQuery_RunQuery (Event e)
	{
		// first check if user has entered any query
		String query = this.validateEnteredQuery();
		if (query == null)
		{
			Utils.getInstance().showErrorMessage("No query entered!",this.shell);
			return;
		}
		this.agent.executeQuery(query);
	}

	/**
	 * Process request: save query results to a file
	 * 
	 * @param e
	 *           Event that was a cause of this request
	 */
	protected void onQuery_SaveResults (Event e)
	{
		String strResults = this.textRDQLResults.getText();
		if (strResults == null || strResults.length() == 0)
		{
			return;
		}

		// ok, now prompt for file
		FileDialog fileDlg = new FileDialog(this.shell,SWT.SAVE);
		fileDlg.setFilterExtensions(new String[] { "*."
				+ FileExtensions.RDQLQueryDocExt });
		String filename = fileDlg.open();
		String extension = "txt";

		// if file has been selected follow the procedure
		if (filename == null)
		{
			return;
		}
		// extend filename if necessary
		filename = Utils.getInstance().strAppendExtension(filename,extension);

		// if file has been selected follow the procedure
		if (filename == null)
		{
			return;
		}
		// extend filename if necessary
		filename = Utils.getInstance().strAppendExtension(filename,extension);

		Writer output = null;
		try
		{
			File file = new File(filename);
			output = new BufferedWriter(new FileWriter(file));
			output.write(strResults);
		}
		catch (IOException exc)
		{
			Utils.getInstance().showErrorMessage(
					"Error while saving file: " + exc.getMessage(),this.shell);
		}
		finally
		{
			if (output != null)
				try
				{
					output.close();
				}
				catch (IOException exc)
				{
					Utils.getInstance().showErrorMessage(
							"Error while saving file: " + exc.getMessage(),this.shell);
				}
		}
	}

	/**
	 * Check if query entered by user is correct
	 * 
	 * @return Query string if query is correct, otherwise returns null
	 */
	private String validateEnteredQuery ()
	{
		// first check if user has entered any query
		String query = null;
		query = this.textQueryString.getText();
		if (query == null || query.length() == 0)
		{
			return null;
		}
		else
		{
			return query;
		}
	}

	/**
	 * Display RDQL results
	 * 
	 * @param results
	 */
	public void displayRDQLResults (String results)
	{
		this.textRDQLResults.setText(results);
	}

	public void onFile_Font (Event e)
	{
		FontDialog fontDialog = new FontDialog(this.shell);
		Font oldFont = this.textRDQLResults.getFont();
		if (oldFont != null)
		{
			FontData[] fontDataOld = oldFont.getFontData();
			if (fontDataOld.length > 0)
			{
				fontDialog.setFontData(fontDataOld[0]);
			}
		}
		FontData fontData = fontDialog.open();
		if (fontData != null)
		{
			System.out.println(fontData);
			this.textRDQLResults.setFont(new Font(this.display,fontData));
		}
	}
}
