package logrus.gui.tools;

import java.util.Hashtable;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.widgets.Display;

/**
 * Manager holding menu/toolbar icons. Class implementing singleton design pattern
 */
public class ImagesManager
{
	// singleton instance
	private static ImagesManager instance;
	
	// folder wher icons are stored
	private static final String strIconsFolder = "images";
	
	// hashtable storing image data objects
	private Hashtable icons;
	
	// loader used for loading images from files 
	private ImageLoader loader;
	
	/**
	 * Create singleton instance 
	 */
	public ImagesManager ()
	{
		super();
		this.icons = new Hashtable();
		this.loader = new ImageLoader();
		this.loadIcon("System-RunPlatform");
		this.loadIcon("System-StopPlatform");
		this.loadIcon("Tools-RDQLQuery");
		this.loadIcon("Tools-Report");		
		this.loadIcon("RDQLTool-NewQuery");
		this.loadIcon("RDQLTool-SaveQuery");
		this.loadIcon("RDQLTool-RunQuery");
		this.loadIcon("RDQLTool-SaveResults");
		this.loadIcon("RDQLTool-OpenQuery");
		this.loadIcon("RDQLTool-ChangeFont");
		this.loadIcon("Tools-UpdateProcess");
	}
	
	/**
	 * Load specified icon
	 * @param name Name of icon to be loaded
	 */
	private void loadIcon(String name)
	{
		String path = ImagesManager.strIconsFolder + "/" + name +".ico";
		ImageData imageData = this.loader.load(path)[0];
		this.icons.put(name,imageData);
	}
	
	/**
	 * Get image adjusted to specifed display
	 * @param name Name of image
	 * @param display Display to which icon should be adjusted
	 * @return image adjusted to specifed display
	 */
	public Image getImage (String name, Display display)
	{
		return new Image(display,(ImageData)this.icons.get(name));
	}

	/**
	 * Get singleton instance of ImagesManager 
	 * @return Singleton instance of ImagesManager
	 */
	public static ImagesManager getInstance ()
	{
		if (ImagesManager.instance==null)
		{
			ImagesManager.instance = new ImagesManager();
		}
		return ImagesManager.instance;
	}
}
