package logrus.ontmodel;

import logrus.ontmodel.restaurant.tools.RestaurantComparer;

public class ObjectInfoComparerFactory
{
	private static ObjectInfoComparerFactory instance;

	private ObjectInfoComparerFactory ()
	{
		super();
	}

	public static ObjectInfoComparerFactory getInstance ()
	{
		if (ObjectInfoComparerFactory.instance == null)
		{
			ObjectInfoComparerFactory.instance = new ObjectInfoComparerFactory();
		}
		return ObjectInfoComparerFactory.instance;
	}
	
	/**
	 * Create appropriate instance of class implementing <code>IObjectInfoComparer</code> matching
	 * ontological class <code>ontClass</code>
	 * 
	 * @param ontClass
	 *           Ontological class name
	 * @return Appropriate instance of class implementing <code>IObjectInfoComparer</code> matching
	 *         ontological class <code>ontClass</code> or null if class is unknown
	 */
	
	public IObjectInfoComparer createObjectInfoComparer (String ontClass)
	{
		if (ontClass.compareTo(Prefixes.CLASS_RES) == 0)
		{
			return new RestaurantComparer();
		}
		return null;
	}	
}
