package logrus.ontmodel;

import logrus.ontmodel.restaurant.tools.RestaurantSearcherByTag;

public class ObjectInfoSearcherByTagFactory
{
	private static ObjectInfoSearcherByTagFactory instance;

	private ObjectInfoSearcherByTagFactory ()
	{
		super();
	}

	public static ObjectInfoSearcherByTagFactory getInstance ()
	{
		if (ObjectInfoSearcherByTagFactory.instance == null)
		{
			ObjectInfoSearcherByTagFactory.instance = new ObjectInfoSearcherByTagFactory();
		}
		return ObjectInfoSearcherByTagFactory.instance;
	}

	/**
	 * Create appropriate instance of class implementing <code>IObjectInfoSearcherByTag</code> matching
	 * ontological class <code>ontClass</code>
	 * 
	 * @param ontClass
	 *           Ontological class name
	 * @return Appropriate instance of class implementing <code>IObjectInfoSearcherByTag</code> matching
	 *         ontological class <code>ontClass</code> or null if class is unknown
	 */
	
	public IObjectInfoSearcherByTag createObjectSearcherByTag (String ontClass)
	{
		if (ontClass.compareTo(Prefixes.CLASS_RES) == 0)
		{
			return new RestaurantSearcherByTag();
		}
		return null;
	}
}
