package logrus.ontmodel;


import com.hp.hpl.jena.rdf.model.Model;

import java.util.Hashtable;

import logrus.ontmodel.exceptions.ModelNotFoundException;

/**
 */
public class OntModelManager
{
	/**
	 * Instances of OntModelWrapers wrapping models
	 */
	private Hashtable wrappers = new Hashtable();

	/**
	 * Singleton instance of OntModelManager class
	 */
	private static OntModelManager instance;

	/**
	 *  
	 */
	private OntModelManager ()
	{
		super();
	}

	/**
	 * Get singleton instance of OntModelManager class
	 * 
	 * @return
	 */
	public static OntModelManager getInstance ()
	{
		if (OntModelManager.instance==null)
		{
			OntModelManager.instance = new OntModelManager();
		}
		return OntModelManager.instance;
	}

	/**
	 * Get wrapper for model with specified name
	 * 
	 * @param modelName
	 *           Name of model that we want get access to
	 * @return Wrapper for specified model
	 * @throws ModelNotFoundException
	 *            Model with specified name does not exist
	 */
	public IOntModelWrapper getWrapper (String modelName)
			throws ModelNotFoundException
	{
		Object obj = OntModelManager.getInstance().wrappers.get(modelName);
		if (obj==null)
		{
			IOntModelWrapper wrapper = OntModelWrapperFactory.getInstance()
					.createModelFromDatabase(modelName);
			OntModelManager.getInstance().wrappers.put(modelName,wrapper);
			return wrapper;
		}
		else
		{
			return (IOntModelWrapper)obj;
		}
	}

	/**
	 * Get wrapper for model with specified name
	 * 
	 * @param modelName
	 *           Name of model that we want get access to
	 * @param dbaseWrapper
	 *           Wrapper used for database access
	 * @return Wrapper for specified model
	 * @throws ModelNotFoundException
	 *            Model with specified name does not exist
	 */
	public IOntModelWrapper getWrapper (String modelName,
			DatabaseWrapper dbaseWrapper) throws ModelNotFoundException
	{
		Object obj = OntModelManager.getInstance().wrappers.get(modelName);
		if (obj==null)
		{
			IOntModelWrapper wrapper = OntModelWrapperFactory.getInstance()
					.createModelFromDatabase(modelName,dbaseWrapper);
			OntModelManager.getInstance().wrappers.put(modelName,wrapper);
			return wrapper;
		}
		else
		{
			return (IOntModelWrapper)obj;
		}
	}

	/**
	 * Fills specified model with default RDF files located in specified
	 * directory
	 * 
	 * @param model
	 *           Model to be filled
	 * @param rootFolderURL
	 *           Folder in which RDF files are stored
	 */
	public static void fillModel (Model model, String rootFolderURL)
	{
		model.read(rootFolderURL+"Accessibility.rdf");
		model.read(rootFolderURL+"AccessibilityTypesList.rdf");
		model.read(rootFolderURL+"Address.rdf");
		model.read(rootFolderURL+"AlcoholService.rdf");
		model.read(rootFolderURL+"AlcoholServicesList.rdf");
		model.read(rootFolderURL+"CountriesList.rdf");
		model.read(rootFolderURL+"Country.rdf");
		model.read(rootFolderURL+"Cuisine.rdf");
		model.read(rootFolderURL+"CuisineTypesList.rdf");
		model.read(rootFolderURL+"CurrenciesTypesList.rdf");
		model.read(rootFolderURL+"Currency.rdf");
		model.read(rootFolderURL+"DataSource.rdf");
		model.read(rootFolderURL+"DataSourcesList.rdf");
		model.read(rootFolderURL+"Dress.rdf");
		model.read(rootFolderURL+"DressTypesList.rdf");
		model.read(rootFolderURL+"Parking.rdf");
		model.read(rootFolderURL+"ParkingTypesList.rdf");
		model.read(rootFolderURL+"PaymentMethod.rdf");
		model.read(rootFolderURL+"PaymentMethodsTypesList.rdf");
		model.read(rootFolderURL+"Period.rdf");
		model.read(rootFolderURL+"Price.rdf");
		model.read(rootFolderURL+"Reservations.rdf");
		model.read(rootFolderURL+"ReservationsTypesList.rdf");
		model.read(rootFolderURL+"RestaurantSpec.rdf");
		model.read(rootFolderURL+"RestaurantSpecTypesList.rdf");
		model.read(rootFolderURL+"SmokingPermission.rdf");
		model.read(rootFolderURL+"SmokingPermissionsTypesList.rdf");
		model.read(rootFolderURL+"Restaurant.rdf");
	}
}
