package logrus.ontmodel;

import jade.util.leap.Iterator;

import java.util.Hashtable;

import logrus.tools.queue.Queue;

/**
 * Various ontology and RDF related methods holder, singleton
 */
public class OntUtils
{

	private static OntUtils instance = null;

	/**
	 *  
	 */
	private OntUtils ()
	{
		super();
	}

	/**
	 * Get singleton instance of OntUtils
	 * 
	 * @return Instance of OntUtils
	 */
	public static OntUtils getInstance ()
	{
		if (OntUtils.instance == null)
		{
			OntUtils.instance = new OntUtils();
		}
		return OntUtils.instance;
	}

	/**
	 * Calculate list of remove/add operations to transform <code>oldDesc</code> to
	 * <code>newDesc</code>
	 * 
	 * @param oldDesc
	 *           Old description (from database)
	 * @param newDesc
	 *           New description (such should be after remove/add operations)
	 * @return
	 */
	public Queue calculateOperations (QuadsList oldDesc, QuadsList newDesc)
	{
		Queue operations = new Queue();

		// ArrayList auxList = new ArrayList();
		// for (Iterator iter = newDesc.getList().iterator(); iter.hasNext();)
		// {
		// auxList.add((Quad)iter.next());
		// }
		//
		// for (Iterator iter = oldDesc.getList().iterator(); iter.hasNext();)
		// {
		// Quad elem = (Quad)iter.next();
		// Quad matchingQuad = newDesc.getQuadMatching(elem);
		// if (matchingQuad != null)
		// {
		// auxList.remove(matchingQuad);
		// }
		// else
		// {
		// operations.inject(new ModelOperation(ModelOperation.REMOVE,elem));
		// }
		// }
		//
		// for (java.util.Iterator iter = auxList.iterator(); iter.hasNext();)
		// {
		// operations.inject(new ModelOperation(ModelOperation.ADD,(Quad)iter
		// .next()));
		// }
		// return operations;
		Hashtable hashNew = newDesc.getHashtable();
		for (Iterator iter = oldDesc.getList().iterator(); iter.hasNext();)
		{
			Quad elem = (Quad)iter.next();
			String hash = elem.calculateHash();
			if (hashNew.get(hash) == null)
			{
				// if statement has been in old description but it's not included in new one then it
				// should be removed
				operations.inject(new ModelOperation(ModelOperation.REMOVE,elem));
			}
			hashNew.remove(hash);
		}
		for (java.util.Iterator iter = hashNew.values().iterator(); iter
				.hasNext();)
		{
			// if statement hasn't been in old description but it's included in new one then it should
			// be added
			operations.inject(new ModelOperation(ModelOperation.ADD,(Quad)iter
					.next()));
		}
		return operations;
	}

	/**
	 * Assume <code>uri</code> is in NAMESPACE#OBJECT format, and split it into the two strings
	 * 
	 * @param uri
	 *           URI to be splitted
	 * @return Array of two strings: NAMESPACE and OBJECT
	 */
	public String[] splitURI (String uri)
	{
		return uri.split("#");
	}
}
