package logrus.ontmodel;

/**
 * Holds prefixes for ontologies
 */
public interface Prefixes
{
	// standard prefixes
	public static final String RDF   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	public static final String RDFS  = "http://www.w3.org/2000/01/rdf-schema#"; 
   public static final String XSD   = "http://www.w3.org/2001/XMLSchema#"; 
   
   public static final String RDF_TYPE = Prefixes.RDF+"type";
   public static final String RDFS_CLASS = Prefixes.RDFS+"Class";
   
   // ontology prefixes
  	public static final String BaseURI = "http://www.agentlab.net/schemas/";
  	public static final String BaseLocationURI = Prefixes.BaseURI + "location";
  	public static final String BaseMoneyURI = Prefixes.BaseURI + "money";
  	public static final String BaseRestaurantURI = Prefixes.BaseURI + "restaurant";
  	public static final String BaseOntoStorageURI = Prefixes.BaseURI + "ontoStorage";
  	
  	public static final String ProviderProperty = Prefixes.BaseOntoStorageURI + "#DataProviderID";
  	public static final String SourceLink = Prefixes.BaseOntoStorageURI + "#SourceLink";
  	
  	public static final String XSD_BOOLEAN = Prefixes.XSD + "boolean";
  	public static final String XSD_CHAR = Prefixes.XSD + "char";
  	public static final String XSD_DOUBLE = Prefixes.XSD + "double";
  	public static final String XSD_FLOAT = Prefixes.XSD + "float";

  	public static final String XSD_LONG = Prefixes.XSD + "long";
  	public static final String XSD_INTEGER = Prefixes.XSD + "integer";
  	public static final String XSD_POSITIVE_INTEGER = Prefixes.XSD + "positiveInteger";
  	public static final String XSD_NEGATIVE_INTEGER = Prefixes.XSD + " negativeInteger";
  	public static final String XSD_NON_NEGATIVE_INTEGER = Prefixes.XSD + "nonNegativeInteger";
  	public static final String XSD_NON_POSITIVE_INTEGER = Prefixes.XSD + "nonPositiveInteger";
  	public static final String XSD_UNSINED_LONG = Prefixes.XSD + "unsignedLong";
  	public static final String XSD_INT = Prefixes.XSD + "int";
  	public static final String XSD_UNSIGNED_INT = Prefixes.XSD + "unsignedInt";
  	public static final String XSD_SHORT = Prefixes.XSD + "short";
  	public static final String XSD_UNSIGNED_SHORT = Prefixes.XSD + "unsignedShort";
  	public static final String XSD_STRING = Prefixes.XSD + "string";
  	public static final String XSD_NORMALIZED_STRING = Prefixes.XSD + "normalizedString";
  	public static final String XSD_TOKEN = Prefixes.XSD + "token";
  	public static final String XSD_URI = Prefixes.XSD + "anyURI";
  	 
  	// timestamps
  	public static final String TIMESTAMP_LAST = BaseOntoStorageURI + "#LastUpdate";
  	public static final String TIMESTAMP_NEXT = BaseOntoStorageURI + "#NextUpdate";
  	
  	// http://www.agentlab.net/schemas/restaurant#...
	public static final String CLASS_RES = Prefixes.BaseRestaurantURI + "#Restaurant";
  	public static final String RES_CUISINE = Prefixes.BaseRestaurantURI + "#cuisine";
  	public static final String RES_NAME = Prefixes.BaseRestaurantURI + "#title";
  	public static final String RES_WWW = Prefixes.BaseRestaurantURI + "#URL";
  	public static final String RES_EMAIL = Prefixes.BaseRestaurantURI + "#email";
  	public static final String RES_PRI_BREAK = Prefixes.BaseRestaurantURI + "#breakfastPrice";
  	public static final String RES_PRI_LUNCH = Prefixes.BaseRestaurantURI + "#lunchPrice";
  	public static final String RES_PRI_DINNER = Prefixes.BaseRestaurantURI + "#dinnerPrice";
  	public static final String RES_FEAT = Prefixes.BaseRestaurantURI + "#feature";
  	public static final String RES_ACC = Prefixes.BaseRestaurantURI + "#accepts";
  	public static final String RES_SMK = Prefixes.BaseRestaurantURI + "#smoking";
  	public static final String RES_DRESS = Prefixes.BaseRestaurantURI + "#dress";
  	public static final String RES_ALC = Prefixes.BaseRestaurantURI + "#alcohol";  	
  	public static final String RES_RESERV = Prefixes.BaseRestaurantURI + "#reservations";
  	public static final String RES_PARK = Prefixes.BaseRestaurantURI + "#parking";
  	public static final String RES_HANDACC = Prefixes.BaseRestaurantURI + "#accessibility";
  	public static final String RES_SEATS = Prefixes.BaseRestaurantURI + "#capacity";
  	public static final String RES_LANG = Prefixes.BaseRestaurantURI + "#serviceLanguage"; 
  	public static final String RES_PAR_HR = Prefixes.BaseRestaurantURI + "#parsedHours";
  	
  	// http://www.agentlab.net/schemas/money#...
  	public static final String MON_FPC = Prefixes.BaseMoneyURI+ "#FuzzyPriceCode";
  	public static final String MON_FPC_VCHEAP = Prefixes.BaseMoneyURI+ "#VeryCheap";
  	public static final String MON_FPC_INEXP = Prefixes.BaseMoneyURI+ "#Inexpensive";
  	public static final String MON_FPC_MODER = Prefixes.BaseMoneyURI+ "#Moderate";
  	public static final String MON_FPC_EXP = Prefixes.BaseMoneyURI+ "#Expensive";
  	public static final String MON_FPC_VEXP = Prefixes.BaseMoneyURI+ "#VeryExpensive";
  	
  	// http://www.agentlab.net/schemas/location#...
  	public static final String LOC_COUNTRY = Prefixes.BaseLocationURI +"#country";
  	public static final String LOC_CITY = Prefixes.BaseLocationURI +"#city";
  	public static final String LOC_REGION = Prefixes.BaseLocationURI +"#region";
  	public static final String LOC_STREET_ADDR = Prefixes.BaseLocationURI +"#streetAddress";
  	public static final String LOC_ZIP = Prefixes.BaseLocationURI +"#zip";
  	public static final String LOC_PHONE = Prefixes.BaseLocationURI +"#phone";
  	public static final String LOC_NEIGHBORHOOD = Prefixes.BaseLocationURI +"#neighborhood";
  	
	// list of rdf files describing restaurant ontology
	public static final String[] ONTOLOGY_FILES = 
	{
		"location/AttractionCategoryCode.rdf",
		"location/IndexPointCode.rdf",
		"location/Location.rdf",
		"location/LocationCategoryCode.rdf",
		"money/FuzzyPriceCode.rdf",
		"money/MeanOfPayment.rdf",
		"restaurant/AccessibilityCode.rdf",
		"restaurant/AlcoholCode.rdf",
		"restaurant/CuisineCode.rdf",
		"restaurant/DinerReview.rdf",
		"restaurant/DressCode.rdf",
		"restaurant/FeatureCode.rdf",
		"restaurant/ParkingCode.rdf",
		"restaurant/ReservationCode.rdf",
		"restaurant/Restaurant.rdf",
		"restaurant/RestaurantServiceInfo.rdf",
		"restaurant/SmokingCode.rdf",
		"ontoStorage/DataSource.rdf"
	}; 
}
