package logrus.ontmodel.merging;

public class MergingError
{
	// expected plurality was singular but there where multiple statements
	public static final int UnexpectedMultiplicity = 1;

	// one of lists lacks some essential properties
	public static final int LackingEssential = 2;

	// initialization of custom merging/matching java class has failed
	public static final int JavaClassInitializationFailed = 3;

	// matching property values marked as desired to be identical failed
	public static final int RequiredMatchFailed = 4;

	// maps error code to messages
	private static String[] codesMap;

	// error code number
	private int errorCode;

	// merging this property caused error
	private String property;

	public MergingError (String property, int code)
	{
		super();
		this.setErrorCode(code);
		this.setProperty(property);
	}

	/**
	 * Get description of error
	 * 
	 * @return Description of error
	 */
	public String getErrorDescription ()
	{
		return MergingError.translateErrorDescription(this.errorCode);
	}

	public static String translateErrorDescription (int code)
	{
		if (MergingError.codesMap == null)
		{
			MergingError.codesMap = new String[5];
			MergingError.codesMap[0] = "No error";
			MergingError.codesMap[MergingError.LackingEssential] = "One of lists lacks some essential properties";
			MergingError.codesMap[MergingError.UnexpectedMultiplicity] = "Expected plurality was singular but there where multiple statements";
			MergingError.codesMap[MergingError.JavaClassInitializationFailed] = "Initialization of custom merging/matching java class has failed";
			MergingError.codesMap[MergingError.RequiredMatchFailed] = "Matching property values marked as desired to be identical failed";
		}
		return MergingError.codesMap[code];
	}

	/**
	 * @param errorCode
	 *           The errorCode to set.
	 */
	public void setErrorCode (int errorCode)
	{
		this.errorCode = errorCode;
	}

	/**
	 * @return Returns the errorCode.
	 */
	public int getErrorCode ()
	{
		return this.errorCode;
	}

	/**
	 * @param property
	 *           The property to set.
	 */
	public void setProperty (String property)
	{
		this.property = property;
	}

	/**
	 * @return Returns the property.
	 */
	public String getProperty ()
	{
		return this.property;
	}
}
