package logrus.ontmodel.merging;

import java.util.Iterator;

import logrus.ontmodel.Quad;
import logrus.ontmodel.QuadsList;
import logrus.tools.Utils;
import logrus.tools.queue.Queue;

public class MergingReport
{
	// flag determinig if merging process has been done correctly
	private boolean correctMerge;

	// list of statement descriptions - results of merging
	private QuadsList results;

	// collection of error descriptions
	private Queue errors;

	public MergingReport ()
	{
		super();
		this.correctMerge = true;
		this.errors = new Queue();
		this.results = new QuadsList();
	}

	public String toString ()
	{
		StringBuffer buffer = new StringBuffer();
		if (!this.isCorrectMerge())
		{
			buffer.append("There were errors during unification:\n");
			buffer.append(this.getErrorsReport());
		}
		else
		{
			buffer.append("Unification successful:\n");
			buffer.append(this.getVisualResults());
		}
		return buffer.toString();
	}

	/**
	 * Get report of errors during merging
	 * @return String containing report of errors during merging
	 */
	public String getErrorsReport ()
	{
		StringBuffer buffer = new StringBuffer();
		for (Iterator iter = this.errors.iterator(); iter.hasNext();)
		{
			MergingError err = (MergingError)iter.next();
			buffer.append(err.getProperty());
			buffer.append(" : ");
			buffer.append(err.getErrorDescription());
			buffer.append('\n');
		}
		return buffer.toString();
	}
	
	public String getVisualResults ()
	{
		return Utils.getInstance().filterNames(this.results.toString());
	}
	
	/**
	 * Add all statements from <code>stmts</code> to results
	 * 
	 * @param stmts
	 *           List of statement descriptions
	 */
	public void addStatements (Queue stmts)
	{
		for (Iterator iter = stmts.iterator(); iter.hasNext();)
		{
			this.results.add((Quad)iter.next());
		}
	}

	public void addError (String property, int code)
	{
		this.correctMerge = false;
		this.errors.inject(new MergingError(property,code));
	}

	/**
	 * @param correctMerge
	 *           The correctMerge to set.
	 */
	public void setCorrectMerge (boolean correctMerge)
	{
		this.correctMerge = correctMerge;
	}

	/**
	 * @return Returns the correctMerge.
	 */
	public boolean isCorrectMerge ()
	{
		return this.correctMerge;
	}

	/**
	 * @param results
	 *           The results to set.
	 */
	public void setResult (QuadsList results)
	{
		this.results = results;
	}

	/**
	 * @return Returns the results.
	 */
	public QuadsList getResult ()
	{
		return this.results;
	}

	/**
	 * @return Returns the errors.
	 */
	public Queue getErrors ()
	{
		return this.errors;
	}
}
