package logrus.ontmodel.merging;

import java.util.Iterator;

import logrus.ontmodel.validation.Plurality;
import logrus.ontmodel.validation.Significance;
import logrus.tools.queue.Queue;

public class MergingRule
{
	// uri of property that this rule applies to
	private String property = null;

	// significance of this property (essential/required/optional)
	private int significance = Significance.Optional;

	// allowed plurality of this property (singular/multiple)
	private int plurality = Plurality.Singular;

	// flag determinig if exact matching is required
	private boolean requireMatch = false;

	// fully qualified class named used for checking matching of properties
	private String matcherClassName = null;

	// flag determinig if provider significance order is strict
	private boolean providerSignificanceStrict = false;

	// queue describing significance of providers
	private Queue providerSignificanceOrder = null;

	// if this string is not null then data from other providers should be ignored
	private String exclusiveProvider = null;

	// queue of data considered as low significance data
	private Queue lowSignificanceData = null;

	// flag determinig if list of properties from providers should be merged
	private boolean mergeMultiple = false;

	// if this string is not null then whole process should be performed by specified class
	private String javaClassRule = null;

	public MergingRule ()
	{
		super();
	}

	public int getProviderRank (String provider)
	{
		int rank = 0;
		if (this.providerSignificanceOrder != null)
		{
			for (Iterator iter = this.providerSignificanceOrder.iterator(); iter
					.hasNext(); rank++)
			{
				String elem = (String)iter.next();
				if (provider.compareTo(elem) == 0)
				{
					return rank;
				}
			}
		}
		return -1;
	}

	/**
	 * @param requireMatch
	 *           The requireMatch to set.
	 */
	public void setRequireMatch (boolean requireMatch)
	{
		this.requireMatch = requireMatch;
	}

	/**
	 * @return Returns the requireMatch.
	 */
	public boolean isRequireMatch ()
	{
		return this.requireMatch;
	}

	/**
	 * @param matcherClassName
	 *           The matcherClassName to set.
	 */
	public void setMatcherClassName (String matcherClassName)
	{
		this.matcherClassName = matcherClassName;
	}

	/**
	 * @return Returns the matcherClassName.
	 */
	public String getMatcherClassName ()
	{
		return this.matcherClassName;
	}

	/**
	 * @param providerSignificanceStrict
	 *           The providerSignificanceStrict to set.
	 */
	public void setProviderSignificanceStrict (boolean providerSignificanceStrict)
	{
		this.providerSignificanceStrict = providerSignificanceStrict;
	}

	/**
	 * @return Returns the providerSignificanceStrict.
	 */
	public boolean isProviderSignificanceStrict ()
	{
		return this.providerSignificanceStrict;
	}

	/**
	 * @param providerSignificanceOrder
	 *           The providerSignificanceOrder to set.
	 */
	public void setProviderSignificanceOrder (Queue providerSignificanceOrder)
	{
		this.providerSignificanceOrder = providerSignificanceOrder;
	}

	/**
	 * @return Returns the providerSignificanceOrder.
	 */
	public Queue getProviderSignificanceOrder ()
	{
		return this.providerSignificanceOrder;
	}

	/**
	 * @param exclusiveProvider
	 *           The exclusiveProvider to set.
	 */
	public void setExclusiveProvider (String exclusiveProvider)
	{
		this.exclusiveProvider = exclusiveProvider;
	}

	/**
	 * @return Returns the exclusiveProvider.
	 */
	public String getExclusiveProvider ()
	{
		return this.exclusiveProvider;
	}

	/**
	 * @param lowSignificanceData
	 *           The lowSignificanceData to set.
	 */
	public void setLowSignificanceData (Queue lowSignificanceData)
	{
		this.lowSignificanceData = lowSignificanceData;
	}

	/**
	 * @return Returns the lowSignificanceData.
	 */
	public Queue getLowSignificanceData ()
	{
		return this.lowSignificanceData;
	}

	/**
	 * @param mergeMultiple
	 *           The mergeMultiple to set.
	 */
	public void setMergeMultiple (boolean mergeMultiple)
	{
		this.mergeMultiple = mergeMultiple;
	}

	/**
	 * @return Returns the mergeMultiple.
	 */
	public boolean isMergeMultiple ()
	{
		return this.mergeMultiple;
	}

	/**
	 * @param javaClassRule
	 *           The javaClassRule to set.
	 */
	public void setJavaClassRule (String javaClassRule)
	{
		this.javaClassRule = javaClassRule;
	}

	/**
	 * @return Returns the javaClassRule.
	 */
	public String getJavaClassRule ()
	{
		return this.javaClassRule;
	}

	/**
	 * @param property
	 *           The property to set.
	 */
	public void setProperty (String property)
	{
		this.property = property;
	}

	/**
	 * @return Returns the property.
	 */
	public String getProperty ()
	{
		return this.property;
	}

	/**
	 * @param significance
	 *           The significance to set.
	 */
	public void setSignificance (int significance)
	{
		this.significance = significance;
	}

	/**
	 * @param significance
	 *           The significance to set.
	 */
	public void setSignificance (String significance)
	{
		if (significance.compareTo("optional") == 0)
		{
			this.significance = Significance.Optional;
		}
		else if (significance.compareTo("essential") == 0)
		{
			this.significance = Significance.Essential;
		}
		else if (significance.compareTo("required") == 0)
		{
			this.significance = Significance.Required;
		}
	}

	/**
	 * @return Returns the significance.
	 */
	public int getSignificance ()
	{
		return this.significance;
	}

	/**
	 * @param plurality
	 *           The plurality to set.
	 */
	public void setPlurality (int plurality)
	{
		this.plurality = plurality;
	}

	/**
	 * @param plurality
	 *           The plurality to set.
	 */
	public void setPlurality (String plurality)
	{
		if (plurality.compareTo("singular") == 0)
		{
			this.plurality = Plurality.Singular;
		}
		else if (plurality.compareTo("multiple") == 0)
		{
			this.plurality = Plurality.Multiple;
		}
	}

	/**
	 * @return Returns the plurality.
	 */
	public int getPlurality ()
	{
		return this.plurality;
	}
}
