package logrus.ontmodel.merging;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import logrus.tools.Utils;
import logrus.tools.XMLAnalysisException;
import logrus.tools.queue.Queue;

public class MergingRulesCollection
{
	private ArrayList list;

	private Hashtable map;

	public MergingRulesCollection (String filepath) throws XMLAnalysisException
	{
		super();
		this.list = new ArrayList();
		this.map = new Hashtable();
		this.loadRules(filepath);
	}

	private void loadRules (String filepath) throws XMLAnalysisException
	{
		Utils utils = Utils.getInstance();
		Document doc = utils.loadXMLDocument(filepath);
		Node root = doc.getFirstChild();
		if (root.getNodeName() != "MergingRules")
		{
			throw new XMLAnalysisException("Incorrect root element");
		}
		Queue rules = utils.getOnlyNamedChildren(root);
		for (Iterator iter = rules.iterator(); iter.hasNext();)
		{
			Node ruleNode = (Node)iter.next();
			if (ruleNode.getNodeName().compareTo("Rule") != 0)
			{
				throw new XMLAnalysisException(
						"Unexpected node, expected was <Rule> but found was <"
								+ ruleNode.getNodeName() + ">");
			}
			String property = utils.extractStrictXMLAttrValue(ruleNode,"property");
			String significance = utils.extractStrictXMLAttrValue(ruleNode,"significance");
			String plurality = utils.extractStrictXMLAttrValue(ruleNode,"plurality");
			MergingRule rule = new MergingRule();
			rule.setProperty(property);
			rule.setSignificance(significance);
			rule.setPlurality(plurality);
			for (Iterator subIter = utils.getOnlyNamedChildren(ruleNode)
					.iterator(); subIter.hasNext();)
			{
				Node subNode = (Node)subIter.next();
				String subNodeName = subNode.getNodeName();
				if (subNodeName.compareTo("RequireMatch") == 0)
				{
					rule.setRequireMatch(true);
					String classRef = utils.extractXMLAttrValue(subNode,"class");
					if (classRef != null)
					{
						rule.setMatcherClassName(classRef);
					}
					continue;
				}
				if (subNodeName.compareTo("ProviderSignificance") == 0)
				{
					Queue providersOrder = new Queue();
					for (Iterator provIter = utils.getOnlyNamedChildren(subNode)
							.iterator(); provIter.hasNext();)
					{
						providersOrder.inject(utils.extractStrictXMLAttrValue(
								(Node)provIter.next(),"id"));
					}
					rule.setProviderSignificanceOrder(providersOrder);
					continue;
				}
				if (subNodeName.compareTo("ExclusiveProvider") == 0)
				{
					rule.setExclusiveProvider(utils.extractStrictXMLAttrValue(
							subNode,"id"));
					continue;
				}
				if (subNodeName.compareTo("LowSignificanceData") == 0)
				{
					Queue lowSignData = new Queue();
					for (Iterator lowIter = utils.getOnlyNamedChildren(subNode)
							.iterator(); lowIter.hasNext();)
					{
						lowSignData.inject(utils.extractStrictXMLAttrValue(
								(Node)lowIter.next(),"value"));
					}			
					rule.setLowSignificanceData(lowSignData);
					continue;
				}		
				if (subNodeName.compareTo("MergeMultiple") == 0)
				{
					rule.setMergeMultiple(true);
					continue;
				}
				if (subNodeName.compareTo("JavaClassRule") == 0)
				{
					rule.setJavaClassRule(utils.extractStrictXMLAttrValue(subNode,"class"));
					continue;
				}
				System.out.println("Unsupported node type "+subNodeName);
			}
			this.addRule(rule);
		}
	}
	
	/**
	 * Add rule to collection
	 * 
	 * @param rule
	 *           Rule to be added
	 */
	private void addRule (MergingRule rule)
	{
		this.getList().add(rule);
		this.map.put(rule.getProperty(),rule);
	}

	/**
	 * @return Returns the list.
	 */
	public ArrayList getList ()
	{
		return this.list;
	}	
}
