package logrus.ontmodel.randomization;

import logrus.agents.datasources.DataProviderID;
import logrus.ontmodel.Prefixes;
import logrus.ontmodel.Quad;
import logrus.ontmodel.QuadsList;
import logrus.ontmodel.restaurant.AlcoholCodes;
import logrus.ontmodel.restaurant.CuisineTypes;
import logrus.ontmodel.restaurant.FeatureCodes;

public class RestaurantDataRandomizer implements IDataRandomizer
{
	// probability of changing cuisine-related statements
	private static final double ProbabilityCuisine = 0.1;

	// probability of changing features-related statements
	private static final double ProbabilityFeatures = 0.1;

	// probability of changing capacity-related statements
	private static final double ProbabilityCapacity = 0.1;

	// probability of changing alcohol-related statements
	private static final double ProbabilityAlcohol = 1.0;

	public RestaurantDataRandomizer ()
	{
		super();
	}

	/**
	 * Toss data provider
	 * 
	 * @return
	 */
	private String tossProvider ()
	{
		if (Math.random() < 0.5)
		{
			return DataProviderID.CHEFMOZ;
		}
		else
		{
			return DataProviderID.RESTCOM;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see logrus.randomization.IDataRandomizer#randomize()
	 */
	public void randomize (QuadsList input, String providerID)
	{
		if (input == null)
		{
			return;
		}

		String id = input.getObjectURI();
		Quad quad = null;

		// cuisine
		if (this.toss(ProbabilityCuisine))
		{
			String cuisine = Prefixes.BaseRestaurantURI + "#"
					+ CuisineTypes.getInstance().tossCuisine();
			quad = new Quad(id,Prefixes.RES_CUISINE,cuisine,providerID);
			if (!input.containsStatement(quad))
			{
				input.add(quad);
			}
		}

		// features
		if (this.toss(ProbabilityFeatures))
		{
			String feature = FeatureCodes.getInstance().tossFeature();
			quad = new Quad(id,Prefixes.RES_FEAT,feature,providerID);
			if (!input.containsStatement(quad))
			{
				input.add(quad);
			}
		}

		if (providerID.compareTo(DataProviderID.CHEFMOZ) == 0)
		{
			// alcohol
			if (this.toss(ProbabilityAlcohol))
			{
				String alcoholCode = AlcoholCodes.getInstance().tossAlcohol();
				quad = new Quad(id,Prefixes.RES_ALC,alcoholCode,providerID);
				Quad pattern = new Quad(id,Prefixes.RES_ALC,null,null);
				input.removeStatementsMatchingPattern(pattern);
				input.add(quad);
			}
		}
		else if (providerID.compareTo(DataProviderID.RESTCOM) == 0)
		{
			// capacity (seats)
			if (this.toss(ProbabilityCapacity))
			{
				int capacity = (int)(Math.round(Math.random() * 100)) + 10;
				quad = new Quad(id,Prefixes.RES_SEATS,"" + capacity,providerID);
				Quad pattern = new Quad(id,Prefixes.RES_SEATS,null,null);
				input.removeStatementsMatchingPattern(pattern);
				input.add(quad);
			}
		}
	}

	/**
	 * Toss value between 0.0 and 1.0 chcek if it's lower than
	 * 
	 * @param threshold
	 *           Threshold of maximal tossed value that will be considered as success
	 * @return True if number lower than <code>threshold</code> has been tossed, otherwise returns
	 *         false
	 */
	private boolean toss (double threshold)
	{
		return Math.random() < threshold;
	}
}
