package logrus.ontmodel.restaurant.tools;


public class AddressMatcher implements IPropertyMatcher
{
	private Boolean useAdvanced;
	
	public AddressMatcher (Boolean useAdvanced)
	{
		super();
		this.useAdvanced = useAdvanced;
	}

	public boolean match (String strAddr1, String strAddr2)
	{
		// first check exact matching of both strings
		if (strAddr1.compareTo(strAddr2) == 0)
		{
			// perfect match :-)
			return true;
		}
		else
		{
			// ok, it still may be the same restaurant, let's play with some heuristics
			// first: extract street name and string describing house number
			String[] parts1 = this.extractNN(strAddr1);
			String strStreet1 = parts1[0];
			String strHouse1 = parts1[1];
			String[] parts2 = this.extractNN(strAddr2);
			String strStreet2 = parts2[0];
			String strHouse2 = parts2[1];

			// now start guessing
			if (strStreet1 == null || strStreet2 == null)
			{
				// no street name extracted so it's nothing more to do
				return false;
			}

			if (strStreet1.compareTo(strStreet2) == 0
					&& strHouse1.compareTo(strHouse2) == 0)
			{
				// match :-)
				return true;
			}
			else if (strStreet1.indexOf(strStreet2) > 0
					|| strStreet2.indexOf(strStreet1) > 0)
			{
				// street names match a bit...
				// so assume that it's rather the same objet...
				if (strHouse1 != null
						&& strHouse2 != null
						&& (strHouse1.indexOf(strHouse2) > 0 || strHouse2
								.indexOf(strHouse1) > 0))
				{
					// both house numbers extracted and are simillar so assume that it's the same
					// restaurant
					return true;
				}
				else
				{
					// both house numbers extracted and are not simillar although street names, so assume
					// that there are tow different restaurants
					return false;
				}
			}
			else
			{
				// it's rather not the same restaurant
				return false;
			}
		}		
	}
	
	private String[] extractNN (String streetAddress)
	{
		String[] parts = new String[2];
		String strHouseNumber = null, strStreetName = null;

		String strAddr = streetAddress;

		if (strAddr.startsWith("Ul. "))
		{
			strAddr = strAddr.replaceFirst("Ul. ","");
		}
		if (strAddr.startsWith("Al. "))
		{
			strAddr = strAddr.replaceFirst("Al. ","");
		}
		if (strAddr.startsWith("Plac "))
		{
			strAddr = strAddr.replaceFirst("Plac ","");
		}
		strAddr = strAddr.trim();

		// ok, now we should have only street name and house number
		String[] auxParts = strAddr.split(" ");

		// find substring that starts with a digit and assume that it is house number
		for (int i = 0; i < auxParts.length; i++)
		{
			if (auxParts[i].length() > 0)
			{
				// ok this starts with a digit so guess it's the house number
				if (Character.isDigit(auxParts[i].charAt(0)))
				{
					strHouseNumber = auxParts[i];
					break;
				}
			}
		}

		// if house number found then remove it from string and clean multiple spaces
		if (strHouseNumber != null)
		{
			strAddr = strAddr.replaceFirst(strHouseNumber,"");
			strAddr = strAddr.replaceAll("   "," ");
			strAddr = strAddr.replaceAll("  "," ");

		}
		// so the rest of string is probably street name
		strStreetName = strAddr;

		parts[0] = strStreetName;
		parts[1] = strHouseNumber;

		return parts;
	}	
}
