package logrus.ontmodel.restaurant.tools;

import java.util.Iterator;

import logrus.agents.datasources.DataProviderID;
import logrus.ontmodel.Prefixes;
import logrus.ontmodel.Quad;
import logrus.ontmodel.merging.ICustomMerging;
import logrus.ontmodel.merging.MergingReport;
import logrus.ontmodel.merging.MergingRule;
import logrus.ontmodel.restaurant.ParkingCodes;
import logrus.tools.queue.Queue;

/**
 * 
 */
public class ParkingMergingRule implements ICustomMerging
{

	/**
	 * 
	 */
	public ParkingMergingRule (Queue stmts, MergingReport report,
			MergingRule rule)
	{
		super();
	}

	public void merge (Queue stmts, MergingReport report, MergingRule rule)
	{
		if (stmts.count() == 0)
		{
			return;
		}
		Queue cleaned = new Queue();
		int iCM = 0, iRC = 0;
		String uriRes = ((Quad)stmts.getHead().getKey())
				.getSubject();
		// check if we have information NoLocalParking from CM and SomeParking from RC
		Quad noParkCM = new Quad(uriRes,
				Prefixes.RES_PARK,ParkingCodes.NoLocalParking,
				DataProviderID.CHEFMOZ);
		Quad someParkRC = new Quad(uriRes,
				Prefixes.RES_PARK,ParkingCodes.SomeParking,DataProviderID.RESTCOM);
		boolean hasNoParkCM = false, hasSomeParkRC = false;
		for (Iterator iter = stmts.iterator(); iter.hasNext();)
		{
			Quad stmt = (Quad)iter.next();
			if (stmt.isEqual(noParkCM))
				hasNoParkCM = true;
			if (stmt.isEqual(someParkRC))
				hasSomeParkRC = true;
			if (stmt.getProvider().compareTo(DataProviderID.CHEFMOZ)==0)
				++iCM;
			if (stmt.getProvider().compareTo(DataProviderID.RESTCOM)==0)
				++iRC;			
		}
		if (hasNoParkCM && hasSomeParkRC)
		{
			cleaned.inject(someParkRC);
			report.addStatements(cleaned);
			return;
		}
		
		String prov = null;
		if (iCM>0)
		{
			// add only statements form CM			
			prov = DataProviderID.CHEFMOZ;
		}
		else
		{
			// add only statements form RC
			prov = DataProviderID.RESTCOM;
		}

		for (Iterator iter = stmts.iterator(); iter.hasNext();)
		{
			Quad elem =(Quad)iter.next();
			if(elem.getProvider().compareTo(prov)==0)
			{
				cleaned.inject(elem);
			}
		}		
		report.addStatements(cleaned);
	}
}
