package logrus.ontmodel.restaurant.tools;

import java.util.Iterator;

import logrus.agents.datasources.DataProviderID;
import logrus.ontmodel.Prefixes;
import logrus.ontmodel.Quad;
import logrus.ontmodel.QuadsList;
import logrus.ontmodel.merging.ICustomMerging;
import logrus.ontmodel.merging.MergingReport;
import logrus.ontmodel.merging.MergingRule;
import logrus.ontmodel.restaurant.AcceptedPaymentMethods;
import logrus.tools.queue.Queue;

/**
 * 
 */
public class PaymentMergingRule implements ICustomMerging
{

	/**
	 * 
	 */
	public PaymentMergingRule (Queue stmts, MergingReport report,
			MergingRule rule)
	{
		super();
	}

	public void merge (Queue stmts, MergingReport report, MergingRule rule)
	{
		// init working queues and auxiliary variables
		Queue localQueue = new Queue();
		localQueue.addAll(stmts);
		Queue cleaned = new Queue();
		String uriRes = null;
		if (localQueue.count() > 0)
		{
			// get URI of restaurant object
			uriRes = ((Quad)localQueue.getHead().getKey()).getSubject();
		}
		else
		{
			// no statements to merge
			return;
		}

		// filter cash quads
		QuadsList quadsCash = QuadsList.filterStatements(localQueue,null,
				Prefixes.RES_ACC,AcceptedPaymentMethods.Cash,null);

		// if contains cash statememt from CM include it, otherwise include statement from RC
		if (quadsCash.containsStatement(uriRes,Prefixes.RES_ACC,
				AcceptedPaymentMethods.Cash,DataProviderID.CHEFMOZ))
		{
			cleaned.addUnique(new Quad(uriRes,Prefixes.RES_ACC,
					AcceptedPaymentMethods.Cash,DataProviderID.CHEFMOZ));
		}
		else
		{
			cleaned.addUnique(new Quad(uriRes,Prefixes.RES_ACC,
					AcceptedPaymentMethods.Cash,DataProviderID.RESTCOM));
		}
		// remove cash statements from local working queue
		quadsCash.removeSelfFromQueue(localQueue);

		// add named credit cards
		String[] cardNames = { AcceptedPaymentMethods.AmericanExpressCard,
				AcceptedPaymentMethods.DinersClubCard,
				AcceptedPaymentMethods.DiscoverCard,
				AcceptedPaymentMethods.EnRouteCard,
				AcceptedPaymentMethods.JapanCreditBureauCard,
				AcceptedPaymentMethods.MasterCardEuroCard,
				AcceptedPaymentMethods.MaestroCard, AcceptedPaymentMethods.Polcard,
				AcceptedPaymentMethods.VisaCard };
		boolean hasNamedCards = false;
		for (int i = 0; i < cardNames.length; i++)
		{
			// filter named card quads
			String cardURI = cardNames[i];
			QuadsList cardQuads = QuadsList.filterStatements(localQueue,null,
					Prefixes.RES_ACC,cardURI,null);

			// add quads
			if (cardQuads.containsStatement(uriRes,Prefixes.RES_ACC,cardURI,
					DataProviderID.RESTCOM))
			{
				cleaned.addUnique(new Quad(uriRes,Prefixes.RES_ACC,cardURI,
						DataProviderID.RESTCOM));
				hasNamedCards = true;
			}
			else if (cardQuads.containsStatement(uriRes,Prefixes.RES_ACC,cardURI,
					DataProviderID.CHEFMOZ))
			{
				cleaned.addUnique(new Quad(uriRes,Prefixes.RES_ACC,cardURI,
						DataProviderID.CHEFMOZ));
				hasNamedCards = true;
			}

			// remove quads from local queue
			cardQuads.removeSelfFromQueue(localQueue);
		}

		boolean hasGeneralCard = false;

		// process general card-related tags
		String cardURI = AcceptedPaymentMethods.BankDebitCards;
		QuadsList cardQuads = QuadsList.filterStatements(localQueue,null,
				Prefixes.RES_ACC,cardURI,DataProviderID.CHEFMOZ);
		if (cardQuads.count() > 0)
		{
			cleaned.add(new Quad(uriRes,Prefixes.RES_ACC,cardURI,
					DataProviderID.CHEFMOZ));
			hasGeneralCard = true;
			cardQuads.removeSelfFromQueue(localQueue);
		}

		cardURI = AcceptedPaymentMethods.BankCard;
		cardQuads = QuadsList.filterStatements(localQueue,null,Prefixes.RES_ACC,
				cardURI,null);
		if (cardQuads.count() > 0)
		{
			if (cardQuads.containsStatement(uriRes,Prefixes.RES_ACC,cardURI,
					DataProviderID.RESTCOM))
			{
				cleaned.add(new Quad(uriRes,Prefixes.RES_ACC,cardURI,
						DataProviderID.RESTCOM));
				hasGeneralCard = true;
			}
			else if (cardQuads.containsStatement(uriRes,Prefixes.RES_ACC,cardURI,
					DataProviderID.CHEFMOZ))
			{
				cleaned.add(new Quad(uriRes,Prefixes.RES_ACC,cardURI,
						DataProviderID.CHEFMOZ));
				hasGeneralCard = true;
			}
			cardQuads.removeSelfFromQueue(localQueue);
		}

		cardURI = AcceptedPaymentMethods.CreditCard;
		cardQuads = QuadsList.filterStatements(localQueue,null,Prefixes.RES_ACC,
				cardURI,DataProviderID.CHEFMOZ);
		if (cardQuads.count() > 0)
		{
			cleaned.add(new Quad(uriRes,Prefixes.RES_ACC,cardURI,
					DataProviderID.CHEFMOZ));
			hasGeneralCard = true;
			cardQuads.removeSelfFromQueue(localQueue);
		}

		// OK, now check if there is NoBankCard tag from restauracje.com and no other information
		// about bank cards
		cardURI = AcceptedPaymentMethods.NoBankCard;
		cardQuads = QuadsList.filterStatements(localQueue,null,Prefixes.RES_ACC,
				cardURI,DataProviderID.CHEFMOZ);
		if (cardQuads.count() > 0 && !hasNamedCards && !hasGeneralCard)
		{
			cleaned.add(new Quad(uriRes,Prefixes.RES_ACC,cardURI,
					DataProviderID.RESTCOM));
			cardQuads.removeSelfFromQueue(localQueue);
		}

		// OK, now just add the rest of statement from local queue to the list of accepted statements
		for (Iterator iter = localQueue.iterator(); iter.hasNext();)
		{
			cleaned.add((Quad)iter.next());
		}

		// and now add all the quads considered as valid to the result
		QuadsList result = report.getResult();
		for (Iterator iter = cleaned.iterator(); iter.hasNext();)
		{
			result.add((Quad)iter.next());
		}
	}
}
