package logrus.ontmodel.restaurant.tools;

import logrus.ontmodel.QuadsList;
import logrus.tools.Utils;

public class RestaurantCleaner
{
	private static RestaurantCleaner instance;

	private RestaurantCleaner ()
	{
		super();
	}

	public static RestaurantCleaner getInstance ()
	{
		if (RestaurantCleaner.instance == null)
		{
			RestaurantCleaner.instance = new RestaurantCleaner();
		}
		return RestaurantCleaner.instance;
	}

	public void cleanInfo (QuadsList quads)
	{
		// remove quads with at least one value (subject, predicate, object or provider ID) equal null
		quads.cleanQuadsWithNull();
		RestaurantTag tag = (RestaurantTag)new RestaurantTag().getTag(quads);

		// correct city names to Polish
		this.correctCityName(tag);

		// try to clean restaurant name
		this.cleanRestaurantName(tag);
		
		// try to clean street address
		this.cleanStreetAddress(tag);
	}
	
	/**
	 * 
	 * @param tag
	 */
	public void cleanStreetAddress (RestaurantTag tag)
	{
		if (tag.hasStreetAddress())
		{
			Utils utils = Utils.getInstance();
			String strAddr = tag.getStreetAddress();
			if (strAddr.startsWith("null") && strAddr.length()>4)
			{
				strAddr = strAddr.substring(4);
			}
			strAddr = utils.strToXxxx(strAddr);
			// replace non-standard street/square tags
			if (strAddr.startsWith("Ul "))
			{
				strAddr = strAddr.replaceFirst("Ul ","Ul. ");
			}
			else if (strAddr.startsWith("Al "))
			{
				strAddr = strAddr.replaceFirst("Al ","Al. ");
			}
			else  if (strAddr.startsWith("Pl. "))
			{
				strAddr = strAddr.replaceFirst("Pl. ","Plac ");
			}
			else  if (strAddr.startsWith("Pl "))
			{
				strAddr = strAddr.replaceFirst("Pl ","Plac ");
			}			
			
			// clean multiple spaces
			tag.setStreetAddressValue(utils.strCleanMultipleSpaces(strAddr));
		}
	}

	/**
	 * Try to clean restaurant name by:
	 * <ul>
	 * <li>Removing chatacters like '-', '_', '\t', '\n', '\r', '\"'</li>
	 * <li>Replacing multiple spaces by single one</li>
	 * <li>Chaging all strings to format with first letter in upper case and rest in lower case</li>
	 * <li>Removing strings like Restauracja, Bar, Pizzeria, Kawiarnia, Lodziarnia, etc.</li>
	 * </ul>
	 * 
	 * @param tag
	 *           Restaurant tag to be cleaned
	 */
	public void cleanRestaurantName (RestaurantTag tag)
	{
		if (tag.hasName())
		{
			Utils utils = Utils.getInstance();
			String name = tag.getName();

			// change unnecessary characters to spaces
			name = utils.strCharsToSpaces(name, new char[]{'-','_','\t','\n','\r','\\'});
			
			// clean multiple spaces
			name = utils.strCleanMultipleSpaces(name);
			
			// switch all substrings to Xxxxx format
			name = utils.strToXxxx(name);

			// clean object type included in name
			name = name.replaceAll("Zajazd & Restauracja","");
			name = name.replaceAll("Indyjska Restauracja","");
			name = name.replaceAll("Tajska Restauracja","");
			name = name.replaceAll("Restauracja Rosyjska","");
			name = name.replaceAll("Restauracja Orbis","");
			name = name.replaceAll("Restauracja Gromada","");
			name = name.replaceAll("Restauracja","");
			name = name.replaceAll("Restaurant","");

			name = name.replaceAll("Bar Saatkowy","");
			name = name.replaceAll("Coctail Bar","");
			name = name.replaceAll("Cafe Bar","");
			name = name.replaceAll("Grill Wiejski","");
			name = name.replaceAll("Sandwich&grill Bar","");
			name = name.replaceAll("Bar","");

			name = name.replaceAll("Pizzeria","");
			name = name.replaceAll("Kawiarnia","");
			name = name.replaceAll("Lodziarnia","");
			name = name.replaceAll("Gospoda","");
			name = name.replaceAll("Karczma","");
			name = name.replaceAll("Zajazd","");
			name = name.replaceAll("Motel","");
			name = name.replaceAll("Pub","");
			name = name.replaceAll("Delikatesy","");
			name = name.replaceAll("Hotel","");
			// name = name.replaceAll("","");
			// name = name.replaceAll("","");
			tag.setNameValue(name.trim());
		}
	}

	/**
	 * Change English city names to Polish
	 * 
	 * @param tag
	 *           Restaurant tag to be cleaned
	 */
	public void correctCityName (RestaurantTag tag)
	{
		// correct city names to Polish
		if (tag.hasCity())
		{
			if (tag.getCity().compareTo("Warsaw") == 0)
			{
				tag.setCityValue("Warszawa");
			}
			else if (tag.getCity().compareTo("Poznan") == 0)
			{
				tag.setCityValue("Pozna");
			}
			else if (tag.getCity().compareTo("Cracow") == 0)
			{
				tag.setCityValue("Krakw");
			}
		}
	}
}
