package logrus.ontmodel.restaurant.tools;

import java.util.ArrayList;

import logrus.ontmodel.IObjectInfoSearcherByTag;
import logrus.ontmodel.IObjectInfoTag;
import logrus.ontmodel.IOntModelWrapper;
import logrus.ontmodel.Prefixes;
import logrus.ontmodel.QuadsList;
import logrus.ontmodel.exceptions.OntPropertyNotFoundException;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

/**
 * Searches JENA ontological object for restaurant matching specified name and location information
 */
public class RestaurantSearcherByTag implements IObjectInfoSearcherByTag
{
	/**
	 * 
	 */
	public RestaurantSearcherByTag ()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see logrus.ontmodel.IObjectInfoSearcherByTag#findObject(logrus.ontmodel.IOntModelWrapper,
	 *      logrus.ontmodel.IObjectInfoTag)
	 */
	public QuadsList findObject (IOntModelWrapper wrapper, IObjectInfoTag tag) throws OntPropertyNotFoundException
	{
		RestaurantTag resTag = (RestaurantTag)tag;
		OntModel oModel = wrapper.getModel();
		Property nameProperty = null;
		Property cityProperty = null;
		ArrayList auxList = new ArrayList();
		RestaurantComparer comparer = new RestaurantComparer();

		// get restaurant name property from model
		nameProperty = wrapper.getProperty(Prefixes.RES_NAME);
		cityProperty = wrapper.getProperty(Prefixes.LOC_CITY);

		// first locate restaurants with name as in tag
		for (StmtIterator iterWithName = oModel.listStatements(null,nameProperty,
				resTag.getName()); iterWithName.hasNext();)
		{
			Statement stmt = iterWithName.nextStatement();

			// check if this restaurant is located in the city we want
			if (oModel.listStatements(stmt.getSubject(),cityProperty,
					resTag.getCity()).hasNext())
			{
				// ok name and city matched so time for full comparison
				QuadsList fromDBinfo = wrapper.getObjectInfo(stmt.getSubject()
						.getURI());
				RestaurantTag fromDBTag = (RestaurantTag)new RestaurantTag()
						.getTag(fromDBinfo);
				if (comparer.isRepresentingSame(fromDBTag,resTag))
				{
					// hooray! match!
					return fromDBinfo;
				}
			}
		}

		return null;
	}
}
