package logrus.ontmodel.validation;

import java.util.ArrayList;
import java.util.Hashtable;

import logrus.tools.Utils;
import logrus.tools.XMLAnalysisException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class ValidationRulesCollection
{
	private ArrayList list;

	private Hashtable map;

	/**
	 * Create validation rules collection from file
	 * 
	 * @param filepath
	 *           Flie from which rules should be loaded
	 * @throws XMLAnalysisException
	 */
	public ValidationRulesCollection (String filepath)
			throws XMLAnalysisException
	{
		super();
		this.list = new ArrayList();
		this.map = new Hashtable();
		this.loadRules(filepath);
	}

	private void loadRules (String filepath) throws XMLAnalysisException
	{
		Utils utils = Utils.getInstance();
		Document doc = utils.loadXMLDocument(filepath);
		Node root = doc.getFirstChild();
		if (root.getNodeName() != "ValidationRules")
		{
			throw new XMLAnalysisException("Incorrect root element");
		}
		NodeList nodes = root.getChildNodes();
		for (int i = 0; i < nodes.getLength(); i++)
		{
			Node elem = nodes.item(i);
			if (elem.getNodeName().compareTo("#text") == 0
					|| elem.getNodeName().compareTo("#comment") == 0)
			{
				continue;
			}
			if (elem.getNodeName() != "Rule")
			{
				throw new XMLAnalysisException("Unexpected node "
						+ elem.getNodeName() + ", expected was <Rule>");
			}
			// get property name
			String strProperty = utils.extractStrictXMLAttrValue(elem,"property");

			Node child = utils.getNamedChild(elem,"Content");
			if (child == null)
			{
				throw new XMLAnalysisException("No <Content> specifed for "
						+ strProperty);
			}

			String strSignificance = utils.extractStrictXMLAttrValue(child,
					"significance");
			String strPlurality = utils.extractStrictXMLAttrValue(child,
					"plurality");
			this.addRule(new ValidationRule(strProperty,strSignificance,
					strPlurality));
		}
	}

	/**
	 * Get validation rule for property <code>uriProperty</code>
	 * 
	 * @param uriProperty
	 * @return
	 */
	public ValidationRule getRule (String uriProperty)
	{
		return (ValidationRule)this.map.get(uriProperty);
	}

	/**
	 * Add rule to collection
	 * 
	 * @param rule
	 *           Rule to be added
	 */
	private void addRule (ValidationRule rule)
	{
		this.getList().add(rule);
		this.map.put(rule.getProperty(),rule);
	}

	/**
	 * @return Returns the list.
	 */
	public ArrayList getList ()
	{
		return this.list;
	}
}
