package logrus.tools;

import java.security.SecureRandom;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import logrus.tools.IdentifierGenerationFailedException;

/**
 * Unique identifier generator (unique with high-probability).
 * This class is a singleton.
 */
public class IdentifierGenerator
{
	// singleton instance
	private static IdentifierGenerator instance;
	
	/**
	 * Default constructor 
	 */
	private IdentifierGenerator ()
	{
		super();
	}

	/**
	 * Get singleton instance
	 * @return Instance of IdentifierGenerator
	 */
	public static IdentifierGenerator getInstance()
	{
		if (IdentifierGenerator.instance == null) 
		{
			IdentifierGenerator.instance = new IdentifierGenerator();
	   }
	   return IdentifierGenerator.instance;
	}
	
	/**
	 * Encodes(hex) byte array into the string. 
	 * @param arr Byte array to be translated
	 * @return String representing byte array
	 */
	private String hexEncode (byte[] arr)
	{
		StringBuffer result = new StringBuffer();
	   char[] digits = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
	   for (int i=0; i<arr.length; ++i)
	   {
	   	byte b = arr[i];
	      result.append(digits[(b&0xf0)>>4]);
	      result.append(digits[b&0x0f]);
	   }
	   return result.toString();
	}
	
	/**
	 * Generates "unique" identifier - string with 40 characters
	 * @return "unique" identifier
	 * @throws IdentifierGenerationFailedException
	 */
	public String GenerateID () throws IdentifierGenerationFailedException 
	{
		try
		{
			SecureRandom prng = SecureRandom.getInstance("SHA1PRNG");
			String randomNum = new Integer(prng.nextInt()).toString();
			MessageDigest sha = MessageDigest.getInstance("SHA-1");
			return IdentifierGenerator.getInstance().hexEncode
				(sha.digest(randomNum.getBytes()));
		}
		catch(NoSuchAlgorithmException exc)
		{
			throw new IdentifierGenerationFailedException(exc);
		}
	}	
}
