package logrus.tools;

import java.util.HashMap;

public class WeekDaysMappings
{
	// hashmap holding weekday mappings
	private HashMap map;
	
	// singleton instance
	private static WeekDaysMappings instance = null; 
	
	/**
	 * Private constructor - singleton design pattern
	 *
	 */
	private WeekDaysMappings ()
	{
		super();
		this.map = new HashMap();
		this.map.put("Mon",new Integer(0));
		this.map.put("Tue",new Integer(1));
		this.map.put("Wed",new Integer(2));
		this.map.put("Thu",new Integer(3));
		this.map.put("Fri",new Integer(4));
		this.map.put("Sat",new Integer(5));
		this.map.put("Sun",new Integer(6));
		
		this.map.put("Monday",new Integer(0));
		this.map.put("Tuesday",new Integer(1));
		this.map.put("Wednesday",new Integer(2));
		this.map.put("Thursday",new Integer(3));
		this.map.put("Friday",new Integer(4));
		this.map.put("Saturday",new Integer(5));
		this.map.put("Sunday",new Integer(6));
		
		this.map.put("Pon",new Integer(0));
		this.map.put("Wto",new Integer(1));
		this.map.put("ro",new Integer(2));
		this.map.put("Czw",new Integer(3));
		this.map.put("Pi",new Integer(4));
		this.map.put("Sob",new Integer(5));
		this.map.put("Nie",new Integer(6));			
	}
	
	/**
	 * Translate weekday name to index
	 * @param name Weekday name
	 * @return Index (0-Monday, 1-Tuesday, ... , 6-Sunday)
	 */
	public int getIndex (String name)
	{
		Object obj = this.map.get(name);
		if (obj!=null)
		{
			return ((Integer)obj).intValue();
		}
		else
		{
			return -1;
		}
	}
	
	/**
	 * Get singleton instance of WeekDaysMappings 
	 * @return singleton instance of WeekDaysMappings
	 */
	public static WeekDaysMappings getInstance ()
	{
		if (WeekDaysMappings.instance==null)
		{
			WeekDaysMappings.instance = new WeekDaysMappings();
		}
		return WeekDaysMappings.instance;
	}
}
