package logrus.tools.logging;

import jade.util.Logger;

import java.io.IOException;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import jade.core.Agent;
import logrus.tools.IdentifierGenerationFailedException;


/**
 * Class wrapping logging mechanism 
 */
public class LoggerWrapper
{
	// singleton instance
	private static LoggerWrapper instance;
	
	// Name of folder where log files are stored 
	private String logsFolder;
	
	
	/**
	 * Default constructor
	 * @param logsFolder Name of folder where log files are stored 
	 * @throws IdentifierGenerationFailedException
	 */
	private LoggerWrapper (String logsFolder)
	{
		super();
		this.setLogsFolder(logsFolder);
	}
	
	/**
	 * Get singleton instance
	 * @return Instance of IdentifierGenerator
	 */
	public static LoggerWrapper getInstance (String logsFolder)
	{
		if (LoggerWrapper.instance == null) 
		{
			LoggerWrapper.instance = new LoggerWrapper(logsFolder);
	   }
	   return LoggerWrapper.instance;
	}
	
	/**
	 * Get singleton instance
	 * @return Instance of IdentifierGenerator
	 */
	public static LoggerWrapper getInstance ()
	{
		if (LoggerWrapper.instance == null) 
		{
			LoggerWrapper.instance = 
				new LoggerWrapper("logs/");
	   }
	   return LoggerWrapper.instance;
	}	
	
	/**
	 * Initializes logging for specified agent
	 * @param agent
	 * @throws InitializeLoggerFailedException
	 */
	public Logger InitializeLogger (Agent agent) 
		throws InitializeLoggerFailedException
	{
	   try
		{
	   	String agentClassname = agent.getClass().getName(); 
	   	Handler fHandler = new FileHandler
				(this.logsFolder+agentClassname+".log",false);
			Logger logger = jade.util.Logger.getMyLogger(agentClassname);
			logger.addHandler(fHandler);
			return logger;
		}
		catch (SecurityException e)
		{
			throw new InitializeLoggerFailedException(e);
		}
		catch (IOException e)
		{
			throw new InitializeLoggerFailedException(e);
		}		
	}

	/**
	 * @param logsFolder The logsFolder to set.
	 */
	public void setLogsFolder (String logsFolder)
	{
		this.logsFolder = logsFolder;
	}

	/**
	 * @return Returns the logsFolder.
	 */
	public String getLogsFolder ()
	{
		return this.logsFolder;
	}
}
