package logrus.tools.queue;

/**
 * Implementation of queue node
 */
public class QueueNode
{
	// object stored in node
	private Object key;

	// next node in queue
	private QueueNode next;

	// previous node in queue
	private QueueNode prev;

	/**
	 * Create new node
	 * 
	 * @param key
	 *           Content to be stored in this node
	 * @param next
	 *           Node following this one in queue
	 * @param prev
	 *           Node prior to this one in queue
	 */
	public QueueNode (Object content, QueueNode next, QueueNode prev)
	{
		super();
		this.key = content;
		this.next = next;
		this.prev = prev;
	}

	/**
	 * Create new node
	 * 
	 * @param key
	 *           Content to be stored in this node
	 */
	public QueueNode (Object content)
	{
		super();
		this.key = content;
		this.next = null;
		this.prev = null;
	}

	/**
	 * Remove node from queue (set neighbours' pointers)
	 * 
	 * @return Content stored in this node
	 */
	public Object remove ()
	{
		if (this.prev != null)
		{
			this.prev.setNext(this.next);
		}
		if (this.next != null)
		{
			this.next.setPrev(this.prev);
		}
		return this.key;
	}

	/**
	 * @param next
	 *           The next to set.
	 */
	public void setNext (QueueNode next)
	{
		this.next = next;
	}

	/**
	 * @return Returns the next.
	 */
	public QueueNode getNext ()
	{
		return this.next;
	}

	/**
	 * @param key
	 *           The key to set.
	 */
	public void setKey (Object content)
	{
		this.key = content;
	}

	/**
	 * @return Returns the key.
	 */
	public Object getKey ()
	{
		return this.key;
	}

	/**
	 * @param prev
	 *           The prev to set.
	 */
	public void setPrev (QueueNode prev)
	{
		this.prev = prev;
	}

	/**
	 * @return Returns the prev.
	 */
	public QueueNode getPrev ()
	{
		return this.prev;
	}
}
