package logrus.tools.settings;

import java.io.Serializable;

/**
 * Class storing OntoStorage settings (bean)
 */
public class Settings implements Serializable
{
	// number of simultaneously running DBA used for reading data from model 
	// (must be equal at least 1) 
	private int numberReadDBA;
	
	// number of simultaneously running WA
	private int numberWA;
	
	// maximal number of simultaneously running indexing behaviours
	private int numberMaxIndexingBehaviours;
	
	// encoding used by XML files
	private String encoding;
	
	// initial term of validity of object description
	private long initialValidityTerm;
	
	// flag determinig if data should be randomly modified
	private boolean useDataRandomization;
	
	// name of computer where SQL database server is running
	private String dbServer;
	
	// provider for SQL database
	private String dbProvider;
	
	// schemata used for storing persistent RDF models
	private String dbSchemata;
	
	// login for accessing SQL database
	private String dbUser;
	
	// password for accessing SQL database
	private String dbPassword;
 
	
	/**
	 * Default constructor 
	 */
	public Settings ()
	{
		super();
	}

	/**
	 * @param numberReadDBA The numberReadDBA to set.
	 */
	public void setNumberReadDBA (int numberReadDBA)
	{
		this.numberReadDBA = numberReadDBA;
	}

	/**
	 * @return Returns the numberReadDBA.
	 */
	public int getNumberReadDBA ()
	{
		return this.numberReadDBA;
	}
	
	/**
	 * @param encoding The encoding to set.
	 */
	public void setEncoding (String encoding)
	{
		this.encoding = encoding;
	}

	/**
	 * @return Returns the encoding.
	 */
	public String getEncoding ()
	{
		return this.encoding;
	}

	/**
	 * @param numberMaxIndexingBehaviours The numberMaxIndexingBehaviours to set.
	 */
	public void setNumberMaxIndexingBehaviours (int numberMaxIndexingBehaviours)
	{
		this.numberMaxIndexingBehaviours = numberMaxIndexingBehaviours;
	}

	/**
	 * @return Returns the numberMaxIndexingBehaviours.
	 */
	public int getNumberMaxIndexingBehaviours ()
	{
		return this.numberMaxIndexingBehaviours;
	}

	/**
	 * @param numberWA The numberWA to set.
	 */
	public void setNumberWA (int numberWA)
	{
		this.numberWA = numberWA;
	}

	/**
	 * @return Returns the numberWA.
	 */
	public int getNumberWA ()
	{
		return this.numberWA;
	}

	/**
	 * @param initialValidityTerm The initialValidityTerm to set.
	 */
	public void setInitialValidityTerm (long initialValidityTerm)
	{
		this.initialValidityTerm = initialValidityTerm;
	}

	/**
	 * @return Returns the initialValidityTerm.
	 */
	public long getInitialValidityTerm ()
	{
		return this.initialValidityTerm;
	}

	/**
	 * @param useDataRandomization The useDataRandomization to set.
	 */
	public void setUseDataRandomization (boolean useDataRandomization)
	{
		this.useDataRandomization = useDataRandomization;
	}

	/**
	 * @return Returns the useDataRandomization.
	 */
	public boolean isUseDataRandomization ()
	{
		return this.useDataRandomization;
	}

	/**
	 * @param dbProvider The dbProvider to set.
	 */
	public void setDbProvider (String dbProvider)
	{
		this.dbProvider = dbProvider;
	}

	/**
	 * @return Returns the dbProvider.
	 */
	public String getDbProvider ()
	{
		return this.dbProvider;
	}

	/**
	 * @param dbSchemata The dbSchemata to set.
	 */
	public void setDbSchemata (String dbSchemata)
	{
		this.dbSchemata = dbSchemata;
	}

	/**
	 * @return Returns the dbSchemata.
	 */
	public String getDbSchemata ()
	{
		return this.dbSchemata;
	}

	/**
	 * @param dbUser The dbUser to set.
	 */
	public void setDbUser (String dbUser)
	{
		this.dbUser = dbUser;
	}

	/**
	 * @return Returns the dbUser.
	 */
	public String getDbUser ()
	{
		return this.dbUser;
	}

	/**
	 * @param dbPassword The dbPassword to set.
	 */
	public void setDbPassword (String dbPassword)
	{
		this.dbPassword = dbPassword;
	}

	/**
	 * @return Returns the dbPassword.
	 */
	public String getDbPassword ()
	{
		return this.dbPassword;
	}

	/**
	 * @param dbServer The dbServer to set.
	 */
	public void setDbServer (String dbServer)
	{
		this.dbServer = dbServer;
	}

	/**
	 * @return Returns the dbServer.
	 */
	public String getDbServer ()
	{
		return this.dbServer;
	}	
}
