package logrus.tools.web.visitors;

import logrus.tools.Utils;

import org.w3c.dom.Node;

/**
 */
public class NthTagSearcherNodeVisitor extends NodeVisitor
{
	// found node is assigned here
	private Node result;
	
	// tag for which we are searching for
	private String searchedTag;
	
	// tag causing end of search
	private String limitingTag;
	
	// flag determinig if end tag has been found instead of desired one
	private boolean foundLimitingTag;
	
	// Number of occurence to be found
	private int nth;
	
	// Occurencies found so far
	private int foundSoFar;
	
	/**
	 * @param searchedTag Tag for which we are searching for
	 * @param endTag Tag causing end of search
	 * @param n Number of occurence to be found
	 */
	public NthTagSearcherNodeVisitor (String searchedTag, String limitingTag, int n)
	{
		this.searchedTag = searchedTag;
		this.limitingTag = limitingTag;
		this.setFoundLimitingTag(false);
		this.nth = n;
		this.foundSoFar = 0;
	}
	
	/* (non-Javadoc)
	 * @see logrus.tools.web.NodeVisitor#processNode(org.w3c.dom.Node)
	 */
	public boolean processNode (Node node)
	{
		if (this.limitingTag!=null && Utils.strIsEqual(node.getNodeName(),this.limitingTag))
		{
			this.setFoundLimitingTag(true);
			return true;
		}
		if (Utils.strIsEqual(node.getNodeName(),this.searchedTag))
		{
			if (++this.foundSoFar==this.nth)
			{
				this.result = node;
				return true;	
			}
			else
			{
				return false;
			}
		}
		else
		{
			return false;	
		}
	}

	/* (non-Javadoc)
	 * @see logrus.tools.web.NodeVisitor#getResult()
	 */
	public Object getResult ()
	{
		return this.result;
	}

	/* (non-Javadoc)
	 * @see logrus.tools.web.NodeVisitor#setResult(java.lang.Object)
	 */
	public void setResult (Object result)
	{
		this.result = (Node) result;
	}

	/**
	 * @param foundLimitingTag The foundLimitingTag to set.
	 */
	public void setFoundLimitingTag (boolean foundLimitingTag)
	{
		this.foundLimitingTag = foundLimitingTag;
	}

	/**
	 * @return Returns the foundLimitingTag.
	 */
	public boolean isFoundLimitingTag ()
	{
		return this.foundLimitingTag;
	}
}
