package logrus.tools.web.visitors;

import logrus.tools.web.WebUtils;

import org.w3c.dom.Node;

/**
 */
public class TagSearcherNodeVisitor extends NodeVisitor
{
	// found node is assigned here
	private Node result;
	
	// tag for which we are searching for
	private String searchedTag;
	
	// tag causing end of search
	private String limitingTag;
	
	// flag determinig if end tag has been found instead of desired one
	private boolean foundLimitingTag;
	
	/**
	 * @param searchedTag Tag for which we are searching for
	 * @param endTag Tag causing end of search
	 */
	public TagSearcherNodeVisitor (String searchedTag, String limitingTag)
	{
		this.searchedTag = searchedTag;
		this.limitingTag = limitingTag;
		this.setFoundLimitingTag(false);
	}
	
	/* (non-Javadoc)
	 * @see logrus.tools.web.NodeVisitor#processNode(org.w3c.dom.Node)
	 */
	public boolean processNode (Node node)
	{
		if (this.limitingTag!=null && WebUtils.getInstance().compareNode(node,this.limitingTag))
		{
			this.setFoundLimitingTag(true);
			return true;
		}
		if (WebUtils.getInstance().compareNode(node,this.searchedTag))
		{
			this.result = node;
			return true;
		}
		else
		{
			return false;	
		}
	}

	/* (non-Javadoc)
	 * @see logrus.tools.web.NodeVisitor#getResult()
	 */
	public Object getResult ()
	{
		return this.result;
	}

	/* (non-Javadoc)
	 * @see logrus.tools.web.NodeVisitor#setResult(java.lang.Object)
	 */
	public void setResult (Object result)
	{
		this.result = (Node) result;
	}

	/**
	 * @param foundLimitingTag The foundLimitingTag to set.
	 */
	public void setFoundLimitingTag (boolean foundLimitingTag)
	{
		this.foundLimitingTag = foundLimitingTag;
	}

	/**
	 * @return Returns the foundLimitingTag.
	 */
	public boolean isFoundLimitingTag ()
	{
		return this.foundLimitingTag;
	}
}
