/**
 * 
 */
package tests;

import jade.util.leap.ArrayList;
import junit.framework.TestCase;
import logrus.agents.datasources.WebpageParseException;
import logrus.agents.datasources.DataProviderID;
import logrus.agents.datasources.chefmoz.ChefMozPropertiesExtractor;
import logrus.ontmodel.Prefixes;
import logrus.ontmodel.QuadsList;
import logrus.ontmodel.restaurant.AcceptedPaymentMethods;
import logrus.ontmodel.restaurant.AccessibilityCodes;
import logrus.ontmodel.restaurant.AlcoholCodes;
import logrus.ontmodel.restaurant.DressCodes;
import logrus.ontmodel.restaurant.FeatureCodes;
import logrus.ontmodel.restaurant.ParkingCodes;
import logrus.ontmodel.restaurant.ReservationCodes;
import logrus.ontmodel.restaurant.SmokingCodes;
import logrus.ontmodel.restaurant.tools.RestaurantTag;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.XMLAnalysisException;

/**
 * 
 */
public class ChefMozPropertiesExtractorTest extends TestCase
{
	protected void setUp () throws Exception
	{
		super.setUp();
	}

	private void checkHoursArr (String[] returned, String[] expected)
	{
		for (int i = 0; i < 7; i++)
		{
			assertEquals(returned[i],expected[i]);
		}
	}

	/*
	 * Test method for: logrus.ontmodel.ChefMozPropertiesExtractor.parseHours(ArrayList)
	 */
	public void testParseHours ()
	{
		ArrayList daily24h = new ArrayList();
		daily24h.add("24h");
		String[] daily24hRes = { "24h", "24h", "24h", "24h", "24h", "24h", "24h" };
		ArrayList daily = new ArrayList();
		daily.add("daily 11-23");

		String[] dailyRes = { "11-23", "11-23", "11-23", "11-23", "11-23",
				"11-23", "11-23" };
		ArrayList combined = new ArrayList();
		combined.add("Mon-Fri 10-21");
		combined.add("Sat 12-23");
		combined.add("Sun closed");
		String[] combinedRes = { "10-21", "10-21", "10-21", "10-21", "10-21",
				"12-23", "closed" };

		try
		{
			ChefMozPropertiesExtractor extr = new ChefMozPropertiesExtractor();
			this.checkHoursArr(extr.parseHours(daily),dailyRes);
			this.checkHoursArr(extr.parseHours(daily24h),daily24hRes);
			this.checkHoursArr(extr.parseHours(combined),combinedRes);
		}
		catch (IdentifierGenerationFailedException e)
		{
			fail(e.toString());
		}
	}

	public void testFast ()
	{
		String url = "http://chefmoz.org/Poland/MA/Warsaw/Antich_Caffe1040506216.html";
		String id = "da62b3949bba4477eed8ea00950218cd529bf2b7";
		ChefMozPropertiesExtractor extractor = new ChefMozPropertiesExtractor(id);
		try
		{
			QuadsList elems = extractor.extract(url);
			if (elems == null)
			{
				fail("No info extracted");
			}
			else
			{
				System.out.println(elems);
			}
		}
		catch (WebpageParseException exc)
		{
			fail(exc.toString());
		}
		catch (XMLAnalysisException exc)
		{
			fail(exc.toString());
		}
	}

	/*
	 * Test method for: logrus.ontmodel.ChefMozPropertiesExtractor.extract(String)
	 */
	public void testExtract ()
	{
		String uri = "http://chefmoz.org/Poland/MA/Warsaw/Antich_Caffe1040506216.html";
		try
		{
			String id = "da62b3949bba4477eed8ea00950218cd529bf2b7";
			ChefMozPropertiesExtractor extractor = new ChefMozPropertiesExtractor(
					id);
			QuadsList elems = extractor.extract(uri);
			String uriRes = Prefixes.BaseRestaurantURI + "/" + id;
			String uriProv = DataProviderID.CHEFMOZ;
			String[] uriCuisine = {
					Prefixes.BaseRestaurantURI + "#ItalianCuisine",
					Prefixes.BaseRestaurantURI + "#CafeCuisine",
					Prefixes.BaseRestaurantURI + "#CoffeeShopCuisine",
					Prefixes.BaseRestaurantURI + "#DessertCuisine",
					Prefixes.BaseRestaurantURI + "#IceCreamCuisine" }; // check cuisine
			for (int i = 0; i < uriCuisine.length; i++)
			{
				if (!elems.containsStatement(uriRes,Prefixes.RES_CUISINE,
						uriCuisine[i],uriProv))
				{
					fail("Cuisine " + uriCuisine[i] + " not parsed correctly");
				}
			}
			// check name and location
			if (!elems.containsStatement(uriRes,Prefixes.RES_NAME,"Antich Caffe",
					uriProv))
			{
				fail("Incorret restaurant name");
			}
			if (!elems.containsStatement(uriRes,Prefixes.LOC_COUNTRY,"Poland",
					uriProv))
			{
				fail("Incorret country name");
			}
			if (!elems.containsStatement(uriRes,Prefixes.LOC_REGION,"MA",uriProv))
			{
				fail("Incorret region name");
			}
			if (!elems
					.containsStatement(uriRes,Prefixes.LOC_CITY,"Warszawa",uriProv))
			{
				fail("Incorret city name");
			}
			// check address
			if (!elems.containsStatement(uriRes,Prefixes.LOC_NEIGHBORHOOD,
					"Ursynw",uriProv))
			{
				fail("Incorret neighborhood name");
			}
			if (!elems.containsStatement(uriRes,Prefixes.LOC_STREET_ADDR,
					"Ul. Bronikowskiego 1 (wwozowa, Os. Kabaty)",uriProv))
			{
				fail("Incorret street address");
			}
			if (!elems.containsStatement(uriRes,Prefixes.LOC_ZIP,"02-796",uriProv))
			{
				fail("Incorret zip code");
			}
			if (!elems.containsStatement(uriRes,Prefixes.LOC_PHONE,
					"+48 (22) 648 43 71",uriProv))
			{
				fail("Incorret phone number");
			}

			// check web information
			if (!elems.containsStatement(uriRes,Prefixes.RES_WWW,
					"http://www.antichcaffe.getin.pl",uriProv))
			{
				fail("Incorret web page address");
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_EMAIL,
					"poczta@antichcaffe.getin.pl",uriProv))
			{
				fail("Incorret mail address");
			}

			// check price information
			if (!elems.containsStatement(uriRes,Prefixes.RES_PRI_BREAK,
					Prefixes.MON_FPC_INEXP,uriProv))
			{
				fail("Incorret breakfast price range");
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_PRI_LUNCH,
					Prefixes.MON_FPC_MODER,uriProv))
			{
				fail("Incorret lunch price range");
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_PRI_DINNER,
					Prefixes.MON_FPC_MODER,uriProv))
			{
				fail("Incorret dinner price range");
			}

			// check features information
			if (!elems.containsStatement(uriRes,Prefixes.RES_FEAT,
					FeatureCodes.Takeout,uriProv))
			{
				fail("No feature " + FeatureCodes.Takeout);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_FEAT,
					FeatureCodes.KidFriendly,uriProv))
			{
				fail("No feature " + FeatureCodes.KidFriendly);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_FEAT,
					FeatureCodes.LargeGroupsOk,uriProv))
			{
				fail("No feature " + FeatureCodes.LargeGroupsOk);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_FEAT,
					FeatureCodes.OutdoorPatioDining,uriProv))
			{
				fail("No feature " + FeatureCodes.OutdoorPatioDining);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_FEAT,
					FeatureCodes.NiceView,uriProv))
			{
				fail("No feature " + FeatureCodes.NiceView);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_FEAT,
					FeatureCodes.Television,uriProv))
			{
				fail("No feature " + FeatureCodes.Television);
			}

			// check accepted payment methods information
			if (!elems.containsStatement(uriRes,Prefixes.RES_ACC,
					AcceptedPaymentMethods.Cash,uriProv))
			{
				fail("No accepted payment method " + AcceptedPaymentMethods.Cash);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_ACC,
					AcceptedPaymentMethods.MasterCardEuroCard,uriProv))
			{
				fail("No accepted payment method "
						+ AcceptedPaymentMethods.MasterCardEuroCard);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_ACC,
					AcceptedPaymentMethods.DinersClubCard,uriProv))
			{
				fail("No accepted payment method "
						+ AcceptedPaymentMethods.DinersClubCard);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_ACC,
					AcceptedPaymentMethods.BankDebitCards,uriProv))
			{
				fail("No accepted payment method "
						+ AcceptedPaymentMethods.BankDebitCards);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_ACC,
					AcceptedPaymentMethods.AmericanExpressCard,uriProv))
			{
				fail("No accepted payment method "
						+ AcceptedPaymentMethods.AmericanExpressCard);
			}
			if (!elems.containsStatement(uriRes,Prefixes.RES_ACC,
					AcceptedPaymentMethods.VisaCard,uriProv))
			{
				fail("No accepted payment method "
						+ AcceptedPaymentMethods.VisaCard);
			}

			// check smoking policy information
			if (!elems.containsStatement(uriRes,Prefixes.RES_SMK,
					SmokingCodes.PermittedSmoking,uriProv))
			{
				fail("Smoking code is not " + SmokingCodes.PermittedSmoking);
			}

			// check dress policy information
			if (!elems.containsStatement(uriRes,Prefixes.RES_DRESS,
					DressCodes.DressyCasual,uriProv))
			{
				fail("Dress code is not " + DressCodes.DressyCasual);
			}

			// check alcohol service information
			if (!elems.containsStatement(uriRes,Prefixes.RES_ALC,
					AlcoholCodes.FullBar,uriProv))
			{
				fail("Alcohol service is not " + AlcoholCodes.FullBar);
			}

			// check reservation information
			if (!elems.containsStatement(uriRes,Prefixes.RES_RESERV,
					ReservationCodes.LargeGroupsOnlyReservations,uriProv))
			{
				fail("Reservations policy is not "
						+ ReservationCodes.LargeGroupsOnlyReservations);
			}

			// check parking information
			if (!elems.containsStatement(uriRes,Prefixes.RES_PARK,
					ParkingCodes.StreetParking,uriProv))
			{
				fail("Parking is not " + ParkingCodes.StreetParking);
			}

			// check handicapped access information
			if (!elems.containsStatement(uriRes,Prefixes.RES_HANDACC,
					AccessibilityCodes.PartiallyAccessible,uriProv))
			{
				fail("Handicapped Access level is not "
						+ AccessibilityCodes.PartiallyAccessible);
			}

			// check parsed open hours information
			if (!elems.containsStatement(uriRes,Prefixes.RES_PAR_HR,
					"8-22|8-22|8-22|8-22|8-22|8-22|10-20",uriProv))
			{
				fail("Parsed open hours should be 8-22|8-22|8-22|8-22|8-22|8-22|10-20");
			}

			// now test restaurant's tag extraction
			RestaurantTag tag = (RestaurantTag)new RestaurantTag().getTag(elems);
			if (tag.getCity().compareTo("Warszawa") != 0)
			{
				fail("Incorrect tag field: city - expected was Warsaw but found "
						+ tag.getCity());
			}
			if (tag.getName().compareTo("Antich Caffe") != 0)
			{
				fail("Incorrect tag field: name - expected was Antich but found "
						+ tag.getName());
			}
			if (tag.getStreetAddress().compareTo(
					"Ul. Bronikowskiego 1 (wwozowa, Os. Kabaty)") != 0)
			{
				fail("Incorrect tag field: name - expected was ul. Bronikowskiego 1 (Wwozowa, os. Kabaty) but found "
						+ tag.getStreetAddress());
			}
			if (tag.getZipCode().compareTo("02-796") != 0)
			{
				fail("Incorrect tag field: name - expected was 02-796 but found "
						+ tag.getZipCode());
			}
		}
		catch (WebpageParseException exc)
		{
			fail(exc.toString());
		}
		catch (XMLAnalysisException exc)
		{
			fail(exc.toString());
		}
	}
}
