package tests;

import java.io.IOException;

import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;

import junit.framework.TestCase;
import logrus.agents.datasources.WebpageParseException;
import logrus.agents.datasources.DataProviderID;
import logrus.agents.datasources.chefmoz.ChefMozPropertiesExtractor;
import logrus.agents.datasources.restauracjecom.RestComPropertiesExtractor;
import logrus.ontmodel.ModelOperation;
import logrus.ontmodel.IOntModelWrapper;
import logrus.ontmodel.OntModelWrapperFactory;
import logrus.ontmodel.OntUtils;
import logrus.ontmodel.Prefixes;
import logrus.ontmodel.Quad;
import logrus.ontmodel.QuadsList;
import logrus.ontmodel.exceptions.MatchingException;
import logrus.ontmodel.exceptions.OntPropertyNotFoundException;
import logrus.ontmodel.exceptions.ResourceNotFoundException;
import logrus.ontmodel.restaurant.AcceptedPaymentMethods;
import logrus.ontmodel.restaurant.AccessibilityCodes;
import logrus.ontmodel.restaurant.ParkingCodes;
import logrus.ontmodel.restaurant.ReservationCodes;
import logrus.ontmodel.restaurant.tools.RestaurantComparer;
import logrus.tools.IdentifierGenerationFailedException;
import logrus.tools.IdentifierGenerator;
import logrus.tools.Utils;
import logrus.tools.XMLAnalysisException;
import logrus.tools.queue.Queue;

public class OntModelWrapperImplTest extends TestCase
{
	private IOntModelWrapper wrapper;

	protected void setUp () throws Exception
	{
		super.setUp();
		System.out.println("Loading model...");
		String[] files = Prefixes.ONTOLOGY_FILES;
		String[] paths = new String[files.length];
		for (int i = 0; i < files.length; i++)
		{
			paths[i] = "file:///F:/Mag-Soft/_Project_/OntoStorage/schemas/"
					+ files[i];
		}
		this.wrapper = OntModelWrapperFactory.getInstance().createModelFromFiles(
				paths,OntModelSpec.RDFS_MEM);
		System.out.println("Model with " + this.wrapper.countStatements()
				+ " statements loaded");
	}

	protected void nsetUp () throws Exception
	{
		super.setUp();
		System.out.println("Loading model...");
		String path = "file:///" + Utils.getInstance().getCWD()
				+ "databases\\default.rdf";
		this.wrapper = OntModelWrapperFactory.getInstance().createModelFromFile(
				path,OntModelSpec.RDFS_MEM);
		System.out.println("Model with " + this.wrapper.countStatements()
				+ " statements loaded");
	}

	public void testDump ()
	{
		try
		{
			String filename = Utils.getInstance().getCWD()
					+ "databases\\default.rdf";
			this.wrapper.dumpToFile(filename);
			System.out.println("Model saved to " + filename);
		}
		catch (IOException e)
		{
			fail(e.toString());
		}
	}

	/*
	 * Test method for: 'logrus.ontmodel.OntModelWrapperImpl.addResource(String)
	 * 'logrus.ontmodel.OntModelWrapperImpl.addDescriptiveStatement(String,String,String,String,String)',
	 * 'logrus.ontmodel.OntModelWrapperImpl.getProvider(String,String)' (tests merged to avoid
	 * time-consuming multiple model creation)
	 */
	public void testBasicOperations ()
	{
		// Test
		// for logrus.ontmodel.OntModelWrapperImpl.addResource(String)
		String uriRes = Prefixes.BaseRestaurantURI + "testRestaurant";
		Resource resBas = this.wrapper.addResource(uriRes);
		assertNotNull(resBas);
		assertEquals(resBas.getURI(),uriRes);

		// Test for
		// logrus.ontmodel.OntModelWrapperImpl.addDescriptiveStatement(String,String,String,String,String)
		String id = null;
		try
		{
			id = IdentifierGenerator.getInstance().GenerateID();
		}
		catch (IdentifierGenerationFailedException e)
		{
			e.printStackTrace();
		}
		String resURI = Prefixes.BaseRestaurantURI + id;
		String propURI = Prefixes.BaseRestaurantURI + "#email";

		String objVal = "logrus@os.pl";
		String provCodeURI = DataProviderID.CHEFMOZ;
		Resource res = null;
		try
		{
			this.wrapper
					.addDescriptiveStatement(provCodeURI,resURI,propURI,objVal);
			res = this.wrapper.getResource(resURI);
			Statement newStmt = res.getProperty(this.wrapper.getProperty(propURI));
			assertEquals(newStmt.getObject().toString(),objVal);
		}
		catch (OntPropertyNotFoundException e)
		{
			e.printStackTrace();
		}

		// Test for
		// logrus.ontmodel.OntModelWrapperImpl.getProvider(String,String)
		try
		{
			String acquiredProvider = this.wrapper.getProvider(resURI,propURI);
			assertEquals(acquiredProvider,provCodeURI);
		}
		catch (ResourceNotFoundException e)
		{
			fail("Exception ResourceNotFoundException thrown by IOntModelWrapper::getProvider."
					+ " Deatils: " + e.getMessage());
		}
		catch (OntPropertyNotFoundException e)
		{
			fail("Exception OntPropertyNotFoundException thrown by IOntModelWrapper::getProvider."
					+ " Deatils: " + e.getMessage());
		}
	}

	public void testModerateOperations ()
	{
		try
		{
			String uriRes = Prefixes.BaseRestaurantURI + "/"
					+ "da62b3949bba4477eed8ea00950218cd529bf2b7";
			QuadsList info = new QuadsList();
			info.add(uriRes,Prefixes.LOC_COUNTRY,"Poland",DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.LOC_CITY,"Warsaw",DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.LOC_STREET_ADDR,"ul. Marszakowska 1",
					DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.LOC_ZIP,"00-001",DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.RES_NAME,"U Zdzicha",DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.RES_ACC,AcceptedPaymentMethods.Cash,
					DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.RES_HANDACC,
					AccessibilityCodes.PartiallyAccessible,DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.RES_CUISINE,Prefixes.BaseRestaurantURI
					+ "#PolishCuisine",DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.RES_CUISINE,Prefixes.BaseRestaurantURI
					+ "#BalkanCuisine",DataProviderID.CHEFMOZ);
			info.add(uriRes,Prefixes.RES_PARK,ParkingCodes.SomeParking,
					DataProviderID.CHEFMOZ);
			QuadsList infoSmall = (QuadsList)info.clone();

			Quad additionalStmt = new Quad(uriRes,Prefixes.RES_RESERV,
					ReservationCodes.RequiredReservations,DataProviderID.CHEFMOZ);
			info.add(additionalStmt);

			this.wrapper.addObjectInfo(info);
			QuadsList infoRet = this.wrapper.getObjectInfo(uriRes);

			assertTrue(info.isEqual(infoRet));

			ModelOperation removeOper = new ModelOperation(ModelOperation.REMOVE,
					(Quad)additionalStmt.clone());
			this.wrapper.performOperation(removeOper);
			infoRet = this.wrapper.getObjectInfo(uriRes);

			System.out.println("InfoSmall:\n" + infoSmall);
			System.out.println("InfoRet:\n" + infoRet);

			assertTrue(infoSmall.isEqual(infoRet));

			// check simple add operation
			OntUtils ontUtils = OntUtils.getInstance();
			Queue operations = ontUtils.calculateOperations(infoSmall,info);
			assertEquals(operations.count(),1);
			ModelOperation oper = (ModelOperation)operations.getHead().getKey();
			assertEquals(oper.getAction(),ModelOperation.ADD);
			assertTrue(oper.getData().isEqual(additionalStmt));

			// check simple remove operation
			operations = ontUtils.calculateOperations(info,infoSmall);
			assertEquals(operations.count(),1);
			oper = (ModelOperation)operations.getHead().getKey();
			assertEquals(oper.getAction(),ModelOperation.REMOVE);
			assertTrue(oper.getData().isEqual(additionalStmt));

			// check simple replace operation (remove+add)
			Quad stmt1 = new Quad(uriRes,Prefixes.RES_CUISINE,
					Prefixes.BaseRestaurantURI + "#PolishCuisine",
					DataProviderID.CHEFMOZ);
			Quad stmt2 = new Quad(uriRes,Prefixes.RES_CUISINE,
					Prefixes.BaseRestaurantURI + "#FrenchCuisine",
					DataProviderID.CHEFMOZ);
			QuadsList list1 = new QuadsList();
			list1.add(stmt1);
			QuadsList list2 = new QuadsList();
			list2.add(stmt2);
			operations = ontUtils.calculateOperations(list1,list2);
			assertEquals(operations.count(),2);
			oper = (ModelOperation)operations.pop();
			assertEquals(oper.getAction(),ModelOperation.REMOVE);
			assertTrue(oper.getData().isEqual((Quad)stmt1.clone()));
			oper = (ModelOperation)operations.pop();
			assertEquals(oper.getAction(),ModelOperation.ADD);
			assertTrue(oper.getData().isEqual((Quad)stmt2.clone()));

			// check replacement of single statement when multiple properties
			Quad stmt3 = new Quad(uriRes,Prefixes.RES_CUISINE,
					Prefixes.BaseRestaurantURI + "#BalkanCuisine",
					DataProviderID.CHEFMOZ);
			Quad stmt4 = new Quad(uriRes,Prefixes.RES_CUISINE,
					Prefixes.BaseRestaurantURI + "#GreekCuisine",
					DataProviderID.CHEFMOZ);
			list1 = new QuadsList();
			list1.add(stmt3);
			list1.add(stmt1);
			list1.add(stmt4);
			list2 = new QuadsList();
			list2.add(stmt4);
			list2.add(stmt3);
			list2.add(stmt2);
			operations = ontUtils.calculateOperations(list1,list2);
			assertEquals(operations.count(),2);
			oper = (ModelOperation)operations.pop();
			assertEquals(oper.getAction(),ModelOperation.REMOVE);
			assertTrue(oper.getData().isEqual((Quad)stmt1.clone()));
			oper = (ModelOperation)operations.pop();
			assertEquals(oper.getAction(),ModelOperation.ADD);
			assertTrue(oper.getData().isEqual((Quad)stmt2.clone()));
		}
		catch (OntPropertyNotFoundException e)
		{
			fail(e.toString());
		}
	}

	public void testAdvancedOperations ()
	{
		try
		{
			String url1 = "http://chefmoz.org/Poland/DS/K%C5%82odzko/Nad_Wodospadem%2C_Restauracja1011063713.html";
			String url2 = "http://www.restauracje.com/proste_szczegoly.php3?id=2034";
			String url3 = "http://chefmoz.org/Poland/LD/Pabianice/Vega%2C_Pizzeria1045057604.html";
			ChefMozPropertiesExtractor exCM;
			RestComPropertiesExtractor exRC;
			RestaurantComparer comparer = new RestaurantComparer();

			exCM = new ChefMozPropertiesExtractor();
			exCM.extract(url1);
			QuadsList obj1 = exCM.getResult();

			exRC = new RestComPropertiesExtractor();
			exRC.extract(url2);
			QuadsList obj2 = exRC.getResult();

			exCM = new ChefMozPropertiesExtractor();
			exCM.extract(url3);
			QuadsList obj3 = exCM.getResult();

			// add object info
			this.wrapper.addObjectInfo(obj1);

			QuadsList result1 = this.wrapper.findMatchingObject(obj1);
			if (!obj1.isEqual(result1))
			{
				fail("Expected was match");
			}

			QuadsList result2 = this.wrapper.findMatchingObject(obj2);
			if (!obj1.isEqual(result2))
			{
				fail("Expected was match");
			}

			QuadsList result3 = this.wrapper.findMatchingObject(obj3);
			assertNull(result3);
		}
		catch (IdentifierGenerationFailedException exc)
		{
			fail(exc.toString());
		}
		catch (WebpageParseException exc)
		{
			fail(exc.toString());
		}
		catch (OntPropertyNotFoundException exc)
		{
			fail(exc.toString());
		}
		catch (MatchingException exc)
		{
			fail(exc.toString());
		}
		catch (XMLAnalysisException exc)
		{
			fail(exc.toString());
		}
	}
}
