package tests;

import java.util.Iterator;

import junit.framework.TestCase;
import logrus.tools.queue.Queue;
import logrus.tools.queue.QueueNode;

public class QueueTests extends TestCase
{
	private String str1 = "1";

	private String str2 = "2";

	private String str3 = "3";

	private String str4 = "4";

	private String str5 = "5";

	private String str6 = "6";

	private String str7 = "7";

	private String str8 = "8";

	private String str9 = "9";

	public void testPush ()
	{
		Queue q = new Queue();

		// check push on empty queue
		q.push(this.str1);
		assertEquals(q.count(),1);
		QueueNode h = q.getHead();
		QueueNode t = q.getTail();
		assertNotNull(h);
		assertNotNull(t);
		assertEquals(h,t);
		assertEquals(h.getPrev(),null);
		assertEquals(h.getNext(),null);
		assertEquals(t.getPrev(),null);
		assertEquals(t.getNext(),null);

		// check push on not empty queue
		q.push(this.str2);
		assertEquals(q.count(),2);
		h = q.getHead();
		t = q.getTail();
		assertNotNull(h);
		assertNotNull(t);
		assertNotSame(h,t);
		assertEquals(h.getPrev(),null);
		assertEquals(h.getNext(),t);
		assertEquals(t.getPrev(),h);
		assertEquals(t.getNext(),null);
	}

	public void testInject ()
	{
		Queue q = new Queue();

		// check inject on empty queue
		q.inject(this.str1);
		assertEquals(q.count(),1);
		QueueNode h = q.getHead();
		QueueNode t = q.getTail();
		assertNotNull(h);
		assertNotNull(t);
		assertEquals(h,t);
		assertEquals(h.getPrev(),null);
		assertEquals(h.getNext(),null);
		assertEquals(t.getPrev(),null);
		assertEquals(t.getNext(),null);

		// check inject on not empty queue
		q.inject(this.str2);
		assertEquals(q.count(),2);
		h = q.getHead();
		t = q.getTail();
		assertNotNull(h);
		assertNotNull(t);
		assertNotSame(h,t);
		assertEquals(h.getPrev(),null);
		assertEquals(h.getNext(),t);
		assertEquals(t.getPrev(),h);
		assertEquals(t.getNext(),null);
	}

	public void testPop ()
	{
		Queue q = new Queue();
		QueueNode h, t;
		String str;

		// check pop on empty queue
		Object obj = q.pop();
		assertEquals(q.count(),0);
		assertNull(obj);

		// check pop on single element queue
		q.push(this.str1);
		obj = q.pop();
		assertEquals(q.count(),0);
		assertNotNull(obj);
		str = (String)obj;
		assertEquals(str,this.str1);
		assertEquals(q.getHead(),null);
		assertEquals(q.getTail(),null);

		// check pop on queue with two elements
		q = new Queue();
		q.push(this.str1);
		q.push(this.str2);
		obj = q.pop();
		assertEquals(q.count(),1);
		assertNotNull(obj);
		str = (String)obj;
		assertEquals(str,this.str2);
		h = q.getHead();
		t = q.getTail();
		assertNotNull(h);
		assertNotNull(t);
		assertEquals(h,t);
		assertEquals(h.getPrev(),null);
		assertEquals(h.getNext(),null);
		assertEquals((String)h.getKey(),this.str1);

		// check pop on queue with three elements
		q = new Queue();
		q.push(this.str1);
		q.push(this.str2);
		q.push(this.str3);
		obj = q.pop();
		assertEquals(q.count(),2);
		assertNotNull(obj);
		str = (String)obj;
		assertEquals(str,this.str3);
		h = q.getHead();
		t = q.getTail();
		assertNotNull(h);
		assertNotNull(t);
		assertNotSame(h,t);
		assertEquals(h.getPrev(),null);
		assertEquals(h.getNext(),t);
		assertEquals(t.getPrev(),h);
		assertEquals(t.getNext(),null);
	}

	public void testEject ()
	{
		Queue q = new Queue();
		QueueNode h, t;
		String str;

		// check eject on empty queue
		Object obj = q.eject();
		assertEquals(q.count(),0);
		assertNull(obj);

		// check eject on single element queue
		q.push(this.str1);
		obj = q.eject();
		assertEquals(q.count(),0);
		assertNotNull(obj);
		str = (String)obj;
		assertEquals(str,this.str1);
		assertEquals(q.getHead(),null);
		assertEquals(q.getTail(),null);

		// check pop on queue with two elements
		q = new Queue();
		q.push(this.str1);
		q.push(this.str2);
		obj = q.eject();
		assertEquals(q.count(),1);
		assertNotNull(obj);
		str = (String)obj;
		assertEquals(str,this.str1);
		h = q.getHead();
		t = q.getTail();
		assertNotNull(h);
		assertNotNull(t);
		assertEquals(h,t);
		assertEquals(h.getPrev(),null);
		assertEquals(h.getNext(),null);
		assertEquals((String)h.getKey(),this.str2);

		// check pop on queue with three elements
		q = new Queue();
		q.push(this.str1);
		q.push(this.str2);
		q.push(this.str3);
		obj = q.eject();
		assertEquals(q.count(),2);
		assertNotNull(obj);
		str = (String)obj;
		assertEquals(str,this.str1);
		h = q.getHead();
		t = q.getTail();
		assertNotNull(h);
		assertNotNull(t);
		assertNotSame(h,t);
		assertEquals(h.getPrev(),null);
		assertEquals(h.getNext(),t);
		assertEquals(t.getPrev(),h);
		assertEquals(t.getNext(),null);
	}

	public void testToArray ()
	{
		// check on empty queue
		Queue q = new Queue();
		Object[] arr = q.toArray();
		assertNull(arr);

		// check on not empty queue
		q = new Queue();
		q.inject(this.str3);
		q.inject(this.str4);
		q.inject(this.str5);
		q.push(this.str2);
		q.push(this.str1);
		arr = q.toArray();
		assertEquals(arr[0],this.str1);
		assertEquals(arr[1],this.str2);
		assertEquals(arr[2],this.str3);
		assertEquals(arr[3],this.str4);
		assertEquals(arr[4],this.str5);
	}

	public void testMappings ()
	{
		Queue q = new Queue();
		assertFalse(q.contains(this.str1));
		q.push(this.str1);
		assertTrue(q.contains(this.str1));
		q.pop();
		assertFalse(q.contains(this.str1));
	}

	public void testUnique ()
	{
		Queue q = new Queue();
		q.pushUnique(this.str1);
		assertTrue(q.contains(this.str1));
		assertEquals(q.count(),1);
		q.pushUnique(this.str1);
		assertEquals(q.count(),1);
		q.remove(this.str1);
		assertFalse(q.contains(this.str1));
		assertEquals(q.count(),0);
	}

	public void testIterator ()
	{
		Queue q = new Queue();
		q.inject(this.str3);
		q.inject(this.str4);
		q.inject(this.str5);
		q.push(this.str2);
		q.push(this.str1);
		String[] arr = { this.str1, this.str2, this.str3, this.str4, this.str5 };
		int i=0;
		for (Iterator iter = q.iterator(); iter.hasNext();)
		{
			assertEquals(arr[i++],(String)iter.next());
		}
		assertEquals(i,5);
	}
}
