package tests;

import junit.framework.TestCase;
import logrus.agents.datasources.WebpageParseException;
import logrus.agents.datasources.chefmoz.ChefMozPropertiesExtractor;
import logrus.agents.datasources.restauracjecom.RestComPropertiesExtractor;
import logrus.ontmodel.QuadsList;
import logrus.ontmodel.restaurant.tools.RestaurantCleaner;
import logrus.ontmodel.restaurant.tools.RestaurantComparer;
import logrus.ontmodel.restaurant.tools.RestaurantTag;
import logrus.tools.XMLAnalysisException;

public class RestaurantsComparerTest extends TestCase
{
	
	public void testSimilarCM ()
	{
		// pre-generated id
		String id = "da62b3949bba4477eed8ea00950218cd529bf2b7";
		RestaurantCleaner cleaner = RestaurantCleaner.getInstance();
		RestaurantComparer comparer = new RestaurantComparer();
		
		
		String url1 = "http://chefmoz.org/Poland/LD/Be%C5%82chat%C3%B3w/Fenix%2C_Pizzeria1044999355.html";
		String url2 = "http://chefmoz.org/Poland/LD/Be%C5%82chat%C3%B3w/Fenix%2C_Pizzeria1044999478.html";
		
		
		try
		{
			ChefMozPropertiesExtractor extractorCM1 = new ChefMozPropertiesExtractor(id);
			QuadsList list1 = extractorCM1.extract(url1);
			ChefMozPropertiesExtractor extractorCM2 = new ChefMozPropertiesExtractor(id);
			QuadsList list2 = extractorCM2.extract(url2);			
			
			cleaner.cleanInfo(list1);
			cleaner.cleanInfo(list2);
			
			RestaurantTag tag1 = (RestaurantTag)new RestaurantTag().getTag(list1);
			RestaurantTag tag2 = (RestaurantTag)new RestaurantTag().getTag(list2);
			
			// check if tags are full
			if (!tag1.isComplete() || !tag2.isComplete())
			{
				fail("One of tags is incomplete");
			}

			// check if restaurant comparer will consider them equal
			if (!comparer.isRepresentingSame(tag1,tag2))
			{
				System.out.println("Restaurants considered DIFFERENT:\n " + tag1 + " and\n "
						+ tag2);
			}
			else
			{
				System.out.println("Restaurants considered EQUAL:\n " + tag1 + " and\n "
						+ tag2);
			}			
		}
		catch (WebpageParseException exc)
		{
			fail(exc.toString());
		}
		catch (XMLAnalysisException exc)
		{
			fail(exc.toString());
		}
	}
	

	/*
	 * Test method for
	 * 'logrus.ontmodel.restaurant.tools.RestaurantComparer.isRepresentingSame(QuadsList, QuadsList)'
	 */
	public void testIsRepresentingSame ()
	{
		// pre-generated id
		String id = "da62b3949bba4477eed8ea00950218cd529bf2b7";
		RestaurantCleaner cleaner = RestaurantCleaner.getInstance();
		RestaurantComparer comparer = new RestaurantComparer();

		// pair 1: "Nad Wodospadem" w Kodzku
		String[] pair1 = new String[] {
				"http://chefmoz.org/Poland/DS/K%C5%82odzko/Nad_Wodospadem%2C_Restauracja1011063713.html",
				"http://www.restauracje.com/proste_szczegoly.php3?id=2034" };

		// pair 2: "A-Dong" w Gliwicach
		String[] pair2 = new String[] {
				"http://chefmoz.org/Poland/SL/Gliwice/A-Dong,_Restauracja_Orientalna1014252751.html",
				"http://www.restauracje.com/proste_szczegoly.php3?id=23" };
		
		// pair 3: "Adamek" w Supsku
		String[] pair3 = new String[] {
				"http://chefmoz.org/Poland/PM/S%C5%82upsk/Adamek,_Restauracja_1008639009.html",
				"http://www.restauracje.com/proste_szczegoly.php3?id=30" };
		
		// array of URI pairs
		Object[] pairs = new Object[] { pair1, pair2, pair3};

		for (int i = 0; i < pairs.length; i++)
		{
			ChefMozPropertiesExtractor extractorCM = new ChefMozPropertiesExtractor(
					id);
			RestComPropertiesExtractor extractorRC = new RestComPropertiesExtractor(
					id);

			// prepare URIs
			String[] pair = (String[])pairs[i];
			String uriCM = pair[0];
			String uriRC = pair[1];
			QuadsList quadsCM = null;
			QuadsList quadsRC = null;

			// extract
			try
			{
				extractorCM.extract(uriCM);
				extractorRC.extract(uriRC);
				quadsCM = extractorCM.getResult();
				quadsRC = extractorRC.getResult();
			}
			catch (WebpageParseException exc)
			{
				fail(exc.toString());
			}
			catch (XMLAnalysisException exc)
			{
				fail(exc.toString());
			}

			// clean
			cleaner.cleanInfo(quadsCM);
			cleaner.cleanInfo(quadsRC);

			// extract tags
			RestaurantTag tagCM = (RestaurantTag)new RestaurantTag().getTag(quadsCM);
			RestaurantTag tagRC = (RestaurantTag)new RestaurantTag().getTag(quadsRC);

			// on screen
			System.out.println("Restaurant tag from ChefMoz:");
			System.out.println(tagCM);
			System.out.println("Restaurant tag from restauracje.com:");
			System.out.println(tagRC);
			// System.out.println("");

			// check if tags are full
			if (!tagCM.isComplete() || !tagRC.isComplete())
			{
				fail("One of tags is incomplete");
			}

			// check if restaurant comparer will consider them equal
			if (!comparer.isRepresentingSame(quadsCM,quadsRC))
			{
				fail("Restaurants considered different:\n " + tagCM + " and\n "
						+ tagRC);
			}
			else
			{
				System.out.println("Comparison: \"EQUAL\"");
				System.out
						.println("---------------------------------------------------");
				System.out.println();
			}
		}
	}
}
